(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     26552,        733]*)
(*NotebookOutlinePosition[     27204,        756]*)
(*  CellTagsIndexPosition[     27160,        752]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["RSA Functions", "Title"],

Cell[TextData[
"This is a complete listing of all the functions that I have demonstrated. On \
the menu bar select Kernel \[Rule] Evaluation \[Rule] Evaluate Notebook to \
load all the functions into memory. A link is given at the start of each \
section to the notebook containing the commented code and a demonstration."], 
  "Text"],

Cell[CellGroupData[{

Cell["Prime and Strong Prime Generation", "Subtitle"],

Cell[TextData[ButtonBox["01_ranprime.nb",
  ButtonData:>{"01_ranpr.nb", None},
  Active->True,
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[BoxData[
    \(<< NumberTheory`PrimeQ`\)], "Input"],

Cell[BoxData[
    \(GetNextPrime[num_] := 
      Module[{n}, \n\t\tIf[EvenQ[num], n = num + 1, n = num]; \n\t\t
        While[\(! PrimeQ[n]\), n = n + 2]; \n\t\tn]\)], "Input"],

Cell[BoxData[
    \(GenerateRandomPrime[n_] := 
      GetNextPrime[Random[Integer, {10\^\(n - 1\), 10\^n}]]\)], "Input"],

Cell[BoxData[
    \(GetPmmStrong[ord_] := \n\t
      Module[{pdd, pd}, \n\t\t
        Print[\*"\"\<Generating \!\(\(p\^--\)\) Strong prime\>\""]; \n\t\t
        pdd = GenerateRandomPrime[ord - 2]; \n\t\tpd = Strengthen[pdd]; \n\t\t
        Strengthen[pd]]\)], "Input"],

Cell[BoxData[
    \(Strengthen[pd_, fun_:  Plus] := 
      Module[{p, ad}, \n\t\tad = 2; \n\t\tp = fun[ad*pd, 1]; \n\t\t
        While[\[Not] PrimeQ[p], \n\t\t\tad = ad + 2; \n\t\t\t\t
          p = fun[ad*pd, 1]]; \n\t\tp]\)], "Input"],

Cell[BoxData[
    \(GetStrong[ord_, prn_:  False] := \n\t
      Module[{pmm, pp, r, g, k, pm, a, p}, \n\t\t
        Print["\<Generating Strong Prime\>"]; \n\t\t
        pmm = GenerateRandomPrime[ord/2 - 1]; \n\t\t
        pp = GenerateRandomPrime[ord/2 - 1]; \n
        \t\t{g, {r, k}} = ExtendedGCD[pmm, pp]; \n\t\tr = \(-\((r)\)\); \n\t\t
        a = 1; pm = 4; p = 4; \n\t\t
        While[\[Not] \((PrimeQ[pm] \[And] PrimeQ[p])\), \n\t\t\t
          pm = \((2*a*pmm*pp)\) + \((2*r*pmm)\) + 1; \n\t\t\t
          p = \((2*pm)\) + 1; \n\t\t\t\(a++\)]; \n\t\t
        If[prn, \n\t\t\t
          Print[\*"\"\<\!\(\(P\^+\)\):\>\"", pp, 
            \*"\"\<,\!\(\(p\^--\)\):\>\"", pmm, \*"\"\<,\!\(\(p\^-\)\):\>\"", 
            pm]]; \n\t\tp]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Options[GenerateStrongPrime] = {Algorithm \[Rule] GetPmmStrong}\)], 
  "Input"],

Cell[BoxData[
    \({Algorithm \[Rule] GetPmmStrong}\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(GenerateStrongPrime[ord_] = GenerateStrongPrime[ord, {}]; \n
    GenerateStrongPrime[ord_, opts__] := 
      Module[{alg}, \n\t\t
        alg = \(Algorithm /. opts\) /. Options[GenerateStrongPrime]; \n\t\t
        alg[ord]]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Key Generation", "Subtitle"],

Cell[TextData[ButtonBox["02_keygen.nb",
  ButtonData:>{"02_keyge.nb", None},
  Active->True,
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[BoxData[
    \(GenerateKey[ord_, opts___] := 
      Module[{e, p, q, d, m}, \n\t\tp = GenerateStrongPrime[ord, opts]; \n\t\t
        q = GenerateStrongPrime[ord, opts]; \n\t\t
        m = \((p - 1)\)\ \((q - 1)\); \n\t\te = GenerateRandomPrime[ord]; \n
        \t\tWhile[GCD[e, m] > 1, \n\t\t\te = GenerateRandomPrime[ord]]; \n\t\t
        d = PowerMod[e, \(-1\), m]; \n\t{e, d, p\ q}]\)], "Input"],

Cell[BoxData[
    \(GenerateRSAKey[ord_, opts___] := 
      Module[{e, d, n}, \n\t\t{e, d, n} = GenerateKey[ord, opts]; \n\t\t
        RSAKey[e, d, n]]\)], "Input"],

Cell[BoxData[
    \(SpecifyExponent[ord_, e_, opts___] := \n\t
      Module[{p, q, d, n}, \n\t\tp = e + 1; q = e + 1; \n\t\t
        While[GCD[p - 1, e] > 1, \n\t\t\tp = GenerateStrongPrime[ord, opts]]; 
        \n\t\tWhile[GCD[q - 1, e] > 1, \n\t\t\t
          q = GenerateStrongPrime[ord, opts]]; \n\t\t
        d = PowerMod[e, \(-1\), \((p - 1)\) \((q - 1)\)]; \n\t\t
        RSAKey[e, d, p\ q]]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Key Management", "Subtitle"],

Cell[TextData[ButtonBox["03_keyman.nb",
  ButtonData:>{"03_keyma.nb", None},
  Active->True,
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(GetPair[key_] := 
      key /. RSAKey[e_, d_, n_] \[Rule] {Key[e, n], Key[d, n]}\)], "Input"],

Cell[BoxData[
    \(General::"spell1" \( : \ \) 
      "Possible spelling error: new symbol name \"\!\(Key\)\" is similar to \
existing symbol \"\!\(key\)\"."\)], "Message"]
}, Open  ]],

Cell[BoxData[
    \(GetPublic[RSAKey[e_, d_, n_]] := Key[e, n]; \n
    GetPublic[{Key[e_, n1_], Key[d_, n2_]}] := Key[e, n1]\)], "Input"],

Cell[BoxData[
    \(GetPrivate[RSAKey[e_, d_, n_]] := Key[d, n]; \n
    GetPrivate[{Key[e_, n1_], Key[d_, n2_]}] := Key[d, n2]\)], "Input"],

Cell[BoxData[
    \(GetModulus[Key[e_, n_]] := n; \nGetModulus[RSAKey[e_, d_, n_]] := n\)], 
  "Input"],

Cell[BoxData[
    \(\(GetExponent[Key[e_, n_]] := e; \)\)], "Input"],

Cell[BoxData[
    \(PublicExponent[RSAKey[e_, d_, n_]] := e\)], "Input"],

Cell[BoxData[
    \(PrivateExponent[RSAKey[e_, d_, n_]] := d\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Character Encoding", "Subtitle"],

Cell[TextData[ButtonBox["04_encoding.nb",
  ButtonData:>{"04_encod.nb", None},
  Active->True,
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[BoxData[
    \(\(ListToNumber[l_] = ListToNumber[l, 0]; \n
    ListToNumber[l_, num_] := \n\t
      If[Length[l] == 0, num, \n\t\t
        ListToNumber[Rest[l], \((num*256)\) + First[l]]]\t\t\)\)], "Input"],

Cell[BoxData[
    \(BreakUpMessage[cds_, blen_] := 
      Module[{c = cds}, \n\t\t (*\ Pad\ the\ sentence\ with\ spaces\ *) \n\t\t
        While[Mod[Length[c], blen] != 0, \n\t\t\tc = Join[c, {32}]]; \n\t\t
        Table[Take[c, {i, i + blen - 1}], {i, 1, Length[c] - 1, blen}]]\)], 
  "Input"],

Cell[BoxData[
    \(EncodeMessage[mes_, n_] := \n (*\ n\ is\ the\ public\ modulus\ *) \n
      Module[{blklen, bitlen, l, blocks}, \n\t\t
        bitlen = Length[IntegerDigits[n, 2]]; \n\t\t
        blklen = Quotient[bitlen, 8]; \n\t\tl = ToCharacterCode[mes]; \n\t\t
        blocks = BreakUpMessage[l, blklen]; \n\t\t
        Map[ListToNumber[#]&, blocks]]\)], "Input"],

Cell[BoxData[
    \(DecodeMessage[l_] := \n\t\t
      FromCharacterCode[\n\t\tFlatten[\n\t\t\tMap[IntegerDigits[#, 256]&, l]]]
        \)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Encryption Functions", "Subtitle"],

Cell[TextData[ButtonBox["05_encrypt.nb",
  ButtonData:>{"05_encry.nb", None},
  Active->True,
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(EncryptBlock[mess_, key : Key[e_, n_]] := PowerMod[mess, e, n]\)], 
  "Input"],

Cell[BoxData[
    \(General::"spell1" \( : \ \) 
      "Possible spelling error: new symbol name \"\!\(mess\)\" is similar to \
existing symbol \"\!\(mes\)\"."\)], "Message"]
}, Open  ]],

Cell[BoxData[
    \(EncryptMessage[mess_, Key[e_, n_]] := 
      Module[{enc1, enc2}, \n\t\tenc1 = EncodeMessage[mess, n]; \n\t\t
        enc2 = Map[\((EncryptBlock[#, Key[e, n]])\)&, enc1]]\)], "Input"],

Cell[BoxData[
    \(DecryptMessage[cipher_, Key[d_, n_]] := 
      Module[{dec}, \n\t\t
        dec = Map[\((EncryptBlock[#, Key[d, n]])\)&, cipher]; \n\t\t
        DecodeMessage[dec]]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Factoring Functions", "Subtitle"],

Cell[TextData[ButtonBox["07_factoring.nb",
  ButtonData:>{"07_facto.nb", None},
  Active->True,
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[CellGroupData[{

Cell["TrialDivision", "Subsubtitle"],

Cell[BoxData[
    \(TrialDivision[n_, lim_:  1000000] := 
      Module[{i, r, j, stop, facs}, \n\t\tr = n; j = 2; 
        stop = Min[Ceiling[\@n], lim]; facs = {}; \n\t\t
        Print["\<Trial Division\>"]; \n\t\t
        While[j < stop, \n\t\t\t
          For[i = j, i < j + 100000 \[And] i < stop, \(i++\), \n\t\t\t\t
            If[Mod[r, i] == 0, \n\t\t\t\t\tfacs = Insert[facs, i, 1]; \n
              \t\t\t\t\tr = r/i; \(i--\)]]; \n\t\t\tj = j + 100000; \n\t\t\t
          If[PrimeQ[r], \tfacs = Insert[facs, r, 1]; r = 1; Break[]]; \n\t\t\t
          Print["\<Passing \>", j]\n\t\t]; \n\t\t{r, facs}]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Fermat", "Subsubtitle"],

Cell[BoxData[
    \(\(FermatFactor[n_] := 
      \(Module[{st, u, v, r, a, b}, \n\t\tst = Ceiling[Sqrt[n]]; \n\t\t
        u = \((2*st)\) + 1; \n\t\tv = 1; \n\t\tr = st\^2 - n; \n\t\t
        While[r \[NotEqual] 0, \n\t\t\t
          If[r > 0, \n\t\t\t\t
            While[r > 0, \n\t\t\t\t\tr = r - v; \n\t\t\t\t\tv = v + 2]]; \n
          \t\t\tIf\ [r < 0, \n\t\t\t\tr = r + u; \n\t\t\t\tu = u + 2]]; \n\t\t
        a = \((u + v - 2)\)/2; \n\t\tb = \((u - v)\)/2; \n\t\t{a, b}]\n\)\t\t
    \)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Pollard p-1", "Subsubtitle"],

Cell[CellGroupData[{

Cell[BoxData[
    \(PollardP[n_, c_, max_] := 
      Module[{m, i, g, result}, \n\t\tm = c; \n\t\tresult = 1; \n\t\t
        For[i = 1, i \[LessEqual] max, i = i + 1, \n\t\t\t
          m = PowerMod[m, i, n]; \n\t\t\t
          If[Mod[i, 10] == 0, \n\t\t\t\tg = GCD[m - 1, n]; \n\t\t\t\t
            If[g > 1, result = g; i = max]]]; \n\t\tresult]\)], "Input"],

Cell[BoxData[
    \(General::"spell1" \( : \ \) 
      "Possible spelling error: new symbol name \"\!\(max\)\" is similar to \
existing symbol \"\!\(Max\)\"."\)], "Message"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Pollard Rho", "Subsubtitle"],

Cell[BoxData[
    \(PollRho[n_, c_, mx_] := \n\t
      Module[{x1, x2, rng, prod, trms, j}, \n\t\tx1 = 2; x2 = x1\^2 + c; 
        rng = 1; prod = 1; trms = 0; \n\t\t
        Catch[\n\t\t\t
          While[trms \[LessEqual] mx, \n\t\t\t\t
            For[j = 1, j < rng, \(j++\), \n\t\t\t\t\tx2 = Mod[x2\^2 + c, n]; 
              \n\t\t\t\t\tprod = Mod[prod*\((x1 - x2)\), n]; \n\t\t\t\t\t
              \(trms++\); \n\t\t\t\t\t
              If[Mod[trms, Min[10, trms]] == 0, \n\t\t\t\t\t\t
                prod = GCDCheck[n, prod]]]; \n
            \t\t\t\t{x1, x2, rng} = ResetVars[x1, x2, rng, c, n]]; \n\t\t\t
          "\<Enter a new c or longer range\>"]\n\t\t]; \n
    GCDCheck[n_, prd_] := 
      Module[{g}, \n\t\tg = GCD[n, prd]; \n\t\tIf[g > 1, Throw[g]]; \n\t\t1]; 
    \nResetVars[x1_, x2_, rng_, c_, n_] := 
      Module[{x2new, j, rngnew, x1new}, \n\t\tx1new = x2; rngnew = 2*rng; 
        x2new = x2; \n\t\t
        For[j = 1, j \[LessEqual] rng, \(j++\), \n\t\t\t
          x2new = Mod[x2new\^2 + c, n]]; \n\t\t{x1new, x2new, rngnew}]\)], 
  "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Elliptic Curve Method", "Subsubtitle"],

Cell[CellGroupData[{

Cell[BoxData[
    \(NextValues[x_, z_, k_, n_, a_, b_] := \n\t
      Module[{c, len, X1, X2, Z1, Z2, i, u1, u2, temp}, \n\t\t
        c = IntegerDigits[k, 2]; \n\t\tlen = Length[c]; \n\t\tX1 = x; Z1 = z; 
        \n\t\t (*\ Compute\ Loop\ *) \n\t\tX2 = XSub2I[x, z, a, b, n]; \n\t\t
        Z2 = ZSub2I[x, z, a, b, n]; \n\t\t
        For[i = len - 1, i \[GreaterEqual] 1, \(i--\), \n\t\t\t
          u1 = XSub2IPlus1[X1, Z1, X2, Z2, a, b, z, n]; \n\t\t\t
          u2 = ZSub2IPlus1[X1, Z1, X2, Z2, x, n]; \n\t\t\t
          If[c\[LeftDoubleBracket]i\[RightDoubleBracket] == 0, \n\t\t\t\t
            temp = XSub2I[X1, Z1, a, b, n]; \n\t\t\t\t
            Z1 = ZSub2I[X1, Z1, a, b, n]; \n\t\t\t\tX1 = temp; X2 = u1; 
            Z2 = u2, \n\t\t\t\t (*\ else\ do\ *) \n\t\t\t\t
            temp = XSub2I[X2, Z2, a, b, n]; \n\t\t\t\t
            Z2 = ZSub2I[X2, Z2, a, b, n]; \n\t\t\t\tX2 = temp; X1 = u1; 
            Z1 = u2]]; \n\t\t{X1, Z1}]\)], "Input"],

Cell[BoxData[
    \(General::"spell1" \( : \ \) 
      "Possible spelling error: new symbol name \"\!\(ZSub2I\)\" is similar \
to existing symbol \"\!\(XSub2I\)\"."\)], "Message"],

Cell[BoxData[
    \(General::"spell1" \( : \ \) 
      "Possible spelling error: new symbol name \"\!\(ZSub2IPlus1\)\" is \
similar to existing symbol \"\!\(XSub2IPlus1\)\"."\)], "Message"]
}, Open  ]],

Cell[BoxData[
    \(XSub2I[r_, s_, a_, b_, n_] := 
      Module[{term}, \n\t\tterm = Mod[r\^2 - a*s\^2, n]; \n\t\t
        Mod[term\^2 - 8*b*r*s\^3, n]]\)], "Input"],

Cell[BoxData[
    \(ZSub2I[r_, s_, a_, b_, n_] := 
      Module[{term}, \n\t\tterm = Mod[r\^3 + a*r*s\^2 + b*s\^3, n]; \n\t\t
        Mod[4*s*term, n]]\)], "Input"],

Cell[BoxData[
    \(XSub2IPlus1[r_, s_, u_, v_, a_, b_, z_, n_] := 
      Module[{term1, term2}, \n\t\tterm1 = Mod[r*u - a*s*v, n]; \n\t\t
        term2 = Mod[b*s*v*\((r*v + s*u)\), n]; \n\t\t
        Mod[z \((term1\^2 - 4*term2)\), n]]\)], "Input"],

Cell[BoxData[
    \(ZSub2IPlus1[r_, s_, u_, v_, x_, n_] := 
      Module[{term}, \n\t\tterm = Mod[u*s - r*v, n]; \n\t\tMod[x*term\^2, n]]
        \)], "Input"],

Cell[BoxData[
    \(GetEquation[EC[a_, b_]] := x\^3 + a\ x + b\  - y\^2 == 0\)], "Input"],

Cell[BoxData[
    \(ECM[n_, X_, y_, a_, max_] := 
      Module[{b, g, z, k, i, x}, \n\t\tx = X; \n\t\t
        b = Mod[y\^2 - x\^3 - a*x, n]; \n\t\t
        Print["\<Trying curve:\>", GetEquation[EC[a, b]]]; \n\t\t
        g = GCD[4*a\^3 + 27*b\^2, n]; \n\t\tIf[g \[NotEqual] 1, Return[g]]; \n
        \t\tz = 1; k = 2; \n\t\t
        While[k \[LessEqual] max, \n\t\t\t
          For[i = 1, i \[LessEqual] 10, \(i++\), \n
            \t\t\t\t{x, z} = NextValues[x, z, k, n, a, b]; \n\t\t\t\t
            \(k++\)]; \n\t\t\tg = GCD[z, n]; \n\t\t\t
          If[g \[NotEqual] 1, Break[]]]; \n\t\tg]\)], "Input"],

Cell[BoxData[
    \(FactorECM[n_, max_, curves_] := 
      Module[{x, y, a, g, i}, \n\t\tg = 1; \n\t\ti = 0; \n\t\t
        While[\((g \[Equal] 1 \[Or] g \[Equal] n)\) \[And] i < curves, \n
          \t\t\ta = Random[Integer, {1 - n, n - 1}]; \n\t\t\t
          x = Random[Integer, {1, n - 1}]; \n\t\t\t
          y = Random[Integer, {1, n - 1}]; \n\t\t\tg = ECM[n, x, y, a, max]; 
          \n\t\t\t\(i++\)\n\t\t]; \n\t\tg]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Combining Methods", "Subsubtitle"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CompleteFactor[n_] := 
      Module[{r, facs}, \n\t\t{r, facs} = TrialDivision[n, 150000]; \n\t\t
        CompleteFactor[r, facs]]; \n
    CompleteFactor[n_, factors_] := \n\t
      Module[{p, facs}, \n\t\tfacs = factors; \n\t\tp = Factorize[n]; \n\t\t
        If[p == 1 \[Or] p == n, Return[facs]]; \n\t\tPrint[p, "\<, \>", n/p]; 
        \n\t\tq = n/p; \n\t\t
        If[PrimeQ[q], facs = Insert[facs, q, 1], 
          facs = CompleteFactor[q, facs]]; \n\t\t
        If[PrimeQ[p], facs = Insert[facs, p, 1], 
          facs = CompleteFactor[p, facs]]; \n\t\tfacs]\)], "Input"],

Cell[BoxData[
    \(General::"spell1" \( : \ \) 
      "Possible spelling error: new symbol name \"\!\(Factorize\)\" is \
similar to existing symbol \"\!\(FactorSize\)\"."\)], "Message"]
}, Open  ]],

Cell[BoxData[
    \(Factorize[n_] := 
      Module[{p, i}, \n\t\t
        For[i = 1, i < 10, \(i++\), \n\t\t\tIf[n == 1, Return[1]]; \n\t\t\t
          Print["\<Pollard p-1 and Rho with n=\>", n, "\< and with c=\>", i]; 
          \n\t\t\tp = PollardP[n, i, 5000]; \n\t\t\t
          If[p > 1 \[And] p \[NotEqual] n, Return[p]]; \n\t\t\t
          p = PollRho[n, i, 5000]; \n\t\t\t
          If[p > 1 \[And] p \[NotEqual] n, Return[p]]; \n\t\t\t
          SeedRandom[101*i]; \n\t\t\tp = FactorECM[n, 300, 1]]; \n\t\tp]\)], 
  "Input"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Cycle Attack", "Subtitle"],

Cell[TextData[ButtonBox["06_cycle.nb",
  ButtonData:>{"06_cycle.nb", None},
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[BoxData[
    \(CycleAttackPhase1[cipherblock_, Key[e_, n_], max_:  5000] := \n\t
      Module[{c, u, p}, \n\t\tu = 1; \n\t\t
        c = EncryptBlock[cipherblock, Key[e, n]]; \n\t\t
        p = GCD[n, c - cipherblock]; \n\t\t
        While[p == 1 \[And] u \[NotEqual] max, \n\t\t\t
          c = EncryptBlock[c, Key[e, n]]; \n\t\t\t
          p = GCD[n, c - cipherblock]; \n\t\t\tu = u + 1; \n\t\t\t]; \n
        \t{u, p}]\)], "Input"],

Cell[BoxData[
    \(CycleAttackPhase2[Key[e_, n_], p_] := \n\t
      Module[{d, q}, \n\t\tq = n/p; \n\t\t
        d = PowerMod[e, \(-1\), \((p - 1)\)\ \((q - 1)\)]; \n\t\tKey[d, n]]
        \)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CompleteCycleAttack[ciphertext_, Key[e_, n_], times_] := \n\t
      Module[{round, i}, \n\t\ti = 3; \n\t\t
        round = Map[\((EncryptBlock[#, Key[e, n]])\)&, ciphertext]; \n\t\t
        While[i < times, \n\t\t\tIf[Mod[i, 1000] == 0, Print[i]]; \n\t\t\t
          round = Map[\((EncryptBlock[#, Key[e, n]])\)&, round]; \n\t\t\t
          i = i + 1]; \n\tDecryptMessage[round, Key[e, n]]]\)], "Input"],

Cell[BoxData[
    \(General::"spell1" \( : \ \) 
      "Possible spelling error: new symbol name \"\!\(times\)\" is similar to \
existing symbol \"\!\(Times\)\"."\)], "Message"],

Cell[BoxData[
    \(General::"spell1" \( : \ \) 
      "Possible spelling error: new symbol name \"\!\(round\)\" is similar to \
existing symbol \"\!\(Round\)\"."\)], "Message"]
}, Open  ]],

Cell[BoxData[
    \(\(CycleAttack[ciphertext_, Key[e_, n_], max_:  50000] := \n\t
      Module[{u, p, priv}, \n
        \t\t{u, p} = 
          CycleAttackPhase1[
            ciphertext\[LeftDoubleBracket]1\[RightDoubleBracket], Key[e, n], 
            max]; \n\t\tPrint["\<Phase 1 complete \>", {u, p}]; \n\t\t
        If[u == max, \n\t\t\t
          Print["\<Specify a larger max, and try again\>"]\n\t\t\t\t\t, \n\t\t
          If[p < n, \n\t\t\tpriv = CycleAttackPhase2[Key[e, n], p]; \n\t\t\t\t
            Print["\<p=\>", p, "\< Key=\>", priv]; \n\t\t\t
            DecryptMessage[ciphertext, priv]\n\t\t\t\t\t, \n\t\t\t
            Print["\<Doing a complete cycle of all blocks\>"]; \n\t\t\t
            CompleteCycleAttack[ciphertext, Key[e, n], u]]]]\n\t\t\t\)\)], 
  "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Common Modulus and Faulty Encryption", "Subtitle"],

Cell[TextData[ButtonBox["08_faulty.nb",
  ButtonData:>{"08_fault.nb", None},
  Active->True,
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[BoxData[
    \(BitFlip[n_] := 
      Module[{bitCount, randBit, binForm, bit}, 
        \[IndentingNewLine]bitCount = Length[IntegerDigits[n, 2]]; 
        \[IndentingNewLine]randBit = Random[Integer, {0, bitCount - 1}]; \n
        \t\tbinForm = IntegerDigits[n, 2]; \n\t\tbit = 1; 
        \[IndentingNewLine]If[
          binForm\[LeftDoubleBracket]randBit\[RightDoubleBracket] == 1, 
          bit = 0]; \n\t\t
        FromDigits[ReplacePart[binForm, bit, randBit + 1], 2]]\)], "Input"],

Cell[BoxData[
    \(GetFaultyKey[Key[e_, n_]] := 
      Module[{faulty}, \[IndentingNewLine]\[EHat] = BitFlip[e]; 
        \[IndentingNewLine]Key[\[EHat], n]]\)], "Input"],

Cell[BoxData[
    \(MultiplicativeInverse[c_, n_] := 
      Module[{g, a, b}, \n\t\t{g, {a, b}} = ExtendedGCD[c, n]; \n\t\ta]\)], 
  "Input"],

Cell[BoxData[
    \(ComModCalc[c1_, a_, c2_, b_, n_] := 
      Module[{c2Inv, p1, p2}, \n\t\tc2Inv = MultiplicativeInverse[c2, n]; \n
        \t\tp1 = PowerMod[c1, a, n]; \n\t\tp2 = PowerMod[c2Inv, Abs[b], n]; \n
        \t\tMod[p1*p2, n]]\)], "Input"],

Cell[BoxData[
    \(CommonModulus[e1_, e2_, n_, c1_, c2_] := 
      Module[{g, a, b}, \n\t\t{g, {a, b}} = ExtendedGCD[e1, e2]; \n\t\t
        If[a < 0, \n\t\t\tComModCalc[c2, b, c1, a, n], \n\t\t\t
          ComModCalc[c1, a, c2, b, n]]]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Exhaustive Search", "Subtitle"],

Cell[TextData[{
  ButtonBox["10_exhaustive.nb",
    ButtonData:>{"10_exhau.nb", None},
    Active->True,
    ButtonStyle->"Hyperlink"],
  " - I don't recomend this attack under any cirucmstances. Any other is \
faster unless one knows somehow where to start."
}], "Text"],

Cell[BoxData[
    \(Search[e_, n_, start_, stop_] := 
      \[IndentingNewLine]Module[{m, c, current, i}, 
        \[IndentingNewLine] (*We\ first\ get\ the\ message\ and\ cipher*) 
          \[IndentingNewLine]m = 2; \[IndentingNewLine]c = PowerMod[m, e, n]; 
        \[IndentingNewLine]current = PowerMod[c, start, n]; 
        \[IndentingNewLine]For[i = start, i < stop, \(i++\), 
          \[IndentingNewLine]If[Mod[i, 100000] \[Equal] 0, Print[i]]; 
          \[IndentingNewLine]If[current \[Equal] m, Break[]]; 
          \[IndentingNewLine]current = Mod[c*current, n]]; 
        \[IndentingNewLine]i]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Having guessed the Decrypting Exponent", "Subtitle"],

Cell[TextData[ButtonBox["09_guessed.nb",
  ButtonData:>{"09_gues.nb", None},
  Active->True,
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[BoxData[
    \(GetST[r_] := 
      Module[{s, t}, \n\t\ts = 0; \n\t\tt = r; \n\t\t
        While[EvenQ[t], \n\t\t\t\(s++\); \n\t\t\tt = t/2]; \n\t\t{s, t}]\)], 
  "Input"],

Cell[BoxData[
    \(GetFactor[e_, d_, n_] := \n\t
      Module[{s, t, a, i, test1, test2, p}, \n\t\t (*\ Get\ s\ and\ t\ *) \n
        \t\t{s, t} = GetST[\((e\ d)\) - 1]; \n\t\tp = 1; \n
        \t\t (*\ Repeat\ as\ often\ as\ necessary\ *) \n\t\t
        While[True, \n\t\t\t (*\ Choose\ a\ random\ a\ *) \n\t\t\t
          a = Random[Integer, {1, n}]; \n
          \t\t\t (*\ Check\ all\ the\ values\ i = 1.  . s\ *) \n\t\t\t
          For[i = 1, i < s, \(i++\), \n\t\t\t\t
            test1 = PowerMod[a, 2\^\(i - 1\)\ t, n]; \n\t\t\t\t
            If[test1 \[Equal] 1 \[Or] test1 \[Equal] \(-1\), Continue[]]; \n
            \t\t\t\ttest2 = PowerMod[a, 2\^i\ t, n]; \n\t\t\t\t
            If[test2 \[Equal] 1, \n\t\t\t\t\tp = GCD[test1 - 1, n]; \n
              \t\t\t\t\tBreak[]]]; \n
          \t\t\t (*\ If\ we\ get\ a\ result\ than\ break\ *) \n\t\t\t
          If[p > 1, Break[]]]; \n\t\tp]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Low Encrypting Exponent", "Subtitle"],

Cell[TextData[ButtonBox["11_lowexponent.nb",
  ButtonData:>{"11_lowex.nb", None},
  Active->True,
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[BoxData[
    \(GetInverse[u_, m_] := 
      Module[{k, d, g}, \n\t\t{g, {d, k}} = ExtendedGCD[u, m]; \n\t\td]\)], 
  "Input"],

Cell[BoxData[
    \(Guass[as_, ns_] := \n\t
      Module[{n, bigNs, bigMs, i, k, x}, \n\t\tn = Fold[Times, 1, ns]; \n\t\t
        k = Length[ns]; \n\t\tbigNs = Map[\((n/#)\)&, ns]; \n\t\t
        bigMs = Table[
            GetInverse[bigNs\[LeftDoubleBracket]i\[RightDoubleBracket], 
              ns\[LeftDoubleBracket]i\[RightDoubleBracket]], {i, 1, k}]; \n
        \t\tx = \[Sum]\+\(i = 1\)\%k Mod[
              as\[LeftDoubleBracket]i\[RightDoubleBracket]*
                bigNs\[LeftDoubleBracket]i\[RightDoubleBracket]*
                bigMs\[LeftDoubleBracket]i\[RightDoubleBracket], n]; \n\t\t
        Mod[x, n]]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Low Private Exponent", "Subtitle"],

Cell[TextData[ButtonBox["13_lowsecretexponent.nb",
  ButtonData:>{"13_lowse.nb", None},
  Active->True,
  ButtonStyle->"Hyperlink"]], "Text"],

Cell[BoxData[{
    \(CFrac[e_, pq_, lim_:  10] := 
      Module[{q, r, n, d, i, frac}, \n\t\tq = {0}; \n\t\tr = {e/pq}; \n\t\t
        i = 0; \n\t\tn = {0}; \n\t\td = {1}; \n\t\t
        While[i < lim, \n\t\t\t (*\ Ensure\ that\ we\ overestimate\ *) \n
          \t\t\tIf[EvenQ[i], \n\t\t\t\t
            frac = Normal[
                ContinuedFractionForm[
                  ReplacePart[q, 
                    q\[LeftDoubleBracket]i + 1\[RightDoubleBracket] + 1, 
                    i + 1]]], \n\t\t\t\t
            frac = Normal[ContinuedFractionForm[q]]]; \n\t\t\t
          res = Test[frac, e, pq]; \n\t\t\t (*Print[res]; *) \n\t\t\t
          If[IntegerQ[res], Return[res]]; \n
          \t\t\t (*\ Now\ to\ calculate\ the\ next\ q, \ r\ *) \n\t\t\t
          i = i + 1; \n\t\t\t
          q = Insert[q, Floor[1/r\[LeftDoubleBracket]i\[RightDoubleBracket]], 
              i + 1]; \n\t\t\t
          r = Insert[r, 
              \((1/r\[LeftDoubleBracket]i\[RightDoubleBracket])\) - 
                q\[LeftDoubleBracket]i + 1\[RightDoubleBracket], i + 1]; \n
          \t\t\t (*Print["\<q=\>", q]*) \n\t\t\t\t\t]; "\<nothing\>"]\), 
    \(Test[frac_, e_, pq_] := 
      Module[{k, dg, edg, pm1qm1, g, ppq, pmq, d}, \n
        \t\t (*Print["\<testing \>", frac]; *) \n\t\tk = Numerator[frac]; \n
        \t\tdg = Denominator[frac]; \n\t\tedg = e*dg; \n\t\t
        pm1qm1 = Floor[edg/k]; \n\t\tg = Mod[edg, k]; \n\t\t
        ppq = \((pq - pm1qm1 + 1)\)/2; \n\t\t
        If[\[Not] IntegerQ[ppq] \[Or] ppq \[Equal] 0, 
          Return[String["\<(p+q)/2=\>", ppq, "\< is not an Integer\>"]]]; \n
        \t\tpmq = ppq\^2 - pq; \n\t\tIf[IntegerQ[\@pmq], Return[dg/g]]; \n\t\t
        False]\n\t\t\)}], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Adaptive Chosen Plaintext Attack", "Subtitle"],

Cell[TextData[{
  "The nature of this attack involves no complicated function. See the \
extended notebook, ",
  ButtonBox["12_adaptive.nb,",
    ButtonData:>{"12_adapt.nb", None},
    Active->True,
    ButtonStyle->"Hyperlink"],
  " for details."
}], "Text"]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"Microsoft Windows 3.0",
ScreenRectangle->{{0, 1024}, {0, 712}},
WindowSize->{356, 559},
WindowMargins->{{Automatic, -24}, {Automatic, 15}}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 30, 0, 105, "Title"],
Cell[1764, 53, 335, 5, 109, "Text"],

Cell[CellGroupData[{
Cell[2124, 62, 53, 0, 93, "Subtitle"],
Cell[2180, 64, 132, 3, 33, "Text"],
Cell[2315, 69, 56, 1, 30, "Input"],
Cell[2374, 72, 176, 3, 90, "Input"],
Cell[2553, 77, 120, 2, 72, "Input"],
Cell[2676, 81, 268, 5, 130, "Input"],
Cell[2947, 88, 236, 4, 170, "Input"],
Cell[3186, 94, 753, 14, 330, "Input"],

Cell[CellGroupData[{
Cell[3964, 112, 99, 2, 50, "Input"],
Cell[4066, 116, 66, 1, 29, "Output"]
}, Open  ]],
Cell[4147, 120, 256, 5, 150, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[4440, 130, 34, 0, 53, "Subtitle"],
Cell[4477, 132, 130, 3, 33, "Text"],
Cell[4610, 137, 403, 6, 210, "Input"],
Cell[5016, 145, 164, 3, 90, "Input"],
Cell[5183, 150, 411, 7, 190, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[5631, 162, 34, 0, 53, "Subtitle"],
Cell[5668, 164, 130, 3, 33, "Text"],

Cell[CellGroupData[{
Cell[5823, 171, 113, 2, 70, "Input"],
Cell[5939, 175, 173, 3, 63, "Message"]
}, Open  ]],
Cell[6127, 181, 137, 2, 110, "Input"],
Cell[6267, 185, 139, 2, 110, "Input"],
Cell[6409, 189, 103, 2, 50, "Input"],
Cell[6515, 193, 68, 1, 30, "Input"],
Cell[6586, 196, 72, 1, 30, "Input"],
Cell[6661, 199, 73, 1, 30, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[6771, 205, 38, 0, 53, "Subtitle"],
Cell[6812, 207, 132, 3, 33, "Text"],
Cell[6947, 212, 211, 4, 110, "Input"],
Cell[7161, 218, 294, 5, 150, "Input"],
Cell[7458, 225, 370, 6, 170, "Input"],
Cell[7831, 233, 148, 3, 90, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[8016, 241, 40, 0, 53, "Subtitle"],
Cell[8059, 243, 131, 3, 33, "Text"],

Cell[CellGroupData[{
Cell[8215, 250, 98, 2, 70, "Input"],
Cell[8316, 254, 174, 3, 63, "Message"]
}, Open  ]],
Cell[8505, 260, 203, 3, 130, "Input"],
Cell[8711, 265, 197, 4, 130, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[8945, 274, 39, 0, 53, "Subtitle"],
Cell[8987, 276, 133, 3, 33, "Text"],

Cell[CellGroupData[{
Cell[9145, 283, 36, 0, 53, "Subsubtitle"],
Cell[9184, 285, 623, 10, 335, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[9844, 300, 29, 0, 42, "Subsubtitle"],
Cell[9876, 302, 507, 9, 372, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[10420, 316, 34, 0, 42, "Subsubtitle"],

Cell[CellGroupData[{
Cell[10479, 320, 360, 6, 210, "Input"],
Cell[10842, 328, 173, 3, 63, "Message"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[11064, 337, 34, 0, 53, "Subsubtitle"],
Cell[11101, 339, 1069, 20, 543, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[12207, 364, 44, 0, 42, "Subsubtitle"],

Cell[CellGroupData[{
Cell[12276, 368, 955, 15, 470, "Input"],
Cell[13234, 385, 179, 3, 63, "Message"],
Cell[13416, 390, 189, 3, 82, "Message"]
}, Open  ]],
Cell[13620, 396, 165, 3, 93, "Input"],
Cell[13788, 401, 164, 3, 93, "Input"],
Cell[13955, 406, 249, 4, 112, "Input"],
Cell[14207, 412, 159, 3, 92, "Input"],
Cell[14369, 417, 89, 1, 50, "Input"],
Cell[14461, 420, 608, 11, 334, "Input"],
Cell[15072, 433, 437, 7, 250, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[15546, 445, 40, 0, 53, "Subsubtitle"],

Cell[CellGroupData[{
Cell[15611, 449, 599, 11, 390, "Input"],
Cell[16213, 462, 186, 3, 82, "Message"]
}, Open  ]],
Cell[16414, 468, 534, 10, 270, "Input"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[16997, 484, 32, 0, 53, "Subtitle"],
Cell[17032, 486, 113, 2, 33, "Text"],
Cell[17148, 490, 440, 8, 270, "Input"],
Cell[17591, 500, 203, 4, 110, "Input"],

Cell[CellGroupData[{
Cell[17819, 508, 423, 6, 290, "Input"],
Cell[18245, 516, 177, 3, 63, "Message"],
Cell[18425, 521, 177, 3, 63, "Message"]
}, Open  ]],
Cell[18617, 527, 785, 14, 430, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[19439, 546, 56, 0, 82, "Subtitle"],
Cell[19498, 548, 130, 3, 33, "Text"],
Cell[19631, 553, 494, 9, 230, "Input"],
Cell[20128, 564, 171, 3, 90, "Input"],
Cell[20302, 569, 141, 3, 90, "Input"],
Cell[20446, 574, 252, 4, 130, "Input"],
Cell[20701, 580, 250, 4, 130, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[20988, 589, 37, 0, 53, "Subtitle"],
Cell[21028, 591, 271, 7, 71, "Text"],
Cell[21302, 600, 620, 10, 250, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[21959, 615, 58, 0, 82, "Subtitle"],
Cell[22020, 617, 130, 3, 33, "Text"],
Cell[22153, 622, 176, 4, 150, "Input"],
Cell[22332, 628, 912, 15, 438, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[23281, 648, 43, 0, 53, "Subtitle"],
Cell[23327, 650, 135, 3, 33, "Text"],
Cell[23465, 655, 130, 3, 70, "Input"],
Cell[23598, 660, 634, 11, 237, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[24269, 676, 40, 0, 53, "Subtitle"],
Cell[24312, 678, 141, 3, 33, "Text"],
Cell[24456, 683, 1717, 31, 875, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[26210, 719, 52, 0, 82, "Subtitle"],
Cell[26265, 721, 259, 8, 52, "Text"]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

