(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[      6519,        226]*)
(*NotebookOutlinePosition[      7170,        249]*)
(*  CellTagsIndexPosition[      7126,        245]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["RSA - 3", "Title"],

Cell["Key Management", "Subtitle"],

Cell["Structures", "Subsubtitle"],

Cell["\<\
Once we have the ability to generate prime numbers quickly, the next step is \
to create the public and private keys.  We will choose a representation for \
the key that will remain constant within these notebooks.\
\>", "Text"],

Cell["\<\
The combination of public and private key will be represented as :
\tRSAKey[ e, d, n ]
The public key:
\tKey[ e, n ]
The private key:
\tKey[ d, n ]
The public-private keypair:
\t{Key[ e, n ], Key[ d, n ]}\
\>", "Text"],

Cell["\<\
We may wish to store public keys for several people.  We can introduce an \
identifier with the public key:
\t{id, {e, n}}
We may have several private keys (to communicate with different groups) so \
the idea can be extended to private keys:
\t{id, {d, n}}\
\>", "Text"],

Cell[TextData[{
  "We need a structure to store multiple keys - a keyring:\n\t",
  Cell[BoxData[
      \(TraditionalForm\`{{id\_1\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`{e\_1\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`n\_1\)]],
  "}}, ",
  Cell[BoxData[
      \(TraditionalForm\`{id\_2\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`{e\_2\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`n\_2\)]],
  "}}, ... }"
}], "Text"],

Cell[CellGroupData[{

Cell["Functions", "Subsubtitle"],

Cell["\<\
The first function simply generates a keypair from the raw output of our \
generation functions.
\tRSAKey[ e, d, n ]\
\>", "Text"],

Cell[BoxData[
    \(GetPair[key_] := 
      key /. RSAKey[e_, d_, n_] \[Rule] {Key[e, n], Key[d, n]}\)], "Input"],

Cell["\<\
Our functions to extract the public or private keys should work on both the \
raw form and the keypair output.\
\>", "Text"],

Cell[BoxData[
    \(GetPublic[RSAKey[e_, d_, n_]] := Key[e, n]; \n
    GetPublic[{Key[e_, n1_], Key[d_, n2_]}] := Key[e, n1]\)], "Input"],

Cell[BoxData[
    \(GetPrivate[RSAKey[e_, d_, n_]] := Key[d, n]; \n
    GetPrivate[{Key[e_, n1_], Key[d_, n2_]}] := Key[d, n2]\)], "Input"],

Cell["\<\
We will now envelope the key generation process.  This means that only the \
following function should be accessible to anybody wishing to generate a key.\
\
\>", "Text"],

Cell[BoxData[
    \(GenerateKeyPair[ord_] := GetPair[GenerateRSAKey[ord]]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(pair = GenerateKeyPair[50]\)], "Input"],

Cell[BoxData[
    \({Key[125569680937817802761712111658061971450775433979861261849, 
        4752777942483173427113340703993553090501487743834275462465240496610973\
13972054393171612535469508491599544009], 
      Key[74865558524725844924364456797520186853995005884007721021239404144345\
319687146936394783747835767823164017649, 
        4752777942483173427113340703993553090501487743834275462465240496610973\
13972054393171612535469508491599544009]}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(GetPublic[pair]\)], "Input"],

Cell[BoxData[
    \(Key[
      748655585247258449243644567975201868539950058840077210212394041443453196\
87146936394783747835767823164017649, 
      475277794248317342711334070399355309050148774383427546246524049661097313\
972054393171612535469508491599544009]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(raw = GenerateRSAKey[50]\)], "Input"],

Cell[BoxData[
    \(RSAKey[122976311752864270048329761154362982350696328463469497093, 
      108990076675774240824194735096712351044183801060383683082051187968657124\
014477559671765116907737782679851725, 
      537295429936055182890753510480042997342991498692294030922393981857874139\
353970219642374324394922615558048553]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(GetPublic[raw]\)], "Input"],

Cell[BoxData[
    \(Key[122976311752864270048329761154362982350696328463469497093, 
      537295429936055182890753510480042997342991498692294030922393981857874139\
353970219642374324394922615558048553]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(GetPrivate[raw]\)], "Input"],

Cell[BoxData[
    \(Key[
      108990076675774240824194735096712351044183801060383683082051187968657124\
014477559671765116907737782679851725, 
      537295429936055182890753510480042997342991498692294030922393981857874139\
353970219642374324394922615558048553]\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Some other handy functions", "Subsubtitle"],

Cell[BoxData[
    \(GetModulus[Key[e_, n_]] := n; \nGetModulus[RSAKey[e_, d_, n_]] := n\)], 
  "Input"],

Cell[BoxData[
    \(\(GetExponent[Key[e_, n_]] := e; \)\)], "Input"],

Cell[BoxData[
    \(PublicExponent[RSAKey[e_, d_, n_]] := e\)], "Input"],

Cell[BoxData[
    \(PrivateExponent[RSAKey[e_, d_, n_]] := d\)], "Input"]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"Microsoft Windows 3.0",
ScreenRectangle->{{0, 1024}, {0, 740}},
WindowSize->{455, 596},
WindowMargins->{{16, Automatic}, {Automatic, 13}}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 24, 0, 105, "Title"],
Cell[1758, 53, 34, 0, 64, "Subtitle"],
Cell[1795, 55, 33, 0, 53, "Subsubtitle"],
Cell[1831, 57, 238, 4, 71, "Text"],
Cell[2072, 63, 228, 9, 166, "Text"],
Cell[2303, 74, 280, 7, 128, "Text"],
Cell[2586, 83, 461, 20, 52, "Text"],

Cell[CellGroupData[{
Cell[3072, 107, 32, 0, 53, "Subsubtitle"],
Cell[3107, 109, 140, 4, 71, "Text"],
Cell[3250, 115, 113, 2, 50, "Input"],
Cell[3366, 119, 134, 3, 52, "Text"],
Cell[3503, 124, 137, 2, 50, "Input"],
Cell[3643, 128, 139, 2, 50, "Input"],
Cell[3785, 132, 180, 4, 52, "Text"],
Cell[3968, 138, 86, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[4079, 143, 59, 1, 30, "Input"],
Cell[4141, 146, 463, 7, 238, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4641, 158, 48, 1, 30, "Input"],
Cell[4692, 161, 274, 5, 143, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5003, 171, 57, 1, 30, "Input"],
Cell[5063, 174, 337, 5, 181, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5437, 184, 47, 1, 30, "Input"],
Cell[5487, 187, 215, 3, 124, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5739, 195, 48, 1, 30, "Input"],
Cell[5790, 198, 275, 5, 143, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[6114, 209, 49, 0, 53, "Subsubtitle"],
Cell[6166, 211, 103, 2, 50, "Input"],
Cell[6272, 215, 68, 1, 30, "Input"],
Cell[6343, 218, 72, 1, 30, "Input"],
Cell[6418, 221, 73, 1, 30, "Input"]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

