(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[      6701,        226]*)
(*NotebookOutlinePosition[      7353,        249]*)
(*  CellTagsIndexPosition[      7309,        245]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["RSA - 4", "Title"],

Cell["Message Encoding", "Subtitle"],

Cell["\<\
The difficulty here is that the size of the blocks depends on the size of the \
modulus.\
\>", "Text"],

Cell["\<\
We will assume that all messages contain only characters from the ascii \
character set. If additional characters are required, then it is up to the \
calling apllication to use escape characters to access extra character sets. \
This means we only need to deal with characters that are 8-bits long - i.e \
256 characters. (NB We probably only need 128 characters for the standard \
English language set but let's not cut too many corners.)\
\>", "Text"],

Cell[TextData[{
  "A string of characters can be converted to a list of numbers in the ASCII \
set very easily in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(sen = ToCharacterCode["\<This is a silly little sentence\>"]\)], "Input"],

Cell[BoxData[
    \({84, 104, 105, 115, 32, 105, 115, 32, 97, 32, 115, 105, 108, 108, 121, 
      32, 108, 105, 116, 116, 108, 101, 32, 115, 101, 110, 116, 101, 110, 99, 
      101}\)], "Output"]
}, Open  ]],

Cell["\<\
Simply concatonating the letters is impossible as some are two digits, some \
three digits long. But we know each can be represented by 8 binary digits.  \
To concatonate, we must shift left 8 binary places and then add.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(m = 84*2\^8 + 104\)], "Input"],

Cell[BoxData[
    \(21608\)], "Output"]
}, Open  ]],

Cell["\<\
That is the first 2 characters. To get them back we just represent them base \
256.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(IntegerDigits[m, 2\^8]\)], "Input"],

Cell[BoxData[
    \({84, 104}\)], "Output"]
}, Open  ]],

Cell["\<\
Automating this process is important. As a first step let's assume that the \
numbers are in a list that is the correct length to make blocks out of.\
\>", "Text"],

Cell[BoxData[
    \(\(ListToNumber[l_] = ListToNumber[l, 0]; \n
    ListToNumber[l_, num_] := \n\t
      If[Length[l] == 0, num, \n\t\t
        ListToNumber[Rest[l], \((num*256)\) + First[l]]]\t\t\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(num = ListToNumber[sen]\)], "Input"],

Cell[BoxData[
    \(149135777980097582634002139189496602511724668690788532393018107177930220\
389\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(IntegerDigits[num, 256]\)], "Input"],

Cell[BoxData[
    \({84, 104, 105, 115, 32, 105, 115, 32, 97, 32, 115, 105, 108, 108, 121, 
      32, 108, 105, 116, 116, 108, 101, 32, 115, 101, 110, 116, 101, 110, 99, 
      101}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(% == sen\)], "Input"],

Cell[BoxData[
    \(True\)], "Output"]
}, Open  ]],

Cell["\<\
So we can easily convert a list of ascii codes to a single number and back. \
Now we need to use this as the internal workings for a function that takes a \
string and returns a list of blocks representing the message. We also need to \
specify the modulus to decide on block size\
\>", "Text"],

Cell[BoxData[
    \(BreakUpMessage[cds_, blen_] := 
      Module[{c = cds}, \n\t\t (*\ Pad\ the\ sentence\ with\ spaces\ *) \n\t\t
        While[Mod[Length[c], blen] != 0, \n\t\t\tc = Join[c, {32}]]; \n\t\t
        Table[Take[c, {i, i + blen - 1}], {i, 1, Length[c] - 1, blen}]]\)], 
  "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(BreakUpMessage[sen, 5]\)], "Input"],

Cell[BoxData[
    \({84, 104, 105, 115, 32, 105, 115, 32, 97, 32, 115, 105, 108, 108, 121, 
      32, 108, 105, 116, 116, 108, 101, 32, 115, 101, 110, 116, 101, 110, 99, 
      101, 32, 32, 32, 32}\)], "Print"],

Cell[BoxData[
    \({{84, 104, 105, 115, 32}, {105, 115, 32, 97, 32}, {115, 105, 108, 108, 
        121}, {32, 108, 105, 116, 116}, {108, 101, 32, 115, 101}, {110, 116, 
        101, 110, 99}, {101, 32, 32, 32, 32}}\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(EncodeMessage[mes_, n_] := \n (*\ n\ is\ the\ public\ modulus\ *) \n
      Module[{blklen, bitlen, l, blocks}, \n\t\t
        bitlen = Length[IntegerDigits[n, 2]] - 1; \n\t\t
        blklen = Quotient[bitlen, 8]; \n\t\tl = ToCharacterCode[mes]; \n\t\t
        blocks = BreakUpMessage[l, blklen]; \n\t\t
        Map[ListToNumber[#]&, blocks]]\)], "Input"],

Cell[BoxData[
    \(Clear[EncodeMessage]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(incode = 
      EncodeMessage["\<The same silly old sentence.\>", 95984593401]\)], 
  "Input"],

Cell[BoxData[
    \({1416127776, 1935764837, 544434540, 1819877487, 1818501235, 1701737573, 
      1852007726}\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(DecodeMessage[l_] := \n\t\t
      FromCharacterCode[\n\t\tFlatten[\n\t\t\tMap[IntegerDigits[#, 256]&, l]]]
        \)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(DecodeMessage[incode]\)], "Input"],

Cell[BoxData[
    \("The same silly old sentence."\)], "Output"]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"Microsoft Windows 3.0",
ScreenRectangle->{{0, 1024}, {0, 740}},
WindowSize->{660, 622},
WindowMargins->{{170, Automatic}, {Automatic, 15}}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 24, 0, 105, "Title"],
Cell[1758, 53, 36, 0, 64, "Subtitle"],
Cell[1797, 55, 112, 3, 33, "Text"],
Cell[1912, 60, 464, 7, 90, "Text"],
Cell[2379, 69, 186, 6, 33, "Text"],

Cell[CellGroupData[{
Cell[2590, 79, 93, 1, 30, "Input"],
Cell[2686, 82, 195, 3, 48, "Output"]
}, Open  ]],
Cell[2896, 88, 244, 4, 52, "Text"],

Cell[CellGroupData[{
Cell[3165, 96, 50, 1, 30, "Input"],
Cell[3218, 99, 39, 1, 29, "Output"]
}, Open  ]],
Cell[3272, 103, 107, 3, 33, "Text"],

Cell[CellGroupData[{
Cell[3404, 110, 55, 1, 31, "Input"],
Cell[3462, 113, 43, 1, 29, "Output"]
}, Open  ]],
Cell[3520, 117, 173, 3, 52, "Text"],
Cell[3696, 122, 211, 4, 90, "Input"],

Cell[CellGroupData[{
Cell[3932, 130, 56, 1, 30, "Input"],
Cell[3991, 133, 111, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4139, 140, 56, 1, 30, "Input"],
Cell[4198, 143, 195, 3, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4430, 151, 41, 1, 30, "Input"],
Cell[4474, 154, 38, 1, 29, "Output"]
}, Open  ]],
Cell[4527, 158, 304, 5, 71, "Text"],
Cell[4834, 165, 294, 5, 110, "Input"],

Cell[CellGroupData[{
Cell[5153, 174, 55, 1, 30, "Input"],
Cell[5211, 177, 210, 3, 44, "Print"],
Cell[5424, 182, 229, 3, 67, "Output"]
}, Open  ]],
Cell[5668, 188, 374, 6, 170, "Input"],
Cell[6045, 196, 53, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[6123, 201, 114, 3, 30, "Input"],
Cell[6240, 206, 124, 2, 29, "Output"]
}, Open  ]],
Cell[6379, 211, 148, 3, 90, "Input"],

Cell[CellGroupData[{
Cell[6552, 218, 54, 1, 30, "Input"],
Cell[6609, 221, 64, 1, 29, "Output"]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

