(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[      6291,        199]*)
(*NotebookOutlinePosition[      6941,        222]*)
(*  CellTagsIndexPosition[      6897,        218]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["RSA - 5", "Title"],

Cell["Encrypting a block", "Subtitle"],

Cell["\<\
The encryption process first translates the message into blocks of large \
numbers.  It then raises each block to the power of the public exponent, e, \
modulus the public modulus, n.\
\>", "Text"],

Cell[BoxData[
    \(EncryptBlock[mess_, key : Key[e_, n_]] := PowerMod[mess, e, n]\)], 
  "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \({pub, priv} = GenerateKeyPair[20]\)], "Input"],

Cell[BoxData[
    \({Key[27467662638016183688123, 
        860838625821754913276271148233644125913624161], 
      Key[528889549391513594108873410247309028661897151, 
        860838625821754913276271148233644125913624161]}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(mess = GenerateRandomPrime[15]\)], "Input"],

Cell[BoxData[
    \(3691251214437431\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(cipher = EncryptBlock[mess, pub]\)], "Input"],

Cell[BoxData[
    \(596093635493882052270407709259535980035287677\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(plain = EncryptBlock[cipher, priv]\)], "Input"],

Cell[BoxData[
    \(3691251214437431\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(mess == plain\)], "Input"],

Cell[BoxData[
    \(True\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Encrypting a message", "Subtitle"],

Cell["\<\
Now the final task of all is to parcel it together into two functions.  The \
first takes a text message and a key and does the encryption, returning a \
list of number blocks.  The second function will accept such blocks along \
with a key and perform a decryption. \
\>", "Text"],

Cell[BoxData[
    \(EncryptMessage[mess_, Key[e_, n_]] := 
      Module[{enc1, enc2}, \n\t\tenc1 = EncodeMessage[mess, n]; \n\t\t
        enc2 = Map[\((EncryptBlock[#, Key[e, n]])\)&, enc1]]\)], "Input"],

Cell[BoxData[
    \(\(message = 
      "\<Now the final task of all is to parcel it together into two \
functions.  The first takes a text message and a key and does the encryption, \
returning a list of number blocks.  The second function will accept such \
blocks along with a key and perform a decryption.\>"; \)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \({public, private} = GetPair[GenerateRSAKey[10]]\)], "Input"],

Cell[BoxData[
    \({Key[32110599683, 4049173415352557528859133], 
      Key[\(-358707482498287636602453\), 4049173415352557528859133]}\)], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(cipher = EncryptMessage[message, public]\)], "Input"],

Cell[BoxData[
    \({1417577655429903638119932, 3200934553390146554028431, 
      2580545181493524032678725, 2430076878871964217071604, 
      1665309734338717181913860, 1426012183037318743233967, 
      3843001873921635962026676, 2426773089057166658737404, 
      3719658436778442893597536, 1871261131365667370014840, 
      2260666644002510857737561, 1327193508041158806062155, 
      797920507422854880041156, 1239803794490939491800252, 
      3794999476510594544731870, 2022227596701426450504585, 
      3123714701439887337728639, 3932975396055364397976173, 
      902317058582441107535224, 170273718715279716276479, 
      3029760548635247630137761, 3565480667646788332547148, 
      265027410013458213530372, 2334817745558079195166831, 
      767212397226631406415255, 3256284210176780694006104, 
      2296381270367759056806137, 1465654357399628550559347, 
      1906162392024543073691746, 3019244373769800314919127, 
      2416570923980495000478573, 2638393070555334607885947, 
      827136553205537529100241}\)], "Output"]
}, Open  ]],

Cell["\<\
Here we reverse the process. First decrypt (re-encrypt with the private key \
this time) each block and then convert back to a String. Note that there may \
be some additional whitespace added to the end of the message. I will not be \
removing it.\
\>", "Text"],

Cell[BoxData[
    \(DecryptMessage[cipher_, Key[d_, n_]] := 
      Module[{dec}, \n\t\t
        dec = Map[\((EncryptBlock[#, Key[d, n]])\)&, cipher]; \n\t\t
        DecodeMessage[dec]]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(DecryptMessage[cipher, private]\)], "Input"],

Cell[BoxData[
    \("Now the final task of all is to parcel it together into two functions. \
 The first takes a text message and a key and does the encryption, returning \
a list of number blocks.  The second function will accept such blocks along \
with a key and perform a decryption.    "\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"Microsoft Windows 3.0",
ScreenRectangle->{{0, 1024}, {0, 740}},
WindowSize->{488, 637},
WindowMargins->{{27, Automatic}, {Automatic, 5}}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 24, 0, 105, "Title"],
Cell[1758, 53, 38, 0, 64, "Subtitle"],
Cell[1799, 55, 207, 4, 52, "Text"],
Cell[2009, 61, 98, 2, 30, "Input"],

Cell[CellGroupData[{
Cell[2132, 67, 66, 1, 30, "Input"],
Cell[2201, 70, 235, 4, 86, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2473, 79, 63, 1, 30, "Input"],
Cell[2539, 82, 50, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2626, 88, 65, 1, 30, "Input"],
Cell[2694, 91, 79, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2810, 97, 67, 1, 30, "Input"],
Cell[2880, 100, 50, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2967, 106, 46, 1, 30, "Input"],
Cell[3016, 109, 38, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3091, 115, 40, 0, 64, "Subtitle"],
Cell[3134, 117, 291, 5, 71, "Text"],
Cell[3428, 124, 203, 3, 70, "Input"],
Cell[3634, 129, 328, 5, 130, "Input"],

Cell[CellGroupData[{
Cell[3987, 138, 80, 1, 30, "Input"],
Cell[4070, 141, 152, 3, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4259, 149, 73, 1, 30, "Input"],
Cell[4335, 152, 1031, 17, 333, "Output"]
}, Open  ]],
Cell[5381, 172, 272, 5, 71, "Text"],
Cell[5656, 179, 197, 4, 70, "Input"],

Cell[CellGroupData[{
Cell[5878, 187, 64, 1, 30, "Input"],
Cell[5945, 190, 306, 4, 105, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

