(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     37522,       1246]*)
(*NotebookOutlinePosition[     38171,       1269]*)
(*  CellTagsIndexPosition[     38127,       1265]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["RSA - 6", "Title"],

Cell["Cycle Attack", "Subtitle"],

Cell[TextData[{
  "The cycle attack is due to Simmons and Norris (reference to come).  Given \
a ciphertext ",
  Cell[BoxData[
      \(TraditionalForm\`C = M\^e\ mod\ n\)]],
  ", the cryptanalyst repeatedly re-encrypts the ciphertext until C is again \
obtained.  If k re-encryptions are performed, then\n\t",
  Cell[BoxData[
      \(TraditionalForm\`C\^\(e\^k\) \[Congruent] \ C\ \((mod\ n)\)\)]],
  ",\nand so\n\t",
  Cell[BoxData[
      \(TraditionalForm
      \`C\^\(e\^\(k - 1\)\) \[Congruent] \ M\ \(\((mod\ n)\) . \)\)]]
}], "Text"],

Cell["We will demostrate this with a small key and message.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(key = GenerateRSAKey[7]\)], "Input"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \(RSAKey[72421, 1392237509989, 3782974780567]\)], "Output"]
}, Open  ]],

Cell["I got the attack working well with the following key", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(key = RSAKey[97, \(-116356097327\), 434191388627]\)], "Input"],

Cell[BoxData[
    \(RSAKey[97, \(-116356097327\), 434191388627]\)], "Output"]
}, Open  ]],

Cell["\<\
The text is taken from the manual to the game Driver, which happened to be \
the nearest document at hand.\
\>", "Text"],

Cell[BoxData[
    \(\(plaintext = 
      "\<You play Tanner, a maverick Cop who hands in his badge to go \
undercover to take on the kingpin of crime - Castaldi. Tanner is called to \
the Police chief's office in New York and given instructions to head for \
Miami and rendezvous with a cat called Rufus. After a tip off following his \
arrival in Miami, Tanner meets with a number of seedy gangsters and violent \
hoods at an underground car park.\>"; \)\)], "Input"],

Cell["The output from the ciphertext is suppressed.", "Text"],

Cell[BoxData[
    \(\(ciphertext = EncryptMessage[plaintext, GetPublic[key]]; \)\)], "Input"],

Cell["We only need one block to perform a cycle attack.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(c = ciphertext\[LeftDoubleBracket]1\[RightDoubleBracket]\)], "Input"],

Cell[BoxData[
    \(274545910344\)], "Output"]
}, Open  ]],

Cell["\<\
The function might not return in reasonable time so it has an optional \
maximum number of re-encryptions.  It will exit when it reaches this limit or \
when it gets a match.  In either case it will return the number of cycles.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CycleAttackAttempt[cipher_, key_, max_:  1000] := 
      Module[{c, i}, \n\t\ti = 1; \n\t\tc = EncryptBlock[cipher, key]; \n\t\t
        While[i < max \[And] cipher != c, \n\t\t\tc = EncryptBlock[c, key]; \n
          \t\t\ti = i + 1]; \n\ti]\)], "Input"],

Cell[BoxData[
    \(General::"spell1" \( : \ \) 
      "Possible spelling error: new symbol name \"\!\(max\)\" is similar to \
existing symbol \"\!\(Max\)\"."\)], "Message"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(CycleAttackAttempt[c, GetPublic[key]]\)], "Input"],

Cell[BoxData[
    \(1000\)], "Output"]
}, Open  ]],

Cell["\<\
Our first attempt (at 1000 cycles) failed.  Try increasing the number of \
cycles.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(k = CycleAttackAttempt[c, GetPublic[key], 10000]\)], "Input"],

Cell[BoxData[
    \(3060\)], "Output"]
}, Open  ]],

Cell["\<\
Once we know the number of cylces we can finish the entire message\
\>", "Text"],

Cell[BoxData[
    \(CompleteCycleAttack[ciphertext_, Key[e_, n_], times_] := \n\t
      Module[{round, i}, \n\t\ti = 3; \n\t\t
        round = Map[\((EncryptBlock[#, Key[e, n]])\)&, ciphertext]; \n\t\t
        While[i < times, \n\t\t\tIf[Mod[i, 1000] == 0, Print[i]]; \n\t\t\t
          round = Map[\((EncryptBlock[#, Key[e, n]])\)&, round]; \n\t\t\t
          i = i + 1]; \n\tDecryptMessage[round, Key[e, n]]]\)], "Input"],

Cell["\<\
Using the number of cycles exactly as the first function returned (the \
fucntion expects this) we can decrypt the entire message.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CompleteCycleAttack[ciphertext, GetPublic[key], k]\)], "Input"],

Cell[BoxData[
    \(1000\)], "Print"],

Cell[BoxData[
    \(2000\)], "Print"],

Cell[BoxData[
    \(3000\)], "Print"],

Cell[BoxData[
    \("You play Tanner, a maverick Cop who hands in his badge to go \
undercover to take on the kingpin of crime - Castaldi. Tanner is called to \
the Police chief's office in New York and given instructions to head for \
Miami and rendezvous with a cat called Rufus. After a tip off following his \
arrival in Miami, Tanner meets with a number of seedy gangsters and violent \
hoods at an underground car park.  "\)], "Output"]
}, Open  ]],

Cell["Voila, the original text", "Text"],

Cell[CellGroupData[{

Cell["Generalizing and Improving", "Subtitle"],

Cell["\<\
One of the problems with the above approach is that it is quite slow.  Having \
found the number of cycles, we have to encrypt each block by that number of \
cycles.  \
\>", "Text"],

Cell[TextData[{
  "Williams and Schmid generalized the attack so that it is always at least \
as efficient as the Simmons and Norris attack, but will often be much better. \
 Rather than looking for the least ",
  StyleBox["k",
    FontSlant->"Italic"],
  " we look for the least ",
  StyleBox["u",
    FontSlant->"Italic"],
  " such that\n\t",
  Cell[BoxData[
      \(TraditionalForm\`gcd(n, C\^\(e\^k\)\  - \ C\ ) > 1\)]],
  "\nTypically the gcd computation will return one of the factors of n, an \
therefore n has been factored.  In the rare case where n itself is returned, \
we can still compute M using the Simmons and Norris attack.  "
}], "Text"],

Cell[BoxData[
    \(CycleAttackPhase1[cipherblock_, Key[e_, n_], max_:  5000] := \n\t
      Module[{c, u, p}, \n\t\tu = 1; \n\t\t
        c = EncryptBlock[cipherblock, Key[e, n]]; \n\t\t
        p = GCD[n, c - cipherblock]; \n\t\t
        While[p == 1 \[And] u \[NotEqual] max, \n\t\t\t
          c = EncryptBlock[c, Key[e, n]]; \n\t\t\t
          p = GCD[n, c - cipherblock]; \n\t\t\tu = u + 1; \n\t\t\t]; \n
        \t{u, p}]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(c\)], "Input"],

Cell[BoxData[
    \(274545910344\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(key\)], "Input"],

Cell[BoxData[
    \(RSAKey[97, \(-116356097327\), 434191388627]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(pub = GetPublic[key]\)], "Input"],

Cell[BoxData[
    \(Key[97, 434191388627]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(CycleAttackPhase1[c, pub]\)], "Input"],

Cell[BoxData[
    \({30, 4637}\)], "Output"]
}, Open  ]],

Cell["\<\
Notice how much fewer cycles were needed - 30 as opposed to 3060.  Having \
completed phase 1, the private key can be discovered.\
\>", "Text"],

Cell[BoxData[
    \(CycleAttackPhase2[Key[e_, n_], p_] := \n\t
      Module[{d, q, e2, n2}, \n\t\tq = n/p; \n
        \t\t{e2, d, n2} = GetKey[e, p, q]; \n\t\tKey[d, n]]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CycleAttackPhase2[pub, 4637]\)], "Input"],

Cell[BoxData[
    \(Key[\(-116356097327\), 434191388627]\)], "Output"]
}, Open  ]],

Cell["\<\
Which is the same as the private key corresponding to our public key.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(GetPrivate[key]\)], "Input"],

Cell[BoxData[
    \(Key[\(-116356097327\), 434191388627]\)], "Output"]
}, Open  ]],

Cell["\<\
Now it is time to create a single function that uses the above to automate \
the process.\
\>", "Text"],

Cell[BoxData[
    \(\(CycleAttack[ciphertext_, Key[e_, n_], max_:  50000] := \n\t
      Module[{u, p, priv}, \n
        \t\t{u, p} = 
          CycleAttackPhase1[
            ciphertext\[LeftDoubleBracket]1\[RightDoubleBracket], Key[e, n], 
            max]; \n\t\tPrint["\<Phase 1 complete \>", {u, p}]; \n\t\t
        If[u == max, \n\t\t\t
          Print["\<Specify a larger max, and try again\>"]\n\t\t\t\t\t, \n\t\t
          If[p < n, \n\t\t\tpriv = CycleAttackPhase2[Key[e, n], p]; \n\t\t\t
            DecryptMessage[ciphertext, priv]\n\t\t\t\t\t, \n\t\t\t
            Print["\<Doing a complete cycle of all blocks\>"]; \n\t\t\t
            CompleteCycleAttack[ciphertext, Key[e, n], u]]]]\n\t\t\t\)\)], 
  "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CycleAttack[ciphertext, pub]\)], "Input"],

Cell[BoxData[
    InterpretationBox[\("Phase 1 complete "\[InvisibleSpace]{30, 4637}\),
      SequenceForm[ "Phase 1 complete ", {30, 4637}],
      Editable->False]], "Print"],

Cell[BoxData[
    \("You play Tanner, a maverick Cop who hands in his badge to go \
undercover to take on the kingpin of crime - Castaldi. Tanner is called to \
the Police chief's office in New York and given instructions to head for \
Miami and rendezvous with a cat called Rufus. After a tip off following his \
arrival in Miami, Tanner meets with a number of seedy gangsters and violent \
hoods at an underground car park.  "\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Large Key", "Subtitle"],

Cell["\<\
We have to now increase the keylength to see just how good this attack is.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(key2 = GenerateRSAKey[7]\)], "Input"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \(RSAKey[89591, 81729046763, 358635249437]\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(ct2 = EncryptMessage[plaintext, GetPublic[key2]]; \)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[CycleAttack[ct2, GetPublic[key2]]]\)], "Input"],

Cell[BoxData[
    InterpretationBox[
      \("Phase 1 complete "\[InvisibleSpace]{27479, 109919}\),
      SequenceForm[ "Phase 1 complete ", {27479, 109919}],
      Editable->False]], "Print"],

Cell[BoxData[
    \({14.6100000000001273`\ Second, 
      "You play Tanner, a maverick Cop who hands in his badge to go \
undercover to take on the kingpin of crime - Castaldi. Tanner is called to \
the Police chief's office in New York and given instructions to head for \
Miami and rendezvous with a cat called Rufus. After a tip off following his \
arrival in Miami, Tanner meets with a number of seedy gangsters and violent \
hoods at an underground car park.  "}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(key3 = GenerateRSAKey[8]\)], "Input"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \(RSAKey[959759, 4324853491438639, 7902555270238621]\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(ct3 = EncryptMessage[plaintext, GetPublic[key3]]; \)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[CycleAttack[ct3, GetPublic[key3]]]\)], "Input"],

Cell[BoxData[
    InterpretationBox[\("Phase 1 complete "\[InvisibleSpace]{50000, 1}\),
      SequenceForm[ "Phase 1 complete ", {50000, 1}],
      Editable->False]], "Print"],

Cell[BoxData[
    \("Specify a larger max, and try again"\)], "Print"],

Cell[BoxData[
    \({38.990000000000009`\ Second, Null}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[CycleAttack[ct3, GetPublic[key3], 100000]]\)], "Input"],

Cell[BoxData[
    InterpretationBox[\("Phase 1 complete "\[InvisibleSpace]{100000, 1}\),
      SequenceForm[ "Phase 1 complete ", {100000, 1}],
      Editable->False]], "Print"],

Cell[BoxData[
    \("Specify a larger max, and try again"\)], "Print"],

Cell[BoxData[
    \({78.7599999999999944`\ Second, Null}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "I wonder is it easier if we don't use the ",
  Cell[BoxData[
      \(TraditionalForm\`\(p\^--\)\)]],
  " strong prime algorithm. We can use the optional parameter to avoid using \
this algorithm"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(key4 = GenerateRSAKey[6, Algorithm \[Rule] GenerateRandomPrime]\)], 
  "Input"],

Cell[BoxData[
    \(RSAKey[3557, \(-43288555539\), 73603526671]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Length[IntegerDigits[GetModulus[key3]]]\)], "Input"],

Cell[BoxData[
    \(10\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Length[IntegerDigits[GetModulus[key4]]]\)], "Input"],

Cell[BoxData[
    \(11\)], "Output"]
}, Open  ]],

Cell["\<\
Now we have similar sized keys - is there any difference in the attack.\
\>", "Text"],

Cell[BoxData[
    \(\(ct4 = EncryptMessage[plaintext, GetPublic[key4]]; \)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[CycleAttack[ct4, GetPublic[key4]]]\)], "Input"],

Cell[BoxData[
    InterpretationBox[
      \("Phase 1 complete "\[InvisibleSpace]{25152, 201473}\),
      SequenceForm[ "Phase 1 complete ", {25152, 201473}],
      Editable->False]], "Print"],

Cell[BoxData[
    \({9.8900000000000432`\ Second, 
      "You play Tanner, a maverick Cop who hands in his badge to go \
undercover to take on the kingpin of crime - Castaldi. Tanner is called to \
the Police chief's office in New York and given instructions to head for \
Miami and rendezvous with a cat called Rufus. After a tip off following his \
arrival in Miami, Tanner meets with a number of seedy gangsters and violent \
hoods at an underground car park.  "}\)], "Output"]
}, Open  ]],

Cell["\<\
Well this time it got it all right. We'll have to do further tests though. \
And I don't expect it to work for larger sized keys.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(key5 = GenerateRSAKey[10, Algorithm \[Rule] GenerateRandomPrime]\)], 
  "Input"],

Cell[BoxData[
    \(RSAKey[5712719, \(-8794249243106542801\), 32629344062392260103]\)], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(bits = Length[IntegerDigits[GetModulus[key5], 2]]\)], "Input"],

Cell[BoxData[
    \(65\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(ct5 = EncryptMessage[plaintext, GetPublic[key5]]; \)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[CycleAttack[ct5, GetPublic[key5], 1000000]]\)], "Input"],

Cell[BoxData[
    InterpretationBox[
      \("Phase 1 complete "\[InvisibleSpace]{147600, 5587188007}\),
      SequenceForm[ "Phase 1 complete ", {147600, 5587188007}],
      Editable->False]], "Print"],

Cell[BoxData[
    \({144.46000000000005`\ Second, 
      "You play Tanner, a maverick Cop who hands in his badge to go \
undercover to take on the kingpin of crime - Castaldi. Tanner is called to \
the Police chief's office in New York and given instructions to head for \
Miami and rendezvous with a cat called Rufus. After a tip off following his \
arrival in Miami, Tanner meets with a number of seedy gangsters and violent \
hoods at an underground car park.      "}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "This time it worked. But again, does using a ",
  Cell[BoxData[
      \(TraditionalForm\`\(p\^--\)\)]],
  " strong prime help matters."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(key6 = GenerateRSAKey[10]\)], "Input"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \(RSAKey[8984077, \(-24905784696896991459\), 67315412133965909401]\)], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(bits = Length[IntegerDigits[GetModulus[key6], 2]]\)], "Input"],

Cell[BoxData[
    \(66\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(ct6 = EncryptMessage[plaintext, GetPublic[key6]]; \)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[CycleAttack[ct6, GetPublic[key6], 1000000]]\)], "Input"],

Cell[BoxData[
    InterpretationBox[\("Phase 1 complete "\[InvisibleSpace]{1000000, 1}\),
      SequenceForm[ "Phase 1 complete ", {1000000, 1}],
      Editable->False]], "Print"],

Cell[BoxData[
    \("Specify a larger max, and try again"\)], "Print"],

Cell[BoxData[
    \({924.230000000000018`\ Second, Null}\)], "Output"]
}, Open  ]],

Cell["\<\
Back from my cup of tea and I see there's been no result. Well I don't plan \
on trying again.  Surely this is an indication that the strong primes are \
more resistant. Further testing is required.\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["More Tests", "Subtitle"],

Cell["Generate a range of keyh sizes.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(keys18 = Table[GetPublic[GenerateRSAKey[i]], {i, 3, 10}]\)], "Input"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \({Key[137, 2773], Key[7789, 2026907], Key[54617, 1056133747], 
      Key[501779, 3015405142667], Key[7968617, 778387792569367], 
      Key[24236269, 28775678395299287], Key[264340337, 4284886968834812009], 
      Key[4321309873, 502246790654963602189]}\)], "Output"]
}, Closed]],

Cell["\<\
Get the ciphertexts for them (and a different shorter message)\
\>", "Text"],

Cell[BoxData[
    \(\(ptext = "\<It Worked!\>"; \)\)], "Input"],

Cell[BoxData[
    \(\(cts20 = \(EncryptMessage[ptext, #]&\)/@keys18; \)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{i + 2, 
        Timing[CycleAttack[cts20\[LeftDoubleBracket]i\[RightDoubleBracket], 
            keys18\[LeftDoubleBracket]i\[RightDoubleBracket]]]}, {i, 1, 
        Length[cts20]}]\)], "Input"],

Cell[BoxData[
    InterpretationBox[\("Phase 1 complete "\[InvisibleSpace]{2, 47}\),
      SequenceForm[ "Phase 1 complete ", {2, 47}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[
      \("p="\[InvisibleSpace]47
        \[InvisibleSpace]" Key="\[InvisibleSpace]Key[1149, 2773]\),
      SequenceForm[ "p=", 47, " Key=", 
        Key[ 1149, 2773]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Phase 1 complete "\[InvisibleSpace]{148, 1193}\),
      SequenceForm[ "Phase 1 complete ", {148, 1193}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[
      \("p="\[InvisibleSpace]1193
        \[InvisibleSpace]" Key="\[InvisibleSpace]Key[1460389, 2026907]\),
      SequenceForm[ "p=", 1193, " Key=", 
        Key[ 1460389, 2026907]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Phase 1 complete "\[InvisibleSpace]{3676, 22063}\),
      SequenceForm[ "Phase 1 complete ", {3676, 22063}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[
      \("p="\[InvisibleSpace]22063
        \[InvisibleSpace]" Key="\[InvisibleSpace]Key[491709593, 1056133747]
          \),
      SequenceForm[ "p=", 22063, " Key=", 
        Key[ 491709593, 1056133747]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Phase 1 complete "\[InvisibleSpace]{9871, 197423}\),
      SequenceForm[ "Phase 1 complete ", {9871, 197423}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[
      \("p="\[InvisibleSpace]197423
        \[InvisibleSpace]" Key="\[InvisibleSpace]Key[2272147338515, 
          3015405142667]\),
      SequenceForm[ "p=", 197423, " Key=", 
        Key[ 2272147338515, 3015405142667]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Phase 1 complete "\[InvisibleSpace]{50000, 1}\),
      SequenceForm[ "Phase 1 complete ", {50000, 1}],
      Editable->False]], "Print"],

Cell[BoxData[
    \("Specify a larger max, and try again"\)], "Print"],

Cell[BoxData[
    InterpretationBox[\("Phase 1 complete "\[InvisibleSpace]{50000, 1}\),
      SequenceForm[ "Phase 1 complete ", {50000, 1}],
      Editable->False]], "Print"],

Cell[BoxData[
    \("Specify a larger max, and try again"\)], "Print"],

Cell[BoxData[
    InterpretationBox[\("Phase 1 complete "\[InvisibleSpace]{50000, 1}\),
      SequenceForm[ "Phase 1 complete ", {50000, 1}],
      Editable->False]], "Print"],

Cell[BoxData[
    \("Specify a larger max, and try again"\)], "Print"],

Cell[BoxData[
    InterpretationBox[\("Phase 1 complete "\[InvisibleSpace]{50000, 1}\),
      SequenceForm[ "Phase 1 complete ", {50000, 1}],
      Editable->False]], "Print"],

Cell[BoxData[
    \("Specify a larger max, and try again"\)], "Print"],

Cell[BoxData[
    \({{3, {0.`\ Second, "It Worked"}}, {
        4, {0.0500000000001818989`\ Second, "It Worked!"}}, {
        5, {1.21000000000003637`\ Second, "It Worked!  "}}, {
        6, {5.32999999999992724`\ Second, "It Worked!"}}, {
        7, {43.7700000000004418`\ Second, Null}}, {
        8, {49.050000000000189`\ Second, Null}}, {
        9, {54.2699999999995341`\ Second, Null}}, {
        10, {62.8900000000003345`\ Second, Null}}}\)], "Output"]
}, Open  ]],

Cell["\<\
Automate this test. That's the next thing. And allow non-strong primes to be \
used.
I also want to use my improved attack. This\
\>", "Text"],

Cell[BoxData[
    \(AutomaticTest[kLow_, kHigh_, strong_] := 
      Module[{ks, cts, opt, len}, \n\t\t
        If[strong, opt = {}, opt = {Algorithm \[Rule] GenerateRandomPrime}]; 
        \n\t\tks = 
          Table[GetPublic[GenerateRSAKey[i, opt]], {i, kLow, kHigh}]; \n\t\t
        cts = \(EncryptMessage["\<It Worked!\>", #]&\)/@ks; \n\t\t
        len = \(Length[IntegerDigits[GetModulus[#], 2]]&\)/@ks; \n\t\t
        Table[{i + 2, len\[LeftDoubleBracket]i\[RightDoubleBracket], 
            Timing[CycleAttack[cts\[LeftDoubleBracket]i\[RightDoubleBracket], 
                ks\[LeftDoubleBracket]i\[RightDoubleBracket]]]}, {i, 1, 
            Length[cts]}]]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(AutomaticTest[4, 12, False]\)], "Input"],

Cell[BoxData[
    InterpretationBox[\("Phase 1 complete "\[InvisibleSpace]{138, 5839}\),
      SequenceForm[ "Phase 1 complete ", {138, 5839}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[
      \("p="\[InvisibleSpace]5839
        \[InvisibleSpace]" Key="\[InvisibleSpace]Key[29345243, 51891193]\),
      SequenceForm[ "p=", 5839, " Key=", 
        Key[ 29345243, 51891193]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Phase 1 complete "\[InvisibleSpace]{24, 21313}\),
      SequenceForm[ "Phase 1 complete ", {24, 21313}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[
      \("p="\[InvisibleSpace]21313
        \[InvisibleSpace]" Key="\[InvisibleSpace]Key[247909369, 916522939]\),
      SequenceForm[ "p=", 21313, " Key=", 
        Key[ 247909369, 916522939]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Phase 1 complete "\[InvisibleSpace]{966, 542921}\),
      SequenceForm[ "Phase 1 complete ", {966, 542921}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[
      \("p="\[InvisibleSpace]542921
        \[InvisibleSpace]" Key="\[InvisibleSpace]Key[95518310539, 95716429379]
          \),
      SequenceForm[ "p=", 542921, " Key=", 
        Key[ 95518310539, 95716429379]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Phase 1 complete "\[InvisibleSpace]{50000, 1}\),
      SequenceForm[ "Phase 1 complete ", {50000, 1}],
      Editable->False]], "Print"],

Cell[BoxData[
    \("Specify a larger max, and try again"\)], "Print"],

Cell[BoxData[
    InterpretationBox[\("Phase 1 complete "\[InvisibleSpace]{50000, 1}\),
      SequenceForm[ "Phase 1 complete ", {50000, 1}],
      Editable->False]], "Print"],

Cell[BoxData[
    \("Specify a larger max, and try again"\)], "Print"],

Cell[BoxData[
    InterpretationBox[
      \("Phase 1 complete "\[InvisibleSpace]{10620, 397439569}\),
      SequenceForm[ "Phase 1 complete ", {10620, 397439569}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[
      \("p="\[InvisibleSpace]397439569
        \[InvisibleSpace]" Key="\[InvisibleSpace]Key[324357427404280451, 
          334331376272989159]\),
      SequenceForm[ "p=", 397439569, " Key=", 
        Key[ 324357427404280451, 334331376272989159]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Phase 1 complete "\[InvisibleSpace]{50000, 1}\),
      SequenceForm[ "Phase 1 complete ", {50000, 1}],
      Editable->False]], "Print"],

Cell[BoxData[
    \("Specify a larger max, and try again"\)], "Print"],

Cell[BoxData[
    InterpretationBox[\("Phase 1 complete "\[InvisibleSpace]{50000, 1}\),
      SequenceForm[ "Phase 1 complete ", {50000, 1}],
      Editable->False]], "Print"],

Cell[BoxData[
    \("Specify a larger max, and try again"\)], "Print"],

Cell[BoxData[
    InterpretationBox[\("Phase 1 complete "\[InvisibleSpace]{50000, 1}\),
      SequenceForm[ "Phase 1 complete ", {50000, 1}],
      Editable->False]], "Print"],

Cell[BoxData[
    \("Specify a larger max, and try again"\)], "Print"],

Cell[BoxData[
    \({{3, 26, {0.0599999999994906829`\ Second, "It Worked!  "}}, {4, 
        30, {0.0500000000001818989`\ Second, "It Worked!  "}}, {5, 
        37, {0.550000000000181898`\ Second, "It Worked!  "}}, {6, 
        46, {34.1599999999998571`\ Second, Null}}, {7, 
        54, {53.3900000000003327`\ Second, Null}}, {8, 
        59, {13.0699999999997107`\ Second, "It Worked!    "}}, {9, 
        65, {62.840000000000149`\ Second, Null}}, {10, 
        73, {76.1800000000002963`\ Second, Null}}, {11, 
        79, {79.9200000000000798`\ Second, Null}}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(AutomaticTest[4, 12, True]\)], "Input"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    InterpretationBox[\("Phase 1 complete "\[InvisibleSpace]{89, 359}\),
      SequenceForm[ "Phase 1 complete ", {89, 359}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[
      \("p="\[InvisibleSpace]359
        \[InvisibleSpace]" Key="\[InvisibleSpace]Key[11899, 222221]\),
      SequenceForm[ "p=", 359, " Key=", 
        Key[ 11899, 222221]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Phase 1 complete "\[InvisibleSpace]{866, 3467}\),
      SequenceForm[ "Phase 1 complete ", {866, 3467}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[
      \("p="\[InvisibleSpace]3467
        \[InvisibleSpace]" Key="\[InvisibleSpace]Key[1961397, 28536877]\),
      SequenceForm[ "p=", 3467, " Key=", 
        Key[ 1961397, 28536877]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[
      \("Phase 1 complete "\[InvisibleSpace]{43242, 691877}\),
      SequenceForm[ "Phase 1 complete ", {43242, 691877}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[
      \("p="\[InvisibleSpace]691877
        \[InvisibleSpace]" Key="\[InvisibleSpace]Key[315812759383, 
          894886857463]\),
      SequenceForm[ "p=", 691877, " Key=", 
        Key[ 315812759383, 894886857463]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Phase 1 complete "\[InvisibleSpace]{50000, 1}\),
      SequenceForm[ "Phase 1 complete ", {50000, 1}],
      Editable->False]], "Print"],

Cell[BoxData[
    \("Specify a larger max, and try again"\)], "Print"],

Cell[BoxData[
    InterpretationBox[\("Phase 1 complete "\[InvisibleSpace]{50000, 1}\),
      SequenceForm[ "Phase 1 complete ", {50000, 1}],
      Editable->False]], "Print"],

Cell[BoxData[
    \("Specify a larger max, and try again"\)], "Print"],

Cell[BoxData[
    InterpretationBox[\("Phase 1 complete "\[InvisibleSpace]{50000, 1}\),
      SequenceForm[ "Phase 1 complete ", {50000, 1}],
      Editable->False]], "Print"],

Cell[BoxData[
    \("Specify a larger max, and try again"\)], "Print"],

Cell[BoxData[
    InterpretationBox[\("Phase 1 complete "\[InvisibleSpace]{50000, 1}\),
      SequenceForm[ "Phase 1 complete ", {50000, 1}],
      Editable->False]], "Print"],

Cell[BoxData[
    \("Specify a larger max, and try again"\)], "Print"],

Cell[BoxData[
    InterpretationBox[\("Phase 1 complete "\[InvisibleSpace]{50000, 1}\),
      SequenceForm[ "Phase 1 complete ", {50000, 1}],
      Editable->False]], "Print"],

Cell[BoxData[
    \("Specify a larger max, and try again"\)], "Print"],

Cell[BoxData[
    InterpretationBox[\("Phase 1 complete "\[InvisibleSpace]{50000, 1}\),
      SequenceForm[ "Phase 1 complete ", {50000, 1}],
      Editable->False]], "Print"],

Cell[BoxData[
    \("Specify a larger max, and try again"\)], "Print"],

Cell[BoxData[
    \({{3, 18, {0.0500000000001818989`\ Second, "It Worked!"}}, {4, 
        25, {0.279999999999745341`\ Second, "It Worked!  "}}, {5, 
        40, {27.2399999999997843`\ Second, "It Worked!"}}, {6, 
        50, {42.5100000000002165`\ Second, Null}}, {7, 
        50, {53.0599999999994942`\ Second, Null}}, {8, 
        61, {63.5500000000001818`\ Second, Null}}, {9, 
        70, {69.8699999999998944`\ Second, Null}}, {10, 
        73, {69.7500000000000053`\ Second, Null}}, {11, 
        76, {80.8500000000003637`\ Second, Null}}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(AutomaticTest[4, 10, False]\)], "Input"],

Cell[BoxData[
    InterpretationBox[\("Phase 1 complete "\[InvisibleSpace]{2, 3011}\),
      SequenceForm[ "Phase 1 complete ", {2, 3011}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[
      \("p="\[InvisibleSpace]3011
        \[InvisibleSpace]" Key="\[InvisibleSpace]Key[12775729, 18659167]\),
      SequenceForm[ "p=", 3011, " Key=", 
        Key[ 12775729, 18659167]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Phase 1 complete "\[InvisibleSpace]{336, 92107}\),
      SequenceForm[ "Phase 1 complete ", {336, 92107}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[
      \("p="\[InvisibleSpace]92107
        \[InvisibleSpace]" Key="\[InvisibleSpace]Key[18505583, 4079695351]\),
      SequenceForm[ "p=", 92107, " Key=", 
        Key[ 18505583, 4079695351]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Phase 1 complete "\[InvisibleSpace]{30, 104347}\),
      SequenceForm[ "Phase 1 complete ", {30, 104347}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[
      \("p="\[InvisibleSpace]104347
        \[InvisibleSpace]" Key="\[InvisibleSpace]Key[29082463207, 70170540131]
          \),
      SequenceForm[ "p=", 104347, " Key=", 
        Key[ 29082463207, 70170540131]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[
      \("Phase 1 complete "\[InvisibleSpace]{12510, 4295149}\),
      SequenceForm[ "Phase 1 complete ", {12510, 4295149}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[
      \("p="\[InvisibleSpace]4295149
        \[InvisibleSpace]" Key="\[InvisibleSpace]Key[3804042539059, 
          17532665068381]\),
      SequenceForm[ "p=", 4295149, " Key=", 
        Key[ 3804042539059, 17532665068381]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[
      \("Phase 1 complete "\[InvisibleSpace]{1296, 41830277}\),
      SequenceForm[ "Phase 1 complete ", {1296, 41830277}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[
      \("p="\[InvisibleSpace]41830277
        \[InvisibleSpace]" Key="\[InvisibleSpace]Key[1070777999759807, 
          2302079140192877]\),
      SequenceForm[ "p=", 41830277, " Key=", 
        Key[ 1070777999759807, 2302079140192877]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Phase 1 complete "\[InvisibleSpace]{50000, 1}\),
      SequenceForm[ "Phase 1 complete ", {50000, 1}],
      Editable->False]], "Print"],

Cell[BoxData[
    \("Specify a larger max, and try again"\)], "Print"],

Cell[BoxData[
    InterpretationBox[\("Phase 1 complete "\[InvisibleSpace]{50000, 1}\),
      SequenceForm[ "Phase 1 complete ", {50000, 1}],
      Editable->False]], "Print"],

Cell[BoxData[
    \("Specify a larger max, and try again"\)], "Print"],

Cell[BoxData[
    \({{3, 25, {0.`\ Second, "It Worked!  "}}, {4, 
        32, {0.219999999999345163`\ Second, "It Worked!  "}}, {5, 
        37, {0.`\ Second, "It Worked!  "}}, {6, 
        44, {7.96000000000003637`\ Second, "It Worked!"}}, {7, 
        52, {1.49000000000069121`\ Second, "It Worked!  "}}, {8, 
        58, {58.8799999999991996`\ Second, Null}}, {9, 
        65, {65.2500000000000035`\ Second, Null}}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(kp = 
      SpecifyExponent[15, 2\^16 - 1, Algorithm \[Rule] GenerateRandomPrime]
        \)], "Input"],

Cell[BoxData[
    \(RSAKey[65535, 34204336104306437695843169482, 
      46876370618283995784041533951]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ct = EncryptMessage["\<It Worked\>", GetPublic[kp]]\)], "Input"],

Cell[BoxData[
    \({16099283158136025415003463751}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[CycleAttack[ct, GetPublic[kp], 1000000]]\)], "Input"],

Cell[BoxData[
    \($Aborted\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(key = 
      RSAKey[65535, 34204336104306437695843169482, 
        46876370618283995784041533951]\)], "Input"],

Cell[BoxData[
    \(RSAKey[65535, 34204336104306437695843169482, 
      46876370618283995784041533951]\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"Microsoft Windows 3.0",
ScreenRectangle->{{0, 1024}, {0, 712}},
WindowSize->{453, 636},
WindowMargins->{{2, Automatic}, {Automatic, 5}}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 24, 0, 105, "Title"],
Cell[1758, 53, 32, 0, 64, "Subtitle"],
Cell[1793, 55, 539, 13, 132, "Text"],
Cell[2335, 70, 69, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[2429, 74, 56, 1, 30, "Input"],
Cell[2488, 77, 74, 1, 25, "Print"],
Cell[2565, 80, 74, 1, 25, "Print"],
Cell[2642, 83, 77, 1, 29, "Output"]
}, Open  ]],
Cell[2734, 87, 68, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[2827, 91, 82, 1, 30, "Input"],
Cell[2912, 94, 77, 1, 29, "Output"]
}, Open  ]],
Cell[3004, 98, 130, 3, 52, "Text"],
Cell[3137, 103, 468, 7, 190, "Input"],
Cell[3608, 112, 61, 0, 33, "Text"],
Cell[3672, 114, 93, 1, 50, "Input"],
Cell[3768, 117, 65, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[3858, 121, 89, 1, 30, "Input"],
Cell[3950, 124, 46, 1, 29, "Output"]
}, Open  ]],
Cell[4011, 128, 251, 4, 71, "Text"],

Cell[CellGroupData[{
Cell[4287, 136, 275, 4, 170, "Input"],
Cell[4565, 142, 173, 3, 63, "Message"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4775, 150, 70, 1, 30, "Input"],
Cell[4848, 153, 38, 1, 29, "Output"]
}, Open  ]],
Cell[4901, 157, 106, 3, 33, "Text"],

Cell[CellGroupData[{
Cell[5032, 164, 81, 1, 30, "Input"],
Cell[5116, 167, 38, 1, 29, "Output"]
}, Open  ]],
Cell[5169, 171, 90, 2, 33, "Text"],
Cell[5262, 175, 423, 6, 230, "Input"],
Cell[5688, 183, 154, 3, 52, "Text"],

Cell[CellGroupData[{
Cell[5867, 190, 83, 1, 30, "Input"],
Cell[5953, 193, 37, 1, 25, "Print"],
Cell[5993, 196, 37, 1, 25, "Print"],
Cell[6033, 199, 37, 1, 25, "Print"],
Cell[6073, 202, 442, 6, 181, "Output"]
}, Open  ]],
Cell[6530, 211, 40, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[6595, 215, 46, 0, 64, "Subtitle"],
Cell[6644, 217, 191, 4, 52, "Text"],
Cell[6838, 223, 655, 15, 151, "Text"],
Cell[7496, 240, 440, 8, 250, "Input"],

Cell[CellGroupData[{
Cell[7961, 252, 34, 1, 30, "Input"],
Cell[7998, 255, 46, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8081, 261, 36, 1, 30, "Input"],
Cell[8120, 264, 77, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8234, 270, 53, 1, 30, "Input"],
Cell[8290, 273, 55, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8382, 279, 58, 1, 30, "Input"],
Cell[8443, 282, 44, 1, 29, "Output"]
}, Open  ]],
Cell[8502, 286, 153, 3, 52, "Text"],
Cell[8658, 291, 182, 3, 110, "Input"],

Cell[CellGroupData[{
Cell[8865, 298, 61, 1, 30, "Input"],
Cell[8929, 301, 70, 1, 29, "Output"]
}, Open  ]],
Cell[9014, 305, 93, 2, 33, "Text"],

Cell[CellGroupData[{
Cell[9132, 311, 48, 1, 30, "Input"],
Cell[9183, 314, 70, 1, 29, "Output"]
}, Open  ]],
Cell[9268, 318, 113, 3, 52, "Text"],
Cell[9384, 323, 723, 13, 310, "Input"],

Cell[CellGroupData[{
Cell[10132, 340, 61, 1, 30, "Input"],
Cell[10196, 343, 175, 3, 25, "Print"],
Cell[10374, 348, 442, 6, 181, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[10865, 360, 29, 0, 64, "Subtitle"],
Cell[10897, 362, 98, 2, 33, "Text"],

Cell[CellGroupData[{
Cell[11020, 368, 57, 1, 30, "Input"],
Cell[11080, 371, 74, 1, 25, "Print"],
Cell[11157, 374, 74, 1, 25, "Print"],
Cell[11234, 377, 74, 1, 29, "Output"]
}, Open  ]],
Cell[11323, 381, 87, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[11435, 386, 74, 1, 30, "Input"],
Cell[11512, 389, 192, 4, 25, "Print"],
Cell[11707, 395, 481, 7, 181, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[12225, 407, 57, 1, 30, "Input"],
Cell[12285, 410, 74, 1, 25, "Print"],
Cell[12362, 413, 74, 1, 25, "Print"],
Cell[12439, 416, 84, 1, 29, "Output"]
}, Open  ]],
Cell[12538, 420, 87, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[12650, 425, 74, 1, 30, "Input"],
Cell[12727, 428, 175, 3, 25, "Print"],
Cell[12905, 433, 70, 1, 25, "Print"],
Cell[12978, 436, 69, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[13084, 442, 82, 1, 30, "Input"],
Cell[13169, 445, 177, 3, 25, "Print"],
Cell[13349, 450, 70, 1, 25, "Print"],
Cell[13422, 453, 70, 1, 29, "Output"]
}, Open  ]],
Cell[13507, 457, 226, 6, 52, "Text"],

Cell[CellGroupData[{
Cell[13758, 467, 99, 2, 50, "Input"],
Cell[13860, 471, 77, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[13974, 477, 72, 1, 30, "Input"],
Cell[14049, 480, 36, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[14122, 486, 72, 1, 30, "Input"],
Cell[14197, 489, 36, 1, 29, "Output"]
}, Open  ]],
Cell[14248, 493, 95, 2, 33, "Text"],
Cell[14346, 497, 87, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[14458, 502, 74, 1, 30, "Input"],
Cell[14535, 505, 192, 4, 25, "Print"],
Cell[14730, 511, 480, 7, 181, "Output"]
}, Open  ]],
Cell[15225, 521, 153, 3, 52, "Text"],

Cell[CellGroupData[{
Cell[15403, 528, 100, 2, 50, "Input"],
Cell[15506, 532, 100, 2, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[15643, 539, 82, 1, 30, "Input"],
Cell[15728, 542, 36, 1, 29, "Output"]
}, Open  ]],
Cell[15779, 546, 87, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[15891, 551, 83, 1, 30, "Input"],
Cell[15977, 554, 202, 4, 25, "Print"],
Cell[16182, 560, 484, 7, 181, "Output"]
}, Open  ]],
Cell[16681, 570, 166, 5, 33, "Text"],

Cell[CellGroupData[{
Cell[16872, 579, 58, 1, 30, "Input"],
Cell[16933, 582, 74, 1, 25, "Print"],
Cell[17010, 585, 74, 1, 25, "Print"],
Cell[17087, 588, 101, 2, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[17225, 595, 82, 1, 30, "Input"],
Cell[17310, 598, 36, 1, 29, "Output"]
}, Open  ]],
Cell[17361, 602, 87, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[17473, 607, 83, 1, 30, "Input"],
Cell[17559, 610, 179, 3, 25, "Print"],
Cell[17741, 615, 70, 1, 25, "Print"],
Cell[17814, 618, 70, 1, 29, "Output"]
}, Open  ]],
Cell[17899, 622, 222, 4, 71, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[18158, 631, 30, 0, 64, "Subtitle"],
Cell[18191, 633, 47, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[18263, 637, 89, 1, 50, "Input"],
Cell[18355, 640, 74, 1, 25, "Print"],
Cell[18432, 643, 74, 1, 25, "Print"],
Cell[18509, 646, 74, 1, 25, "Print"],
Cell[18586, 649, 74, 1, 25, "Print"],
Cell[18663, 652, 74, 1, 25, "Print"],
Cell[18740, 655, 74, 1, 25, "Print"],
Cell[18817, 658, 74, 1, 25, "Print"],
Cell[18894, 661, 74, 1, 25, "Print"],
Cell[18971, 664, 74, 1, 25, "Print"],
Cell[19048, 667, 74, 1, 25, "Print"],
Cell[19125, 670, 74, 1, 25, "Print"],
Cell[19202, 673, 74, 1, 25, "Print"],
Cell[19279, 676, 74, 1, 25, "Print"],
Cell[19356, 679, 74, 1, 25, "Print"],
Cell[19433, 682, 74, 1, 25, "Print"],
Cell[19510, 685, 74, 1, 25, "Print"],
Cell[19587, 688, 285, 4, 124, "Output"]
}, Closed]],
Cell[19887, 695, 86, 2, 30, "Text"],
Cell[19976, 699, 63, 1, 30, "Input"],
Cell[20042, 702, 84, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[20151, 707, 221, 4, 70, "Input"],
Cell[20375, 713, 169, 3, 25, "Print"],
Cell[20547, 718, 237, 6, 25, "Print"],
Cell[20787, 726, 177, 3, 25, "Print"],
Cell[20967, 731, 253, 6, 25, "Print"],
Cell[21223, 739, 181, 3, 25, "Print"],
Cell[21407, 744, 276, 7, 25, "Print"],
Cell[21686, 753, 183, 3, 25, "Print"],
Cell[21872, 758, 292, 7, 25, "Print"],
Cell[22167, 767, 175, 3, 25, "Print"],
Cell[22345, 772, 70, 1, 25, "Print"],
Cell[22418, 775, 175, 3, 25, "Print"],
Cell[22596, 780, 70, 1, 25, "Print"],
Cell[22669, 783, 175, 3, 25, "Print"],
Cell[22847, 788, 70, 1, 25, "Print"],
Cell[22920, 791, 175, 3, 25, "Print"],
Cell[23098, 796, 70, 1, 25, "Print"],
Cell[23171, 799, 459, 8, 143, "Output"]
}, Open  ]],
Cell[23645, 810, 152, 4, 52, "Text"],
Cell[23800, 816, 677, 11, 230, "Input"],

Cell[CellGroupData[{
Cell[24502, 831, 60, 1, 30, "Input"],
Cell[24565, 834, 177, 3, 25, "Print"],
Cell[24745, 839, 257, 6, 25, "Print"],
Cell[25005, 847, 177, 3, 25, "Print"],
Cell[25185, 852, 263, 6, 25, "Print"],
Cell[25451, 860, 181, 3, 25, "Print"],
Cell[25635, 865, 284, 7, 25, "Print"],
Cell[25922, 874, 175, 3, 25, "Print"],
Cell[26100, 879, 70, 1, 25, "Print"],
Cell[26173, 882, 175, 3, 25, "Print"],
Cell[26351, 887, 70, 1, 25, "Print"],
Cell[26424, 890, 198, 4, 25, "Print"],
Cell[26625, 896, 318, 7, 44, "Print"],
Cell[26946, 905, 175, 3, 25, "Print"],
Cell[27124, 910, 70, 1, 25, "Print"],
Cell[27197, 913, 175, 3, 25, "Print"],
Cell[27375, 918, 70, 1, 25, "Print"],
Cell[27448, 921, 175, 3, 25, "Print"],
Cell[27626, 926, 70, 1, 25, "Print"],
Cell[27699, 929, 577, 9, 181, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[28313, 943, 59, 1, 30, "Input"],
Cell[28375, 946, 74, 1, 25, "Print"],
Cell[28452, 949, 74, 1, 25, "Print"],
Cell[28529, 952, 74, 1, 25, "Print"],
Cell[28606, 955, 74, 1, 25, "Print"],
Cell[28683, 958, 74, 1, 25, "Print"],
Cell[28760, 961, 74, 1, 25, "Print"],
Cell[28837, 964, 74, 1, 25, "Print"],
Cell[28914, 967, 74, 1, 25, "Print"],
Cell[28991, 970, 74, 1, 25, "Print"],
Cell[29068, 973, 74, 1, 25, "Print"],
Cell[29145, 976, 74, 1, 25, "Print"],
Cell[29222, 979, 74, 1, 25, "Print"],
Cell[29299, 982, 74, 1, 25, "Print"],
Cell[29376, 985, 74, 1, 25, "Print"],
Cell[29453, 988, 74, 1, 25, "Print"],
Cell[29530, 991, 74, 1, 25, "Print"],
Cell[29607, 994, 74, 1, 25, "Print"],
Cell[29684, 997, 74, 1, 25, "Print"],
Cell[29761, 1000, 173, 3, 25, "Print"],
Cell[29937, 1005, 245, 6, 25, "Print"],
Cell[30185, 1013, 177, 3, 25, "Print"],
Cell[30365, 1018, 255, 6, 25, "Print"],
Cell[30623, 1026, 192, 4, 25, "Print"],
Cell[30818, 1032, 288, 7, 25, "Print"],
Cell[31109, 1041, 175, 3, 25, "Print"],
Cell[31287, 1046, 70, 1, 25, "Print"],
Cell[31360, 1049, 175, 3, 25, "Print"],
Cell[31538, 1054, 70, 1, 25, "Print"],
Cell[31611, 1057, 175, 3, 25, "Print"],
Cell[31789, 1062, 70, 1, 25, "Print"],
Cell[31862, 1065, 175, 3, 25, "Print"],
Cell[32040, 1070, 70, 1, 25, "Print"],
Cell[32113, 1073, 175, 3, 25, "Print"],
Cell[32291, 1078, 70, 1, 25, "Print"],
Cell[32364, 1081, 175, 3, 25, "Print"],
Cell[32542, 1086, 70, 1, 25, "Print"],
Cell[32615, 1089, 560, 9, 181, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[33212, 1103, 60, 1, 30, "Input"],
Cell[33275, 1106, 173, 3, 25, "Print"],
Cell[33451, 1111, 257, 6, 25, "Print"],
Cell[33711, 1119, 179, 3, 25, "Print"],
Cell[33893, 1124, 263, 6, 25, "Print"],
Cell[34159, 1132, 179, 3, 25, "Print"],
Cell[34341, 1137, 284, 7, 25, "Print"],
Cell[34628, 1146, 194, 4, 25, "Print"],
Cell[34825, 1152, 296, 7, 25, "Print"],
Cell[35124, 1161, 194, 4, 25, "Print"],
Cell[35321, 1167, 308, 7, 44, "Print"],
Cell[35632, 1176, 175, 3, 25, "Print"],
Cell[35810, 1181, 70, 1, 25, "Print"],
Cell[35883, 1184, 175, 3, 25, "Print"],
Cell[36061, 1189, 70, 1, 25, "Print"],
Cell[36134, 1192, 432, 7, 143, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[36603, 1204, 123, 3, 52, "Input"],
Cell[36729, 1209, 116, 2, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[36882, 1216, 84, 1, 30, "Input"],
Cell[36969, 1219, 65, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[37071, 1225, 80, 1, 30, "Input"],
Cell[37154, 1228, 42, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[37233, 1234, 130, 3, 50, "Input"],
Cell[37366, 1239, 116, 2, 48, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

