(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     30010,        942]*)
(*NotebookOutlinePosition[     30660,        965]*)
(*  CellTagsIndexPosition[     30616,        961]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Factoring - 7", "Title"],

Cell["Introduction", "Subtitle"],

Cell[TextData[{
  "This notebook contains my implementations of several factoring techniques. \
They are all tied into a final test that switches between them to optimise \
the results. Note that the algorithms are mainly do to Bressoud's ",
  StyleBox["Factorization and Primality Testing",
    FontSlant->"Italic"],
  " as referenced in the report. "
}], "Text"],

Cell[CellGroupData[{

Cell["Trial Division", "Subtitle"],

Cell[TextData[{
  "Tests all the numbers up to ",
  StyleBox["lim",
    FontSlant->"Italic"],
  ". The function does not stop until ",
  StyleBox["lim",
    FontSlant->"Italic"],
  " is reached, and in this way all the small factors are removed."
}], "Text"],

Cell[BoxData[
    \(TrialDivision[n_Integer, lim_:  1000000] := 
      Module[{i, r, j, stop, facs}, \n\t\tr = n; j = 2; 
        stop = Min[Ceiling[\@n], lim]; facs = {}; \n
        \t\t (*Print["\<Trial Division\>"]; *) \n\t\t
        While[j < stop, \n\t\t\t
          For[i = j, i < j + 100000 \[And] i < stop, \(i++\), \n\t\t\t\t
            If[Mod[r, i] == 0, \n\t\t\t\t\tfacs = Insert[facs, i, 1]; \n
              \t\t\t\t\tr = r/i; \(i--\)]]; \n\t\t\tj = j + 100000; \n\t\t\t
          If[PrimeQ[r], \tfacs = Insert[facs, r, 1]; r = 1; Break[]]; \n
          \t\t\t (*Print["\<Passing \>", j]*) \n\t\t]; \n\t\t{r, facs}]\)], 
  "Input"],

Cell[CellGroupData[{

Cell["A Little Test", "Subsubtitle"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Prime[100002]\)], "Input"],

Cell[BoxData[
    \(1299743\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(n = %*3*7\)], "Input"],

Cell[BoxData[
    \(27294603\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(TrialDivision[n, 100000]\)], "Input"],

Cell[BoxData[
    \({1, {1299743, 7, 3}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(n = n*Prime[100003]\)], "Input"],

Cell[BoxData[
    \(35476515079089\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(TrialDivision[n, 100000]\)], "Input"],

Cell[BoxData[
    \({1689357860909, {7, 3}}\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Fermat", "Subtitle"],

Cell["\<\
Here is Fermat's algorithm. It will not be used in the final CompleteFactor \
function.\
\>", "Text"],

Cell[BoxData[
    \(\(FermatFactor[n_] := 
      \(Module[{st, u, v, r, a, b}, \n\t\tst = Ceiling[Sqrt[n]]; \n\t\t
        u = \((2*st)\) + 1; \n\t\tv = 1; \n\t\tr = st\^2 - n; \n\t\t
        While[r \[NotEqual] 0, \n\t\t\t
          If[r > 0, \n\t\t\t\t
            While[r > 0, \n\t\t\t\t\tr = r - v; \n\t\t\t\t\tv = v + 2]]; \n
          \t\t\tIf\ [r < 0, \n\t\t\t\tr = r + u; \n\t\t\t\tu = u + 2]]; \n\t\t
        a = \((u + v - 2)\)/2; \n\t\tb = \((u - v)\)/2; \n\t\t{a, b}]\n\)\t\t
    \)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(FermatFactor[99]\)], "Input"],

Cell[BoxData[
    \({11, 9}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Prime[100]\)], "Input"],

Cell[BoxData[
    \(541\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FermatFactor[1783647329]\)], "Input"],

Cell[BoxData[
    \({84449, 21121}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FermatFactor[Prime[1231]*Prime[2000]]\)], "Input"],

Cell[BoxData[
    \({17389, 10009}\)], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Pollard p-1 Code", "Subtitle"],

Cell["\<\
This test works well if p-1 can be completely factorised with numbers below \
max, and may work well even if several larger factors are also included.\
\>", "Text"],

Cell[BoxData[
    \(PollardP[n_Integer, c_, max_] := 
      Module[{m, i, g, result}, \n\t\tm = c; \n\t\tresult = 1; \n\t\t
        For[i = 1, i \[LessEqual] max, i = i + 1, \n\t\t\t
          m = PowerMod[m, i, n]; \n\t\t\t
          If[Mod[i, 10] == 0, \n\t\t\t\tg = GCD[m - 1, n]; \n\t\t\t\t
            If[g > 1, result = g; i = max]]]; \n\t\tresult]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Pollard Rho Code", "Subtitle"],

Cell[TextData[{
  "Generates a sequence based on the function ",
  Cell[BoxData[
      \(TraditionalForm\`x\^2 + \ c\)]],
  ". Checks pairs of ",
  Cell[BoxData[
      \(TraditionalForm\`x\_i\)]],
  " by looking at the difference. "
}], "Text"],

Cell[BoxData[
    \(PollRho[n_Integer, c_, mx_] := \n\t
      Module[{x1, x2, rng, prod, trms, j}, \n\t\tx1 = 2; x2 = x1\^2 + c; 
        rng = 1; prod = 1; trms = 0; \n\t\t
        Catch[\n\t\t\t
          While[trms \[LessEqual] mx, \n\t\t\t\t
            For[j = 1, j < rng, \(j++\), \n\t\t\t\t\tx2 = Mod[x2\^2 + c, n]; 
              \n\t\t\t\t\tprod = Mod[prod*\((x1 - x2)\), n]; \n\t\t\t\t\t
              \(trms++\); \n\t\t\t\t\t
              If[Mod[trms, Min[10, trms]] == 0, \n\t\t\t\t\t\t
                prod = GCDCheck[n, prod]]]; \n
            \t\t\t\t{x1, x2, rng} = ResetVars[x1, x2, rng, c, n]]; \n\t\t\t
          "\<Enter a new c or longer range\>"]\n\t\t]; \n
    GCDCheck[n_, prd_] := 
      Module[{g}, \n\t\tg = GCD[n, prd]; \n\t\tIf[g > 1, Throw[g]]; \n\t\t1]; 
    \nResetVars[x1_, x2_, rng_, c_, n_] := 
      Module[{x2new, j, rngnew, x1new}, \n\t\tx1new = x2; rngnew = 2*rng; 
        x2new = x2; \n\t\t
        For[j = 1, j \[LessEqual] rng, \(j++\), \n\t\t\t
          x2new = Mod[x2new\^2 + c, n]]; \n\t\t{x1new, x2new, rngnew}]\)], 
  "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Elliptic Curve Code", "Subtitle"],

Cell["\<\
This is a more complicated example. The function NextValues is an optimised \
way of calculating the next x-value of the next point on the curve. By \
generating enough points, given a starting point, we hope to get a factor for \
n.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(NextValues[x_, z_, k_, n_, a_, b_] := \n\t
      Module[{c, len, X1, X2, Z1, Z2, i, u1, u2, temp}, \n\t\t
        c = IntegerDigits[k, 2]; \n\t\tlen = Length[c]; \n\t\tX1 = x; Z1 = z; 
        \n\t\t (*\ Compute\ Loop\ *) \n\t\tX2 = XSub2I[x, z, a, b, n]; \n\t\t
        Z2 = ZSub2I[x, z, a, b, n]; \n\t\t
        For[i = len - 1, i \[GreaterEqual] 1, \(i--\), \n\t\t\t
          u1 = XSub2IPlus1[X1, Z1, X2, Z2, a, b, z, n]; \n\t\t\t
          u2 = ZSub2IPlus1[X1, Z1, X2, Z2, x, n]; \n\t\t\t
          If[c\[LeftDoubleBracket]i\[RightDoubleBracket] == 0, \n\t\t\t\t
            temp = XSub2I[X1, Z1, a, b, n]; \n\t\t\t\t
            Z1 = ZSub2I[X1, Z1, a, b, n]; \n\t\t\t\tX1 = temp; X2 = u1; 
            Z2 = u2, \n\t\t\t\t (*\ else\ do\ *) \n\t\t\t\t
            temp = XSub2I[X2, Z2, a, b, n]; \n\t\t\t\t
            Z2 = ZSub2I[X2, Z2, a, b, n]; \n\t\t\t\tX2 = temp; X1 = u1; 
            Z1 = u2]]; \n\t\t{X1, Z1}]\)], "Input"],

Cell[BoxData[
    \(General::"spell1" \( : \ \) 
      "Possible spelling error: new symbol name \"\!\(ZSub2I\)\" is similar \
to existing symbol \"\!\(XSub2I\)\"."\)], "Message"],

Cell[BoxData[
    \(General::"spell1" \( : \ \) 
      "Possible spelling error: new symbol name \"\!\(ZSub2IPlus1\)\" is \
similar to existing symbol \"\!\(XSub2IPlus1\)\"."\)], "Message"]
}, Closed]],

Cell[BoxData[
    \(XSub2I[r_, s_, a_, b_, n_] := 
      Module[{term}, \n\t\tterm = Mod[r\^2 - a*s\^2, n]; \n\t\t
        Mod[term\^2 - 8*b*r*s\^3, n]]\)], "Input"],

Cell[BoxData[
    \(ZSub2I[r_, s_, a_, b_, n_] := 
      Module[{term}, \n\t\tterm = Mod[r\^3 + a*r*s\^2 + b*s\^3, n]; \n\t\t
        Mod[4*s*term, n]]\)], "Input"],

Cell[BoxData[
    \(XSub2IPlus1[r_, s_, u_, v_, a_, b_, z_, n_] := 
      Module[{term1, term2}, \n\t\tterm1 = Mod[r*u - a*s*v, n]; \n\t\t
        term2 = Mod[b*s*v*\((r*v + s*u)\), n]; \n\t\t
        Mod[z \((term1\^2 - 4*term2)\), n]]\)], "Input"],

Cell[BoxData[
    \(ZSub2IPlus1[r_, s_, u_, v_, x_, n_] := 
      Module[{term}, \n\t\tterm = Mod[u*s - r*v, n]; \n\t\tMod[x*term\^2, n]]
        \)], "Input"],

Cell[BoxData[
    \(GetEquation[EC[a_, b_]] := x\^3 + a\ x + b\  - y\^2 == 0\)], "Input"],

Cell[BoxData[
    \(ECM[n_, X_, y_, a_, max_] := 
      Module[{b, g, z, k, i, x}, \n\t\tx = X; \n\t\t
        b = Mod[y\^2 - x\^3 - a*x, n]; \n
        \t\t (*Print["\<Trying curve:\>", GetEquation[EC[a, b]]]; *) \n\t\t
        g = GCD[4*a\^3 + 27*b\^2, n]; \n\t\tIf[g \[NotEqual] 1, Return[g]]; \n
        \t\tz = 1; k = 2; \n\t\t
        While[k \[LessEqual] max, \n\t\t\t
          For[i = 1, i \[LessEqual] 10, \(i++\), \n
            \t\t\t\t{x, z} = NextValues[x, z, k, n, a, b]; \n\t\t\t\t
            \(k++\)]; \n\t\t\tg = GCD[z, n]; \n\t\t\t
          If[g \[NotEqual] 1, Break[]]]; \n\t\tg]\)], "Input"],

Cell[BoxData[
    \(FactorECM[n_, max_, curves_] := 
      Module[{x, y, a, g, i}, \n\t\tg = 1; \n\t\ti = 0; \n\t\t
        While[\((g \[Equal] 1 \[Or] g \[Equal] n)\) \[And] i < curves, \n
          \t\t\ta = Random[Integer, {1 - n, n - 1}]; \n\t\t\t
          x = Random[Integer, {1, n - 1}]; \n\t\t\t
          y = Random[Integer, {1, n - 1}]; \n\t\t\tg = ECM[n, x, y, a, max]; 
          \n\t\t\t\(i++\)\n\t\t]; \n\t\tg]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Tying it all together", "Subtitle"],

Cell["\<\
The complete factor method first tries to use Trial Division and then \
completes several rounds using Pollard p-1, rho and ECM. Note that the \
function is recursive in that when a factor is found, the remaining nonprime \
factors are passed into the function again.\
\>", "Text"],

Cell[BoxData[
    \(CompleteFactor[n_] := 
      Module[{r, facs}, \n\t\t{r, facs} = TrialDivision[n, 300000]; \n\t\t
        If[Length[facs] \[NotEqual] 0, Print[facs]]; \n\t\t
        CompleteFactor[r, facs, 1]]; \n
    CompleteFactor[n_, factors_, itstart_] := \n\t
      Module[{p, facs, i}, \n\t\tfacs = factors; \n\t\ti = itstart; \n
        \t\t{p, i} = Factorize[n, i]; \n\t\t
        If[p == 1 \[Or] p == n, Return[facs]]; \n
        \t\t (*Print[p, "\<, \>", n/p]; *) \n\t\tq = n/p; \n\t\t
        If[PrimeQ[q], facs = Insert[facs, q, 1], 
          facs = CompleteFactor[q, facs, i]]; \n\t\t
        If[PrimeQ[p], facs = Insert[facs, p, 1], 
          facs = CompleteFactor[p, facs, i]]; \n\t\tfacs]\)], "Input"],

Cell[BoxData[
    \(Factorize[n_, startI_, iterations_:  3] := 
      Module[{p, i}, \n\t\t
        For[i = startI, i < iterations, \(i++\), \n\t\t\t
          If[n == 1, Return[{1, startI}]]; \n\t\t\t\n\t\t\t
          p = PollardP[n, Prime[i], 10000]; \n\t\t\t
          If[p > 1 \[And] p \[NotEqual] n, \n\t\t\t\t
            Print["\<Pollard p-1 (\>", Prime[i], "\<): \>", p]; \n\t\t\t\t
            Return[{p, i}]]; \n\n\t\t\tp = PollRho[n, Prime[i], 10000]; \n
          \t\t\tIf[p > 1 \[And] p \[NotEqual] n, \n\t\t\t\t
            Print["\<Pollard Rho(\>", Prime[i], "\<):\>", p]; \n\t\t\t\t
            Return[{p, i}]]; \t\t\t\n\t\t\tSeedRandom[101*Prime[i]]; \n\t\t\t
          p = FactorECM[n, 500, 1]\n\t\t\t
              If[p > 1 \[And] p \[NotEqual] n, \n\t\t\t\t
                Print["\<EC: \>", p]; \n\t\t\t\tReturn[{p, i}]]]; \t\t\t\t\t\t
        \n\t\t{p, iterations}]\)], "Input"],

Cell[BoxData[
    \(Clear[Factorize]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Tests", "Subtitle"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CompleteFactor[21]\)], "Input"],

Cell[BoxData[
    \("Trial Division"\)], "Print"],

Cell[BoxData[
    \({7, 3}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(CompleteFactor[63]\)], "Input"],

Cell[BoxData[
    \("Trial Division"\)], "Print"],

Cell[BoxData[
    InterpretationBox[\("Passing "\[InvisibleSpace]100002\),
      SequenceForm[ "Passing ", 100002],
      Editable->False]], "Print"],

Cell[BoxData[
    \({7, 3, 3}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(CompleteFactor[232]\)], "Input"],

Cell[BoxData[
    \("Trial Division"\)], "Print"],

Cell[BoxData[
    \({29, 2, 2, 2}\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Partition Numbers", "Subtitle"],

Cell["\<\
I demonstrate the program with some of the partition numbers supplied by RSA.\
\
\>", "Text"],

Cell[CellGroupData[{

Cell["p(9457)", "Subsubtitle"],

Cell[CellGroupData[{

Cell[BoxData[
    \(p9457 = 
      106152954147973372212569591652169182833017044403767639494262158461323654\
902767621593177028579512238092474\)], "Input"],

Cell[BoxData[
    \(106152954147973372212569591652169182833017044403767639494262158461323654\
902767621593177028579512238092474\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(CompleteFactor[p9457]\)], "Input"],

Cell[BoxData[
    \("Trial Division"\)], "Print"],

Cell[BoxData[
    InterpretationBox[\("Passing "\[InvisibleSpace]100002\),
      SequenceForm[ "Passing ", 100002],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Passing "\[InvisibleSpace]200002\),
      SequenceForm[ "Passing ", 200002],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[
      \("Pollard p-1 and Rho with \
n="\[InvisibleSpace]\
176921590246622287020949319420281971388361740672946065823770264102206091504612\
70265529504763252039682079\[InvisibleSpace]" and with c="\[InvisibleSpace]1
        \),
      SequenceForm[ 
      "Pollard p-1 and Rho with n=", 
        1769215902466222870209493194202819713883617406729460658237702641022060\
9150461270265529504763252039682079, " and with c=", 1],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[
      \(4789513\[InvisibleSpace]", \
"\[InvisibleSpace]\
369393694612839107067773528165143243975664625344885932711259504050215734887059\
9216565338639492583\),
      SequenceForm[ 
      4789513, ", ", 
        3693936946128391070677735281651432439756646253448859327112595040502157\
348870599216565338639492583],
      Editable->False]], "Print"],

Cell[BoxData[
    \({4789513, 
      369393694612839107067773528165143243975664625344885932711259504050215734\
8870599216565338639492583, 3, 2}\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["p(8681)", "Subsubtitle"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(p8681 = 
      103044421462757268163291405934796897126888523070780442843814287923536390\
5015759455268038265724661691\ \)\)], "Input"],

Cell[BoxData[
    \(103044421462757268163291405934796897126888523070780442843814287923536390\
5015759455268038265724661691\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(CompleteFactor[p8681]\)], "Input"],

Cell[BoxData[
    \("Trial Division"\)], "Print"],

Cell[BoxData[
    InterpretationBox[\("Passing "\[InvisibleSpace]100002\),
      SequenceForm[ "Passing ", 100002],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Passing "\[InvisibleSpace]200002\),
      SequenceForm[ "Passing ", 200002],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[
      \("Pollard p-1 and Rho with \
n="\[InvisibleSpace]\
854218863158064065019409814596674932660934453044685756808540893007845399167503\
48608806952310757\[InvisibleSpace]" and with c="\[InvisibleSpace]1\),
      SequenceForm[ 
      "Pollard p-1 and Rho with n=", 
        8542188631580640650194098145966749326609344530446857568085408930078453\
9916750348608806952310757, " and with c=", 1],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[
      \(7876147\[InvisibleSpace]", \
"\[InvisibleSpace]\
108456439824963153305722939731403557178520722511233698000880493089812239305272\
42395146631\),
      SequenceForm[ 
      7876147, ", ", 
        1084564398249631533057229397314035571785207225112336980008804930898122\
3930527242395146631],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[
      \("Pollard p-1 and Rho with \
n="\[InvisibleSpace]\
108456439824963153305722939731403557178520722511233698000880493089812239305272\
42395146631\[InvisibleSpace]" and with c="\[InvisibleSpace]1\),
      SequenceForm[ 
      "Pollard p-1 and Rho with n=", 
        1084564398249631533057229397314035571785207225112336980008804930898122\
3930527242395146631, " and with c=", 1],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[
      \("Trying \
curve:"\[InvisibleSpace]\(\
443482749113062064151474491583633468896866888350005089182178360777831462494774\
3164341836 + 
            367764157002971473965312067102041865587742127358978985756026355109\
0362770318875548169724\ x + x\^3 - y\^2 == 0\)\),
      SequenceForm[ "Trying curve:", 
        Equal[ 
          Plus[ 
          44348274911306206415147449158363346889686688835000508918217836077783\
14624947743164341836, 
            Times[ 
            367764157002971473965312067102041865587742127358978985756026355109\
0362770318875548169724, x], 
            Power[ x, 3], 
            Times[ -1, 
              Power[ y, 2]]], 0]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[
      \("Pollard p-1 and Rho with \
n="\[InvisibleSpace]\
108456439824963153305722939731403557178520722511233698000880493089812239305272\
42395146631\[InvisibleSpace]" and with c="\[InvisibleSpace]2\),
      SequenceForm[ 
      "Pollard p-1 and Rho with n=", 
        1084564398249631533057229397314035571785207225112336980008804930898122\
3930527242395146631, " and with c=", 2],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[
      \(76181269
        \[InvisibleSpace]", \
"\[InvisibleSpace]\
142366281434565172845470636268087838204061319208575664446965950501313176215629\
099\),
      SequenceForm[ 
      76181269, ", ", 
        1423662814345651728454706362680878382040613192085756644469659505013131\
76215629099],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[
      \("Pollard p-1 and Rho with \
n="\[InvisibleSpace]\
142366281434565172845470636268087838204061319208575664446965950501313176215629\
099\[InvisibleSpace]" and with c="\[InvisibleSpace]1\),
      SequenceForm[ 
      "Pollard p-1 and Rho with n=", 
        1423662814345651728454706362680878382040613192085756644469659505013131\
76215629099, " and with c=", 1],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[
      \("Trying \
curve:"\[InvisibleSpace]\(\
709140289352225819801422992376218000126423108249460551432738644377741234923726\
11 - 3150376759881888637127683273019143130115821891748381254727231309472241294\
2750907\ x + x\^3 - y\^2 == 0\)\),
      SequenceForm[ "Trying curve:", 
        Equal[ 
          Plus[ 
          70914028935222581980142299237621800012642310824946055143273864437774\
123492372611, 
            
            Times[ -3150376759881888637127683273019143130115821891748381254727\
2313094722412942750907, x], 
            Power[ x, 3], 
            Times[ -1, 
              Power[ y, 2]]], 0]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[
      \("Pollard p-1 and Rho with \
n="\[InvisibleSpace]\
142366281434565172845470636268087838204061319208575664446965950501313176215629\
099\[InvisibleSpace]" and with c="\[InvisibleSpace]2\),
      SequenceForm[ 
      "Pollard p-1 and Rho with n=", 
        1423662814345651728454706362680878382040613192085756644469659505013131\
76215629099, " and with c=", 2],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[
      \("Trying \
curve:"\[InvisibleSpace]\(\
468447477821326391453520327745629926228529799184882145733617677606264777598316\
89 + 9164584154245832816210295944812006242690487562461232566803132023567977569\
7002460\ x + x\^3 - y\^2 == 0\)\),
      SequenceForm[ "Trying curve:", 
        Equal[ 
          Plus[ 
          46844747782132639145352032774562992622852979918488214573361767760626\
477759831689, 
            Times[ 
            916458415424583281621029594481200624269048756246123256680313202356\
79775697002460, x], 
            Power[ x, 3], 
            Times[ -1, 
              Power[ y, 2]]], 0]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[
      \("Pollard p-1 and Rho with \
n="\[InvisibleSpace]\
142366281434565172845470636268087838204061319208575664446965950501313176215629\
099\[InvisibleSpace]" and with c="\[InvisibleSpace]3\),
      SequenceForm[ 
      "Pollard p-1 and Rho with n=", 
        1423662814345651728454706362680878382040613192085756644469659505013131\
76215629099, " and with c=", 3],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[
      \("Trying \
curve:"\[InvisibleSpace]\(\
185605899021243506102751527558238297013608688292690894073775032727052090764624\
48 - 1388726634051369234725646904857802007800966356427988735094896373655995030\
11497617\ x + x\^3 - y\^2 == 0\)\),
      SequenceForm[ "Trying curve:", 
        Equal[ 
          Plus[ 
          18560589902124350610275152755823829701360868829269089407377503272705\
209076462448, 
            
            Times[ -1388726634051369234725646904857802007800966356427988735094\
89637365599503011497617, x], 
            Power[ x, 3], 
            Times[ -1, 
              Power[ y, 2]]], 0]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[
      \("Pollard p-1 and Rho with \
n="\[InvisibleSpace]\
142366281434565172845470636268087838204061319208575664446965950501313176215629\
099\[InvisibleSpace]" and with c="\[InvisibleSpace]4\),
      SequenceForm[ 
      "Pollard p-1 and Rho with n=", 
        1423662814345651728454706362680878382040613192085756644469659505013131\
76215629099, " and with c=", 4],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[
      \("Trying \
curve:"\[InvisibleSpace]\(\
964137873273667170513436932035406372240661912643142846989825281065377461165036\
71 + 1224922481893877743347061453104435962812981362223054184228364947246472663\
27959729\ x + x\^3 - y\^2 == 0\)\),
      SequenceForm[ "Trying curve:", 
        Equal[ 
          Plus[ 
          96413787327366717051343693203540637224066191264314284698982528106537\
746116503671, 
            Times[ 
            122492248189387774334706145310443596281298136222305418422836494724\
647266327959729, x], 
            Power[ x, 3], 
            Times[ -1, 
              Power[ y, 2]]], 0]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[
      \("Pollard p-1 and Rho with \
n="\[InvisibleSpace]\
142366281434565172845470636268087838204061319208575664446965950501313176215629\
099\[InvisibleSpace]" and with c="\[InvisibleSpace]5\),
      SequenceForm[ 
      "Pollard p-1 and Rho with n=", 
        1423662814345651728454706362680878382040613192085756644469659505013131\
76215629099, " and with c=", 5],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[
      \("Trying \
curve:"\[InvisibleSpace]\(\
494649571489368262889310568917186427654278651113482438750323967699491308505181\
57 - 3432771742742069298385744834188112074558251512643724249099373264298442543\
6788361\ x + x\^3 - y\^2 == 0\)\),
      SequenceForm[ "Trying curve:", 
        Equal[ 
          Plus[ 
          49464957148936826288931056891718642765427865111348243875032396769949\
130850518157, 
            
            Times[ -3432771742742069298385744834188112074558251512643724249099\
3732642984425436788361, x], 
            Power[ x, 3], 
            Times[ -1, 
              Power[ y, 2]]], 0]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[
      \("Pollard p-1 and Rho with \
n="\[InvisibleSpace]\
142366281434565172845470636268087838204061319208575664446965950501313176215629\
099\[InvisibleSpace]" and with c="\[InvisibleSpace]6\),
      SequenceForm[ 
      "Pollard p-1 and Rho with n=", 
        1423662814345651728454706362680878382040613192085756644469659505013131\
76215629099, " and with c=", 6],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[
      \("Trying \
curve:"\[InvisibleSpace]\(\
114227808987889282266709685200132959954788066787116930047972139079082544391180\
807 - 101650101613300042063226849945006937630062890055927556400846425360536216\
870257568\ x + x\^3 - y\^2 == 0\)\),
      SequenceForm[ "Trying curve:", 
        Equal[ 
          Plus[ 
          11422780898788928226670968520013295995478806678711693004797213907908\
2544391180807, 
            
            Times[ -1016501016133000420632268499450069376300628900559275564008\
46425360536216870257568, x], 
            Power[ x, 3], 
            Times[ -1, 
              Power[ y, 2]]], 0]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[
      \("Pollard p-1 and Rho with \
n="\[InvisibleSpace]\
142366281434565172845470636268087838204061319208575664446965950501313176215629\
099\[InvisibleSpace]" and with c="\[InvisibleSpace]7\),
      SequenceForm[ 
      "Pollard p-1 and Rho with n=", 
        1423662814345651728454706362680878382040613192085756644469659505013131\
76215629099, " and with c=", 7],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[
      \("Trying \
curve:"\[InvisibleSpace]\(\
730547704917617568978954766937260449376177032131339106265268416468051016743482\
0 - 79209821855374446712293892180461937251033823531636373891010177879279603419\
346620\ x + x\^3 - y\^2 == 0\)\),
      SequenceForm[ "Trying curve:", 
        Equal[ 
          Plus[ 
          73054770491761756897895476693726044937617703213133910626526841646805\
10167434820, 
            
            Times[ -7920982185537444671229389218046193725103382353163637389101\
0177879279603419346620, x], 
            Power[ x, 3], 
            Times[ -1, 
              Power[ y, 2]]], 0]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[
      \("Pollard p-1 and Rho with \
n="\[InvisibleSpace]\
142366281434565172845470636268087838204061319208575664446965950501313176215629\
099\[InvisibleSpace]" and with c="\[InvisibleSpace]8\),
      SequenceForm[ 
      "Pollard p-1 and Rho with n=", 
        1423662814345651728454706362680878382040613192085756644469659505013131\
76215629099, " and with c=", 8],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[
      \("Trying \
curve:"\[InvisibleSpace]\(\
895034360563605130633360275847978268781053523106403477845504495644717004023035\
98 - 1302816652553098844293905104011053553774473406010141113476960132499094276\
99773480\ x + x\^3 - y\^2 == 0\)\),
      SequenceForm[ "Trying curve:", 
        Equal[ 
          Plus[ 
          89503436056360513063336027584797826878105352310640347784550449564471\
700402303598, 
            
            Times[ -1302816652553098844293905104011053553774473406010141113476\
96013249909427699773480, x], 
            Power[ x, 3], 
            Times[ -1, 
              Power[ y, 2]]], 0]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[
      \("Pollard p-1 and Rho with \
n="\[InvisibleSpace]\
142366281434565172845470636268087838204061319208575664446965950501313176215629\
099\[InvisibleSpace]" and with c="\[InvisibleSpace]9\),
      SequenceForm[ 
      "Pollard p-1 and Rho with n=", 
        1423662814345651728454706362680878382040613192085756644469659505013131\
76215629099, " and with c=", 9],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[
      \("Trying \
curve:"\[InvisibleSpace]\(\
799049104644120329178774663325683766674932196680498795756635844088471292778568\
99 + 4417733793757034108360478781201455329715350071505118402517146904776678224\
1391977\ x + x\^3 - y\^2 == 0\)\),
      SequenceForm[ "Trying curve:", 
        Equal[ 
          Plus[ 
          79904910464412032917877466332568376667493219668049879575663584408847\
129277856899, 
            Times[ 
            441773379375703410836047878120145532971535007150511840251714690477\
66782241391977, x], 
            Power[ x, 3], 
            Times[ -1, 
              Power[ y, 2]]], 0]],
      Editable->False]], "Print"],

Cell[BoxData[
    \({7876147, 76181269, 4021, 3}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(facs = {7876147, 76181269, 4021, 3}\)], "Input"],

Cell[BoxData[
    \({7876147, 76181269, 4021, 3}\)], "Output"]
}, Open  ]],

Cell["What's left over.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(p8681/Fold[Times, 1, facs]\)], "Input"],

Cell[BoxData[
    \(142366281434565172845470636268087838204061319208575664446965950501313176\
215629099\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]]
}, Open  ]]
},
FrontEndVersion->"Microsoft Windows 3.0",
ScreenRectangle->{{0, 1024}, {0, 740}},
WindowSize->{528, 683},
WindowMargins->{{28, Automatic}, {Automatic, 5}}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 30, 0, 105, "Title"],
Cell[1764, 53, 32, 0, 64, "Subtitle"],
Cell[1799, 55, 364, 7, 71, "Text"],

Cell[CellGroupData[{
Cell[2188, 66, 34, 0, 64, "Subtitle"],
Cell[2225, 68, 258, 8, 52, "Text"],
Cell[2486, 78, 645, 11, 295, "Input"],

Cell[CellGroupData[{
Cell[3156, 93, 36, 0, 53, "Subsubtitle"],

Cell[CellGroupData[{
Cell[3217, 97, 46, 1, 30, "Input"],
Cell[3266, 100, 41, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3344, 106, 42, 1, 30, "Input"],
Cell[3389, 109, 42, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3468, 115, 57, 1, 30, "Input"],
Cell[3528, 118, 54, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3619, 124, 52, 1, 30, "Input"],
Cell[3674, 127, 48, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3759, 133, 57, 1, 30, "Input"],
Cell[3819, 136, 57, 1, 29, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[3937, 144, 26, 0, 53, "Subtitle"],
Cell[3966, 146, 111, 3, 33, "Text"],
Cell[4080, 151, 507, 9, 352, "Input"],

Cell[CellGroupData[{
Cell[4612, 164, 49, 1, 30, "Input"],
Cell[4664, 167, 41, 1, 70, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4742, 173, 43, 1, 30, "Input"],
Cell[4788, 176, 37, 1, 70, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4862, 182, 57, 1, 30, "Input"],
Cell[4922, 185, 48, 1, 70, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5007, 191, 70, 1, 30, "Input"],
Cell[5080, 194, 48, 1, 70, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[5177, 201, 36, 0, 53, "Subtitle"],
Cell[5216, 203, 174, 3, 52, "Text"],
Cell[5393, 208, 367, 6, 190, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[5797, 219, 36, 0, 53, "Subtitle"],
Cell[5836, 221, 244, 8, 52, "Text"],
Cell[6083, 231, 1076, 20, 503, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7196, 256, 39, 0, 64, "Subtitle"],
Cell[7238, 258, 257, 5, 71, "Text"],

Cell[CellGroupData[{
Cell[7520, 267, 955, 15, 410, "Input"],
Cell[8478, 284, 179, 3, 44, "Message"],
Cell[8660, 289, 189, 3, 63, "Message"]
}, Closed]],
Cell[8864, 295, 165, 3, 70, "Input"],
Cell[9032, 300, 164, 3, 73, "Input"],
Cell[9199, 305, 249, 4, 112, "Input"],
Cell[9451, 311, 159, 3, 72, "Input"],
Cell[9613, 316, 89, 1, 30, "Input"],
Cell[9705, 319, 614, 11, 294, "Input"],
Cell[10322, 332, 437, 7, 230, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10796, 344, 41, 0, 64, "Subtitle"],
Cell[10840, 346, 291, 5, 71, "Text"],
Cell[11134, 353, 723, 13, 350, "Input"],
Cell[11860, 368, 901, 15, 390, "Input"],
Cell[12764, 385, 49, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[12850, 391, 25, 0, 64, "Subtitle"],

Cell[CellGroupData[{
Cell[12900, 395, 51, 1, 30, "Input"],
Cell[12954, 398, 49, 1, 25, "Print"],
Cell[13006, 401, 40, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[13083, 407, 51, 1, 30, "Input"],
Cell[13137, 410, 49, 1, 25, "Print"],
Cell[13189, 413, 149, 3, 25, "Print"],
Cell[13341, 418, 43, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[13421, 424, 52, 1, 30, "Input"],
Cell[13476, 427, 49, 1, 25, "Print"],
Cell[13528, 430, 47, 1, 29, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[13624, 437, 37, 0, 64, "Subtitle"],
Cell[13664, 439, 103, 3, 33, "Text"],

Cell[CellGroupData[{
Cell[13792, 446, 30, 0, 53, "Subsubtitle"],

Cell[CellGroupData[{
Cell[13847, 450, 155, 3, 70, "Input"],
Cell[14005, 455, 141, 2, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[14183, 462, 54, 1, 30, "Input"],
Cell[14240, 465, 49, 1, 25, "Print"],
Cell[14292, 468, 149, 3, 25, "Print"],
Cell[14444, 473, 149, 3, 25, "Print"],
Cell[14596, 478, 491, 11, 82, "Print"],
Cell[15090, 491, 383, 10, 63, "Print"],
Cell[15476, 503, 157, 3, 67, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[15682, 512, 30, 0, 53, "Subsubtitle"],

Cell[CellGroupData[{
Cell[15737, 516, 156, 3, 70, "Input"],
Cell[15896, 521, 136, 2, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[16069, 528, 54, 1, 30, "Input"],
Cell[16126, 531, 49, 1, 25, "Print"],
Cell[16178, 534, 149, 3, 25, "Print"],
Cell[16330, 539, 149, 3, 25, "Print"],
Cell[16482, 544, 464, 10, 63, "Print"],
Cell[16949, 556, 367, 10, 63, "Print"],
Cell[17319, 568, 452, 10, 63, "Print"],
Cell[17774, 580, 729, 19, 101, "Print"],
Cell[18506, 601, 452, 10, 63, "Print"],
Cell[18961, 613, 362, 11, 44, "Print"],
Cell[19326, 626, 436, 10, 63, "Print"],
Cell[19765, 638, 685, 18, 101, "Print"],
Cell[20453, 658, 436, 10, 63, "Print"],
Cell[20892, 670, 684, 18, 101, "Print"],
Cell[21579, 690, 436, 10, 63, "Print"],
Cell[22018, 702, 687, 18, 101, "Print"],
Cell[22708, 722, 436, 10, 63, "Print"],
Cell[23147, 734, 686, 18, 101, "Print"],
Cell[23836, 754, 436, 10, 63, "Print"],
Cell[24275, 766, 685, 18, 101, "Print"],
Cell[24963, 786, 436, 10, 63, "Print"],
Cell[25402, 798, 689, 18, 101, "Print"],
Cell[26094, 818, 436, 10, 63, "Print"],
Cell[26533, 830, 683, 18, 120, "Print"],
Cell[27219, 850, 436, 10, 63, "Print"],
Cell[27658, 862, 687, 18, 120, "Print"],
Cell[28348, 882, 436, 10, 63, "Print"],
Cell[28787, 894, 684, 18, 120, "Print"],
Cell[29474, 914, 62, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[29573, 920, 68, 1, 30, "Input"],
Cell[29644, 923, 62, 1, 29, "Output"]
}, Open  ]],
Cell[29721, 927, 33, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[29779, 931, 59, 1, 30, "Input"],
Cell[29841, 934, 117, 2, 48, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

