(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     14042,        502]*)
(*NotebookOutlinePosition[     14694,        525]*)
(*  CellTagsIndexPosition[     14650,        521]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["RSA - 8", "Title"],

Cell["Faulty RSA Encryption", "Subtitle"],

Cell["\<\
This notebook is based on a paper by M. Joye and J.-J. Quisquater.  See the \
report for the complete reference.\
\>", "Text"],

Cell[TextData[{
  "The basic idea is to change the encryption exponent with a single bit \
flip.  We can denote the changed encryption exponent as ",
  StyleBox["\[EHat]",
    FontSlant->"Italic"],
  ".  The paper says it like this \n\t\[EHat] = e + ",
  Cell[BoxData[
      \(TraditionalForm\`2\^j\)]],
  ", if ",
  Cell[BoxData[
      \(TraditionalForm\`e\_j = 0\)]],
  ", and \[EHat] = e - ",
  Cell[BoxData[
      \(TraditionalForm\`2\^j\)]],
  " if ",
  Cell[BoxData[
      \(TraditionalForm\`e\_j = 0. \)]],
  "\nBut expressing it as a bitwise XOR with ",
  Cell[BoxData[
      \(TraditionalForm\`2\^j\)]],
  " is more simple."
}], "Text"],

Cell[CellGroupData[{

Cell["Flipping a bit", "Subtitle"],

Cell["\<\
We will write a function that randomly changes a bit of the given argument.  \
The first thing we need is to count the number of binary digits.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(IntegerDigits[123, 2]\)], "Input"],

Cell[BoxData[
    \({1, 1, 1, 1, 0, 1, 1}\)], "Output"]
}, Open  ]],

Cell["I programmed the following function when in the college labs", "Text"],

Cell[BoxData[
    \(BitFlip[n_] := 
      Module[{bitCount, randBit}, 
        \[IndentingNewLine]bitCount = Fold[Plus, 0, DigitCount[n, 2]]; 
        \[IndentingNewLine]randBit = Random[Integer, {0, bitCount - 1}]; 
        \[IndentingNewLine]BitXor[n, 2\^randBit]]\)], "Input"],

Cell[TextData[{
  "But then realised that the BitXor and DigitCount functions were not \
available in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " 3.0, which is the one that I use on my personal computer.  So I had to \
write the following instead. It does not look nearly as nice."
}], "Text"],

Cell["Wrap it all up:", "Text"],

Cell[BoxData[
    \(BitFlip[n_] := 
      Module[{bitCount, randBit, binForm, bit}, 
        \[IndentingNewLine]bitCount = Length[IntegerDigits[n, 2]]; 
        \[IndentingNewLine]randBit = Random[Integer, {0, bitCount - 1}]; \n
        \t\tbinForm = IntegerDigits[n, 2]; \n\t\tbit = 1; 
        \[IndentingNewLine]If[
          binForm\[LeftDoubleBracket]randBit\[RightDoubleBracket] == 1, 
          bit = 0]; \n\t\t
        FromDigits[ReplacePart[binForm, bit, randBit + 1], 2]]\)], "Input"],

Cell["We can see it now flipping a bit.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(IntegerDigits[BitFlip[123], 2]\)], "Input"],

Cell[BoxData[
    \({1, 1, 1, 1, 0, 1, 0}\)], "Output"]
}, Open  ]],

Cell["Try again.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(IntegerDigits[BitFlip[123], 2]\)], "Input"],

Cell[BoxData[
    \({1, 0, 1, 1, 0, 1, 1}\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Creating a faulty key", "Subtitle"],

Cell["\<\
The next step is to read in  public key and return a faulty one.\
\>", "Text"],

Cell[BoxData[
    \(GetFaultyKey[Key[e_, n_]] := 
      Module[{faulty}, \[IndentingNewLine]\[EHat] = BitFlip[e]; 
        \[IndentingNewLine]Key[\[EHat], n]]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Get a sample key", "Subtitle"],

Cell[BoxData[
    \(\(kp = GenerateRSAKey[100]; \)\)], "Input"],

Cell[BoxData[
    \(\(pub = GetPublic[kp]; \)\)], "Input"],

Cell[BoxData[
    \(\(faulty = GetFaultyKey[pub]; \)\)], "Input"],

Cell[BoxData[
    \(\(n = GetModulus[kp]; \)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(e1 = GetExponent[pub]\)], "Input"],

Cell[BoxData[
    \(232805222302524252615071155651192574538504452088719449359934218261888799\
75864947993809070674158671504430303026397\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(e2 = GetExponent[faulty]\)], "Input"],

Cell[BoxData[
    \(232805222302524252615071155806606085916562777762276401905815730793281346\
88282064163823569952069905786071971011805\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(e1 == e2\)], "Input"],

Cell[BoxData[
    \(False\)], "Output"]
}, Open  ]],

Cell["Repeat until", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(GCD[e1, e2]\)], "Input"],

Cell[BoxData[
    \(1\)], "Output"]
}, Open  ]],

Cell["\<\
Notice that the faulty key is similar to the original.  The first 10 or so \
digits match.  We could ensure that this is always so by not filpping the \
high order bits. This way there is less chance that the altered key will be \
noticed\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["The Attack", "Subtitle"],

Cell["\<\
We now have two exponents and one modulus.  If the same message can be \
encrypted twice by each of these exponents, then we can break the system by \
employing the common modulus attack.\
\>", "Text"],

Cell[TextData[{
  "Let ",
  Cell[BoxData[
      \(TraditionalForm\`c\_1 = \(m\^e\)(mod\ n)\)]],
  ", and ",
  Cell[BoxData[
      \(TraditionalForm\`c\_2 = \(m\^\[EHat]\)(mod\ n)\)]],
  ".  We can usually be sure that gcd(e,\[EHat])=1 (and can change \[EHat] \
until this is true).  This means that there exist u, v \[Epsilon]\
\[DoubleStruckCapitalZ] such that \n\t",
  Cell[BoxData[
      \(TraditionalForm\`u\ e + v\ \[EHat] = 1\)]],
  "\nTherefore, message m is recovered as\n\t",
  Cell[BoxData[
      \(TraditionalForm
      \`m = m\^\(u\ e + v\ \[EHat]\) \[Congruent] 
          \(c\_1\^u\) \(\(c\_2\^v\)(mod\ n)\)\)]]
}], "Text"],

Cell["Now get one message and two ciphertexts.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(m = Random[Integer, {0, n - 1}]\)], "Input"],

Cell[BoxData[
    \(424522616403644159931241410173325385382157354103910976867085823777897259\
204041709534955241167934359768743377189454198842444370363294907575469915575210\
432047721947344499899598835841530130933806740231460775533\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(c1 = EncryptBlock[m, pub]\)], "Input"],

Cell[BoxData[
    \(275232904832646348849882449688428277523183902885195657439832259112821851\
545636036770027062125113508328418247096561965619977087681310037420319240712533\
539241317237364845834691278780349992747993966350339353432\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(c2 = EncryptBlock[m, faulty]\)], "Input"],

Cell[BoxData[
    \(407088046085876751522239757455403573604393250300714239834281001693878029\
791096139348099778092197731209346109834249049902428350047811637403525172195931\
515052431551862493404203555803794891710902494397903674702\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \({g, {a, b}} = ExtendedGCD[e1, e2]\)], "Input"],

Cell[BoxData[
    \({1, {
        4517440654559573326585904504842973795220706213265368461574872248488506\
047292926560806906622859635629655655504833, 
        \(-4517440654559573326585904501827271193606493608204262135939267529010\
557237760512745287313567711473075761548836940\)}}\)], "Output"]
}, Open  ]],

Cell["\<\
We will calculate the Multiplicative inverse of c2, since b is negative.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \({d, {c2Inv, y}} = ExtendedGCD[c2, n]\)], "Input"],

Cell[BoxData[
    \({1, {
        5348955703612754396864310025069073528407719075689873222785934819410684\
736628883511256761758763286827078031347450891282177751120520011172432642091561\
5843191820246834023334206199897635627935742918762109069441, 
        \(-4518679145840183657656105536463788324876211645055962387643613111890\
673823275737981679609687733489662836117381032168647899000409576619913276969709\
6278156562092027088983671995895641105928157658014092510798019\)}}\)], "Output"]
}, Open  ]],

Cell["Check that it is the inverse.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Mod[c2*c2Inv, n]\)], "Input"],

Cell[BoxData[
    \(1\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "The first part of the equation is to find ",
  Cell[BoxData[
      \(TraditionalForm\`c\_1\^a\ \((mod\ n)\)\)]],
  "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(part1 = PowerMod[c1, a, n]\)], "Input"],

Cell[BoxData[
    \(357093460583929007551414886163122734824931039349712594514083300809947597\
198448094082761462833300041576162663598021174688587644173867821069122964826952\
89741739821699012385498324277151832900428061580490836308\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "The next part is to find ",
  Cell[BoxData[
      \(TraditionalForm\`\((c\_2\^\(-1\))\)\^\( | b | \)\ \((mod\ n)\)\)]]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(part2 = PowerMod[c2Inv, Abs[b], n]\)], "Input"],

Cell[BoxData[
    \(747535312079196823045797045121994486472627994214218118827537217912241901\
451999417175685889484122719063816936879141150551384153697545764645307910016550\
25169199187388784799149819767209500719250951171256743500\)], "Output"]
}, Open  ]],

Cell["Finally these are combined with multiplication modulo n.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(mdec = Mod[part1*part2, n]\)], "Input"],

Cell[BoxData[
    \(424522616403644159931241410173325385382157354103910976867085823777897259\
204041709534955241167934359768743377189454198842444370363294907575469915575210\
432047721947344499899598835841530130933806740231460775533\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(mdec == m\)], "Input"],

Cell[BoxData[
    \(True\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Capture this in one function", "Subtitle"],

Cell["\<\
The attacker must know e1, e2, n, c1, c2.  With this knowledge they can \
discover m.

Because this attack is based on the Common Modulus attack of Simmons, I will \
call the function after that.\
\>", "Text"],

Cell[BoxData[
    \(CommonModulus[e1_, e2_, n_, c1_, c2_] := 
      Module[{g, a, b}, \n\t\t{g, {a, b}} = ExtendedGCD[e1, e2]; \n\t\t
        If[a < 0, \n\t\t\tComModCalc[c2, b, c1, a, n], \n\t\t\t
          ComModCalc[c1, a, c2, b, n]]]\)], "Input"],

Cell[BoxData[
    \(MultiplicativeInverse[c_, n_] := 
      Module[{g, a, b}, \n\t\t{g, {a, b}} = ExtendedGCD[c, n]; \n\t\ta]\)], 
  "Input"],

Cell[BoxData[
    \(ComModCalc[c1_, a_, c2_, b_, n_] := 
      Module[{c2inv, p1, p2}, \n\t\tc2Inv = MultiplicativeInverse[c2, n]; \n
        \t\tp1 = PowerMod[c1, a, n]; \n\t\tp2 = PowerMod[c2Inv, Abs[b], n]; \n
        \t\tMod[p1*p2, n]]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CommonModulus[e1, e2, n, c1, c2]\)], "Input"],

Cell[BoxData[
    \(424522616403644159931241410173325385382157354103910976867085823777897259\
204041709534955241167934359768743377189454198842444370363294907575469915575210\
432047721947344499899598835841530130933806740231460775533\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(m == %\)], "Input"],

Cell[BoxData[
    \(True\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(m = Random[Integer, {0, n - 1}]\)], "Input"],

Cell[BoxData[
    \(426012712986463799770036834123320739127456116949194792314149904846626319\
492185739328618156828518702272883142718496614597088624347670447083209452809430\
366887653789307282153102747433399115394909753112637633503\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ct1 = EncryptBlock[m, pub]\)], "Input"],

Cell[BoxData[
    \(381844163960128408959969500521995929441706212114165146718950784667131311\
961855224713169928298160577852848208009322335412071409937727417374911006125202\
01061268424191043444769446331344477843402331869638547689\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ct2 = EncryptBlock[m, faulty]\)], "Input"],

Cell[BoxData[
    \(261156561292604692410669838869178263619136901030295675126368475633892951\
094220017624218763636105365711252605343497653908535977040910586907302510437889\
261521054849518926741746490087712442266244152961330293527\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(CommonModulus[e1, e2, n, ct1, ct2]\)], "Input"],

Cell[BoxData[
    \(426012712986463799770036834123320739127456116949194792314149904846626319\
492185739328618156828518702272883142718496614597088624347670447083209452809430\
366887653789307282153102747433399115394909753112637633503\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(m == %\)], "Input"],

Cell[BoxData[
    \(True\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"Microsoft Windows 3.0",
ScreenRectangle->{{0, 1024}, {0, 740}},
WindowSize->{465, 624},
WindowMargins->{{134, Automatic}, {20, Automatic}}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 24, 0, 105, "Title"],
Cell[1758, 53, 41, 0, 64, "Subtitle"],
Cell[1802, 55, 136, 3, 52, "Text"],
Cell[1941, 60, 645, 21, 90, "Text"],

Cell[CellGroupData[{
Cell[2611, 85, 34, 0, 64, "Subtitle"],
Cell[2648, 87, 167, 3, 52, "Text"],

Cell[CellGroupData[{
Cell[2840, 94, 54, 1, 30, "Input"],
Cell[2897, 97, 55, 1, 29, "Output"]
}, Open  ]],
Cell[2967, 101, 76, 0, 33, "Text"],
Cell[3046, 103, 279, 5, 92, "Input"],
Cell[3328, 110, 308, 7, 71, "Text"],
Cell[3639, 119, 31, 0, 33, "Text"],
Cell[3673, 121, 494, 9, 170, "Input"],
Cell[4170, 132, 49, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[4244, 136, 63, 1, 30, "Input"],
Cell[4310, 139, 55, 1, 29, "Output"]
}, Open  ]],
Cell[4380, 143, 26, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[4431, 147, 63, 1, 30, "Input"],
Cell[4497, 150, 55, 1, 29, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[4601, 157, 41, 0, 64, "Subtitle"],
Cell[4645, 159, 88, 2, 33, "Text"],
Cell[4736, 163, 171, 3, 70, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4944, 171, 36, 0, 64, "Subtitle"],
Cell[4983, 173, 63, 1, 30, "Input"],
Cell[5049, 176, 58, 1, 30, "Input"],
Cell[5110, 179, 65, 1, 30, "Input"],
Cell[5178, 182, 57, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[5260, 187, 54, 1, 30, "Input"],
Cell[5317, 190, 149, 2, 67, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5503, 197, 57, 1, 30, "Input"],
Cell[5563, 200, 149, 2, 67, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5749, 207, 41, 1, 30, "Input"],
Cell[5793, 210, 39, 1, 29, "Output"]
}, Open  ]],
Cell[5847, 214, 28, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[5900, 218, 44, 1, 30, "Input"],
Cell[5947, 221, 35, 1, 29, "Output"]
}, Open  ]],
Cell[5997, 225, 262, 5, 71, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6296, 235, 30, 0, 64, "Subtitle"],
Cell[6329, 237, 211, 4, 71, "Text"],
Cell[6543, 243, 637, 17, 109, "Text"],
Cell[7183, 262, 56, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[7264, 266, 64, 1, 30, "Input"],
Cell[7331, 269, 245, 3, 105, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7613, 277, 58, 1, 30, "Input"],
Cell[7674, 280, 245, 3, 105, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7956, 288, 61, 1, 30, "Input"],
Cell[8020, 291, 245, 3, 105, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8302, 299, 66, 1, 30, "Input"],
Cell[8371, 302, 294, 5, 143, "Output"]
}, Open  ]],
Cell[8680, 310, 96, 2, 33, "Text"],

Cell[CellGroupData[{
Cell[8801, 316, 69, 1, 30, "Input"],
Cell[8873, 319, 486, 7, 219, "Output"]
}, Open  ]],
Cell[9374, 329, 45, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[9444, 333, 49, 1, 30, "Input"],
Cell[9496, 336, 35, 1, 29, "Output"]
}, Open  ]],
Cell[9546, 340, 149, 5, 33, "Text"],

Cell[CellGroupData[{
Cell[9720, 349, 59, 1, 30, "Input"],
Cell[9782, 352, 244, 3, 105, "Output"]
}, Open  ]],
Cell[10041, 358, 149, 4, 33, "Text"],

Cell[CellGroupData[{
Cell[10215, 366, 67, 1, 30, "Input"],
Cell[10285, 369, 244, 3, 105, "Output"]
}, Open  ]],
Cell[10544, 375, 72, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[10641, 379, 59, 1, 30, "Input"],
Cell[10703, 382, 245, 3, 105, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10985, 390, 42, 1, 30, "Input"],
Cell[11030, 393, 38, 1, 29, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[11117, 400, 48, 0, 64, "Subtitle"],
Cell[11168, 402, 219, 6, 90, "Text"],
Cell[11390, 410, 250, 4, 130, "Input"],
Cell[11643, 416, 141, 3, 70, "Input"],
Cell[11787, 421, 252, 4, 130, "Input"],

Cell[CellGroupData[{
Cell[12064, 429, 65, 1, 30, "Input"],
Cell[12132, 432, 245, 3, 105, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[12414, 440, 39, 1, 30, "Input"],
Cell[12456, 443, 38, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[12531, 449, 64, 1, 30, "Input"],
Cell[12598, 452, 245, 3, 105, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[12880, 460, 59, 1, 30, "Input"],
Cell[12942, 463, 244, 3, 105, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[13223, 471, 62, 1, 30, "Input"],
Cell[13288, 474, 245, 3, 105, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[13570, 482, 67, 1, 30, "Input"],
Cell[13640, 485, 245, 3, 105, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[13922, 493, 39, 1, 30, "Input"],
Cell[13964, 496, 38, 1, 29, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

