(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     10453,        457]*)
(*NotebookOutlinePosition[     11103,        480]*)
(*  CellTagsIndexPosition[     11059,        476]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["RSA 9 - Guessing the Private Key", "Title"],

Cell["Background", "Subtitle"],

Cell[TextData[{
  "If we can guess the private exponent, ",
  Cell[BoxData[
      \(TraditionalForm\`d\)]],
  ", then it is possible to factorise the modulus, thus totally breaking the \
system open. However, this is not a shortcut as it has been shown that it is \
equivalent only to factoring. One would be better off spending ones time \
searching for the factors. But if the exponent is stumbled upon - this method \
can be used to factor n. "
}], "Text"],

Cell[CellGroupData[{

Cell["An Example", "Subtitle"],

Cell["Generate the keys", "Subsubtitle"],

Cell["\<\
To explain the algorithm I'll go through an example.  First I generate p, q, \
n, e, m, and d. (Look at the RSA document to understand this)\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(p = Prime[22]\)], "Input"],

Cell[BoxData[
    \(79\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(q = Prime[29]\)], "Input"],

Cell[BoxData[
    \(109\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(n = p\ q\)], "Input"],

Cell[BoxData[
    \(8611\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(m = EulerPhi[n]\)], "Input"],

Cell[BoxData[
    \(8424\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(e = 19\)], "Input"],

Cell[BoxData[
    \(19\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \({g, {d, k}} = ExtendedGCD[e, m]\)], "Input"],

Cell[BoxData[
    \({1, {3547, \(-8\)}}\)], "Output"]
}, Open  ]],

Cell["\<\
We now have both public and private keys - (e, n) and (d, n) respectively. \
Let's demonstate an encryption and a decryption:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(PowerMod[15, e, n]\)], "Input"],

Cell[BoxData[
    \(4651\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(PowerMod[4651, d, n]\)], "Input"],

Cell[BoxData[
    \(15\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["The Algorithm", "Subsubtitle"],

Cell[TextData[{
  "We first have to get a factorisation of (e * d) - 1 = ",
  Cell[BoxData[
      \(TraditionalForm\`2\^s\)]],
  "* t.  Use FactorInteger to break the number into its prime factorization."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(FactorInteger[\((e\ d)\)\  - 1]\)], "Input"],

Cell[BoxData[
    \({{2, 6}, {3, 4}, {13, 1}}\)], "Output"]
}, Open  ]],

Cell["Therefore s = 6 and t is the rest", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(s = 6\)], "Input"],

Cell[BoxData[
    \(6\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(t = 3\^4*13\)], "Input"],

Cell[BoxData[
    \(1053\)], "Output"]
}, Open  ]],

Cell["Verify", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(2\^s\ t\  == e\ d\  - 1\)], "Input"],

Cell[BoxData[
    \(True\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "The next step is to choose an ",
  StyleBox["a",
    FontSlant->"Italic"],
  " that's less than n."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(a = Random[Integer, {1, n}]\)], "Input"],

Cell[BoxData[
    \(3430\)], "Output"]
}, Open  ]],

Cell["\<\
Take the first value of i (numbers from 1 to s). There are two tests to pass.\
\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(i = 1\)], "Input"],

Cell[BoxData[
    \(1\)], "Output"]
}, Open  ]],

Cell[TextData["The result cannot be congruent to \[PlusMinus]1."], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(test1 = PowerMod[a, 2\^\(i - 1\)\ t, n]\)], "Input"],

Cell[BoxData[
    \(6321\)], "Output"]
}, Open  ]],

Cell["The result of the second test must equal 1.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(PowerMod[a, 2\^i\ t, n]\)], "Input"],

Cell[BoxData[
    \(6321\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "Since that didn't work try another value of ",
  Cell[BoxData[
      \(TraditionalForm\`i\)]],
  "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(i = 2\)], "Input"],

Cell[BoxData[
    \(2\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(test1 = PowerMod[a, 2\^\(i - 1\)\ t, n]\)], "Input"],

Cell[BoxData[
    \(6321\)], "Output"]
}, Open  ]],

Cell["Test 1 passed.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(test2 = PowerMod[a, 2\^i\ t, n]\)], "Input"],

Cell[BoxData[
    \(1\)], "Output"]
}, Open  ]],

Cell["\<\
Ah that's better. Now we can get the factors using the GCD algorithm.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(GCD[test1 - 1, n]\)], "Input"],

Cell[BoxData[
    \(79\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Defining a Function", "Subtitle"],

Cell[TextData[{
  "We'll abstract the problem into two function.  First a little function \
that returns s and t such that ",
  Cell[BoxData[
      \(TraditionalForm\`r = 2\^s\ t\)]],
  ", given  ",
  Cell[BoxData[
      \(TraditionalForm\`r\)]],
  "."
}], "Text"],

Cell[BoxData[
    \(GetST[r_] := 
      Module[{s, t}, \n\t\ts = 0; \n\t\tt = r; \n\t\t
        While[EvenQ[t], \n\t\t\t\(s++\); \n\t\t\tt = t/2]; \n\t\t{s, t}]\)], 
  "Input"],

Cell["Check that it returned the results as above.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(GetST[e\ d\  - 1]\)], "Input"],

Cell[BoxData[
    \({6, 1053}\)], "Output"]
}, Open  ]],

Cell["Now for the main function.", "Text"],

Cell[BoxData[
    \(GetFactor[e_, d_, n_] := \n\t
      Module[{s, t, a, i, test1, test2, p}, \n\t\t (*\ Get\ s\ and\ t\ *) \n
        \t\t{s, t} = GetST[\((e\ d)\) - 1]; \n\t\tp = 1; \n
        \t\t (*\ Repeat\ as\ often\ as\ necessary\ *) \n\t\t
        While[True, \n\t\t\t (*\ Choose\ a\ random\ a\ *) \n\t\t\t
          a = Random[Integer, {1, n}]; \n
          \t\t\t (*\ Check\ all\ the\ values\ i = 1.  . s\ *) \n\t\t\t
          For[i = 1, i < s, \(i++\), \n\t\t\t\t
            test1 = PowerMod[a, 2\^\(i - 1\)\ t, n]; \n\t\t\t\t
            If[test1 \[Equal] 1 \[Or] test1 \[Equal] \(-1\), Continue[]]; \n
            \t\t\t\ttest2 = PowerMod[a, 2\^i\ t, n]; \n\t\t\t\t
            If[test2 \[Equal] 1, \n\t\t\t\t\tp = GCD[test1 - 1, n]; \n
              \t\t\t\t\tBreak[]]]; \n
          \t\t\t (*\ If\ we\ get\ a\ result\ than\ break\ *) \n\t\t\t
          If[p > 1, Break[]]]; \n\t\tp]\)], "Input"],

Cell["Finally check it with our numbers.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(GetFactor[e, d, n]\)], "Input"],

Cell[BoxData[
    \(79\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["A More Durable Example", "Subtitle"],

Cell["We should test with larger numbers.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(kp = GenerateRSAKey[125]\)], "Input"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \(RSAKey[
      169845503896677400855278690222955347943280876130607881835228298747397159\
16260744734564756949479987792978560769384583919476049, 
      198102414916389166421224438787102523355981270908399147105915792670355012\
686138119338597726329079554911851270725049104251438756671744444501964299413491\
022301733293824173812523905537149436279317317195826153561803940746152850450558\
673254647267981130150517249, 
      500438382277738420982438146402976105957376211181221444894433625904451581\
429361736528650478739044370149376292424688531733887615997799352502011743525218\
894378663470294983764231859531823336124337374866154110368347060060983672921774\
117509583343556935205612827]\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(e = GetExponent[GetPublic[kp]]; \)\)], "Input"],

Cell[BoxData[
    \(\(d = GetExponent[GetPrivate[kp]]; \)\)], "Input"],

Cell[BoxData[
    \(\(n = GetModulus[kp]; \)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Length[IntegerDigits[n, 2]]\)], "Input"],

Cell[BoxData[
    \(847\)], "Output"]
}, Open  ]],

Cell["\<\
Alright. Now bear in ming that we don't at the moment now the correct \
factorization of n. Our function should get it for us.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[p = GetFactor[e, d, n]]\)], "Input"],

Cell[BoxData[
    \({1.75`\ Second, 
      190430700082371485431602426238922345279402128104623781831430997008156070\
10549962848140319734318489383075988422210251254969900947}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(q = n/p\)], "Input"],

Cell[BoxData[
    \(262792912099399936920463537479827765819584834636548648534115354954557674\
39177883948921506593814217936458662815357943099171742041\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(n == p\ q\)], "Input"],

Cell[BoxData[
    \(True\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"Microsoft Windows 3.0",
ScreenRectangle->{{0, 1024}, {0, 712}},
WindowSize->{461, 603},
WindowMargins->{{59, Automatic}, {Automatic, 5}}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 49, 0, 150, "Title"],
Cell[1783, 53, 30, 0, 64, "Subtitle"],
Cell[1816, 55, 459, 9, 109, "Text"],

Cell[CellGroupData[{
Cell[2300, 68, 30, 0, 64, "Subtitle"],
Cell[2333, 70, 40, 0, 53, "Subsubtitle"],
Cell[2376, 72, 164, 3, 52, "Text"],

Cell[CellGroupData[{
Cell[2565, 79, 46, 1, 30, "Input"],
Cell[2614, 82, 36, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2687, 88, 46, 1, 30, "Input"],
Cell[2736, 91, 37, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2810, 97, 41, 1, 30, "Input"],
Cell[2854, 100, 38, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2929, 106, 48, 1, 30, "Input"],
Cell[2980, 109, 38, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3055, 115, 39, 1, 30, "Input"],
Cell[3097, 118, 36, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3170, 124, 64, 1, 30, "Input"],
Cell[3237, 127, 53, 1, 29, "Output"]
}, Open  ]],
Cell[3305, 131, 149, 3, 52, "Text"],

Cell[CellGroupData[{
Cell[3479, 138, 51, 1, 30, "Input"],
Cell[3533, 141, 38, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3608, 147, 53, 1, 30, "Input"],
Cell[3664, 150, 36, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3737, 156, 36, 0, 53, "Subsubtitle"],
Cell[3776, 158, 216, 5, 52, "Text"],

Cell[CellGroupData[{
Cell[4017, 167, 64, 1, 30, "Input"],
Cell[4084, 170, 59, 1, 29, "Output"]
}, Open  ]],
Cell[4158, 174, 49, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[4232, 178, 38, 1, 30, "Input"],
Cell[4273, 181, 35, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4345, 187, 44, 1, 30, "Input"],
Cell[4392, 190, 38, 1, 29, "Output"]
}, Open  ]],
Cell[4445, 194, 22, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[4492, 198, 56, 1, 30, "Input"],
Cell[4551, 201, 38, 1, 29, "Output"]
}, Open  ]],
Cell[4604, 205, 130, 5, 33, "Text"],

Cell[CellGroupData[{
Cell[4759, 214, 60, 1, 30, "Input"],
Cell[4822, 217, 38, 1, 29, "Output"]
}, Open  ]],
Cell[4875, 221, 103, 3, 33, "Text"],

Cell[CellGroupData[{
Cell[5003, 228, 38, 1, 30, "Input"],
Cell[5044, 231, 35, 1, 29, "Output"]
}, Open  ]],
Cell[5094, 235, 74, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[5193, 239, 72, 1, 31, "Input"],
Cell[5268, 242, 38, 1, 29, "Output"]
}, Open  ]],
Cell[5321, 246, 59, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[5405, 250, 56, 1, 31, "Input"],
Cell[5464, 253, 38, 1, 29, "Output"]
}, Open  ]],
Cell[5517, 257, 131, 5, 33, "Text"],

Cell[CellGroupData[{
Cell[5673, 266, 38, 1, 30, "Input"],
Cell[5714, 269, 35, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5786, 275, 72, 1, 31, "Input"],
Cell[5861, 278, 38, 1, 29, "Output"]
}, Open  ]],
Cell[5914, 282, 30, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[5969, 286, 64, 1, 31, "Input"],
Cell[6036, 289, 35, 1, 29, "Output"]
}, Open  ]],
Cell[6086, 293, 93, 2, 33, "Text"],

Cell[CellGroupData[{
Cell[6204, 299, 50, 1, 30, "Input"],
Cell[6257, 302, 36, 1, 29, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[6354, 310, 39, 0, 64, "Subtitle"],
Cell[6396, 312, 264, 9, 52, "Text"],
Cell[6663, 323, 176, 4, 150, "Input"],
Cell[6842, 329, 60, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[6927, 333, 50, 1, 30, "Input"],
Cell[6980, 336, 43, 1, 29, "Output"]
}, Open  ]],
Cell[7038, 340, 42, 0, 33, "Text"],
Cell[7083, 342, 912, 15, 418, "Input"],
Cell[7998, 359, 50, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[8073, 363, 51, 1, 30, "Input"],
Cell[8127, 366, 36, 1, 29, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[8212, 373, 42, 0, 64, "Subtitle"],
Cell[8257, 375, 51, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[8333, 379, 57, 1, 30, "Input"],
Cell[8393, 382, 74, 1, 25, "Print"],
Cell[8470, 385, 74, 1, 25, "Print"],
Cell[8547, 388, 716, 11, 314, "Output"]
}, Open  ]],
Cell[9278, 402, 69, 1, 30, "Input"],
Cell[9350, 405, 70, 1, 30, "Input"],
Cell[9423, 408, 57, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[9505, 413, 60, 1, 30, "Input"],
Cell[9568, 416, 37, 1, 29, "Output"]
}, Open  ]],
Cell[9620, 420, 150, 3, 52, "Text"],

Cell[CellGroupData[{
Cell[9795, 427, 63, 1, 30, "Input"],
Cell[9861, 430, 188, 3, 86, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10086, 438, 40, 1, 30, "Input"],
Cell[10129, 441, 164, 2, 67, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10330, 448, 42, 1, 30, "Input"],
Cell[10375, 451, 38, 1, 29, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

