(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     14525,        529]*)
(*NotebookOutlinePosition[     15174,        552]*)
(*  CellTagsIndexPosition[     15130,        548]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["RSA - 11", "Title"],

Cell["Low Exponent Attack", "Subtitle"],

Cell["\<\
This attack can be used whenever the same message (or partial message) is \
encrypted several times with low exponents and different modulus. To be more \
specific the attack is good against systems that use a exponent of 3 to \
increase the speed of encrypting. In this case we only need 3 messages. \
\>", "Text"],

Cell["\<\
The attack used the Chinese Remainder theorem to derive a unique solution x \
based on the three encrypted messages. The following function Guass, uses \
Guass's method for solving the equations. The values passed in are the \
encrypted messages and the different moduli used. The returned x should be \
the message block raised to the power of the exponent given that many \
messages.\
\>", "Text"],

Cell[BoxData[
    \(GetInverse[u_, m_] := 
      Module[{k, d, g}, \n\t\t{g, {d, k}} = ExtendedGCD[u, m]; \n\t\td]\)], 
  "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Guass[as_, ns_] := \n\t
      Module[{n, bigNs, bigMs, i, k, x}, \n\t\tn = Fold[Times, 1, ns]; \n\t\t
        k = Length[ns]; \n\t\tbigNs = Map[\((n/#)\)&, ns]; \n\t\t
        bigMs = Table[
            GetInverse[bigNs\[LeftDoubleBracket]i\[RightDoubleBracket], 
              ns\[LeftDoubleBracket]i\[RightDoubleBracket]], {i, 1, k}]; \n
        \t\tx = \[Sum]\+\(i = 1\)\%k Mod[
              as\[LeftDoubleBracket]i\[RightDoubleBracket]*
                bigNs\[LeftDoubleBracket]i\[RightDoubleBracket]*
                bigMs\[LeftDoubleBracket]i\[RightDoubleBracket], n]; \n\t\t
        Mod[x, n]]\)], "Input"],

Cell[BoxData[
    \(General::"spell1" \( : \ \) 
      "Possible spelling error: new symbol name \"\!\(bigMs\)\" is similar to \
existing symbol \"\!\(bigNs\)\"."\)], "Message"]
}, Open  ]],

Cell["A small test", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Guass[{1, 2, 2}, {2, 3, 5}]\)], "Input"],

Cell[BoxData[
    \(17\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Guass[{9, 6, 4}, {13, 17, 35}]\)], "Input"],

Cell[BoxData[
    \(74\)], "Output"]
}, Open  ]],

Cell["A bigger test. I need to generate 3 keys first.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(kp1 = SpecifyExponent[15, 3]\)], "Input"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \(RSAKey[3, 1712318867478213272677310327429, 
      2568478301217319909015965491143]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(kp2 = SpecifyExponent[15, 3]\)], "Input"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \(RSAKey[3, 2232394521772164416391824180861, 
      3348591782658246624587736271291]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(kp3 = SpecifyExponent[15, 3]\)], "Input"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \(RSAKey[3, 3392588530374698227667923228021, 
      5088882795562047341501884842031]\)], "Output"]
}, Open  ]],

Cell["\<\
Now I will use a random number as the message. Note that this must be less \
than the minimum of the moduli.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(message = 
      Random[Integer, {1, 
          Min[GetModulus[kp1], GetModulus[kp2], GetModulus[kp3]]}]\)], "Input"],

Cell[BoxData[
    \(General::"spell1" \( : \ \) 
      "Possible spelling error: new symbol name \"\!\(message\)\" is similar \
to existing symbol \"\!\(Message\)\"."\)], "Message"],

Cell[BoxData[
    \(224200839328894917664210326223\)], "Output"]
}, Open  ]],

Cell["\<\
Encrypt the block with each of the keys giving three ciphertexts.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(ct1 = EncryptBlock[message, GetPublic[kp1]]\)], "Input"],

Cell[BoxData[
    \(617426256684026878366370838633\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ct2 = EncryptBlock[message, GetPublic[kp2]]\)], "Input"],

Cell[BoxData[
    \(1904480366636689957459387628534\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ct3 = EncryptBlock[message, GetPublic[kp3]]\)], "Input"],

Cell[BoxData[
    \(3777044406072298806186921178706\)], "Output"]
}, Open  ]],

Cell["Extract the modulus from each key.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(ns = {GetModulus[kp1], GetModulus[kp2], GetModulus[kp3]}\)], "Input"],

Cell[BoxData[
    \({2568478301217319909015965491143, 3348591782658246624587736271291, 
      5088882795562047341501884842031}\)], "Output"]
}, Open  ]],

Cell["Put the ciphertexts into a list also", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(as = {ct1, ct2, ct3}\)], "Input"],

Cell[BoxData[
    \({617426256684026878366370838633, 1904480366636689957459387628534, 
      3777044406072298806186921178706}\)], "Output"]
}, Open  ]],

Cell["Use the Guass algorithm to get x", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(x = Guass[as, ns]\)], "Input"],

Cell[BoxData[
    \(112696830566860497143603284529520630642426869332206496730986154883106910\
01499100560051567\)], "Output"]
}, Open  ]],

Cell["Compute the cubed root and check it against the message", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(pt = \@x\%3\)], "Input"],

Cell[BoxData[
    \(224200839328894917664210326223\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(pt == message\)], "Input"],

Cell[BoxData[
    \(True\)], "Output"]
}, Open  ]],

Cell["Success. We should use very large keys to test.", "Text"],

Cell[CellGroupData[{

Cell["Bigger Keys", "Subtitle"],

Cell["\<\
No comments are needed. Just use the ones from the previous section.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(kp4 = SpecifyExponent[100, 3]; \)\)], "Input"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(kp5 = SpecifyExponent[100, 3]; \)\)], "Input"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(kp6 = SpecifyExponent[100, 3]; \)\)], "Input"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"]
}, Open  ]],

Cell[BoxData[
    \(\({n4, n5, n6} = Map[GetModulus[#]&, {kp4, kp5, kp6}]; \)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(plaintext = Random[Integer, {1, Min[n4, n5, n6]}]\)], "Input"],

Cell[BoxData[
    \(536699252907797095357462633991083427612393939388791887477444771627006703\
645225183171376514758888831419067380747745960577381697708683502525818659233851\
8954333978358316873273697103847254474143315841014241\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ciphers = 
      Map[EncryptBlock[plaintext, GetPublic[#]]&, {kp4, kp5, kp6}]\)], "Input"],

Cell[BoxData[
    \(General::"spell1" \( : \ \) 
      "Possible spelling error: new symbol name \"\!\(ciphers\)\" is similar \
to existing symbol \"\!\(cipher\)\"."\)], "Message"],

Cell[BoxData[
    \({18733688143013890564309696242350494762740677685415310095782433802058021\
931231817124972834974765507935151090244135701700967161006698742216300173855880\
892814694579954327200164646882815438115797397025883931106, 
      262807337978045720373047932971165285062807944856236925818027358095084106\
544865003118601925150606209824165517683624190623411746694997689177458489271034\
180704726623427291838903946479328245485946211017873310, 
      262528640692550613587374257560872293474254454433257598054977228041352011\
096249889831225686124640099928310362270150738540647035223096375240704639353887\
8366310193923984275030307624299535253467335043240302951}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(x = Guass[ciphers, {n4, n5, n6}]\)], "Input"],

Cell[BoxData[
    \(154594120271141901886962389560266988724093664111276281928168054916932379\
207281434313780630831653956480808137778296723306670382338868209710886919315265\
860853269674929661488704862568145310717975419602807492029988660696927408335225\
658020426119244160047900632589394125482493460693070304813082108347377826584901\
504479140036121337791061422905534326986044984701863526108241690881656878743147\
100991271265650928540771085692246419707700241266207633931520072557790716790580\
421309454583917328418852539798537768446084143996946511647753567352939873847463\
649040629725873266037326485871880529945171582603410341175524399521\)], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(pt2 = \@x\%3\)], "Input"],

Cell[BoxData[
    \(536699252907797095357462633991083427612393939388791887477444771627006703\
645225183171376514758888831419067380747745960577381697708683502525818659233851\
8954333978358316873273697103847254474143315841014241\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(pt2 == plaintext\)], "Input"],

Cell[BoxData[
    \(True\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["What about higher exponents", "Subtitle"],

Cell[TextData[{
  "The attack can conceivably use higher exponents, but we need to have more \
messages encrypted for it to work. This is why ",
  Cell[BoxData[
      \(TraditionalForm\`2\^16 - 1\)]],
  " is often used as an exponent. It is fairly fast (very fast compared with \
a randomly chosen exponent) and it is unlikely that the same message will be \
encrypted with ",
  Cell[BoxData[
      \(TraditionalForm\`2\^16 - 1\)]],
  " messages."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(kps = Table[SpecifyExponent[15, 5], {i, 1, 5}]\)], "Input"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \({RSAKey[5, 1801598228026779736816776649423, 
        4503995570066949342041941623557], 
      RSAKey[5, 1604850285189629811772414180657, 
        2006062856487037264715517725821], 
      RSAKey[5, 20471109050585282938463616418465, 
        25588886313231603673079520523081], 
      RSAKey[5, 573316548387609136492038774184, 
        2866582741938045682460193870919], 
      RSAKey[5, 372169109740505473438193298345, 
        465211387175631841797741622931]}\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(ns = Map[GetModulus[#]&, kps]; \)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(pln = Random[Integer, {1, Min[ns]}]\)], "Input"],

Cell[BoxData[
    \(27209542974852275049651123997\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ctxts = Map[EncryptBlock[pln, GetPublic[#]]&, kps]\)], "Input"],

Cell[BoxData[
    \({1053670396573545219334069935229, 936753029327140767519498687487, 
      9565589794756761567601702244263, 1661014044353557554292493111032, 
      57631212426888384977299669793}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(x = Guass[ctxts, ns]\)], "Input"],

Cell[BoxData[
    \(149144154365136327815358212788710490901669102236666995641940865323101742\
03914327818337729430974220124636637195406582049916433594966157185219757\)], 
  "Output"]
}, Open  ]],

Cell["This time the fifth root is found.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\@x\%5\)], "Input"],

Cell[BoxData[
    \(27209542974852275049651123997\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(% == pln\)], "Input"],

Cell[BoxData[
    \(True\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"Microsoft Windows 3.0",
ScreenRectangle->{{0, 1024}, {0, 740}},
WindowSize->{438, 632},
WindowMargins->{{7, Automatic}, {Automatic, 5}}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 25, 0, 105, "Title"],
Cell[1759, 53, 39, 0, 64, "Subtitle"],
Cell[1801, 55, 325, 5, 90, "Text"],
Cell[2129, 62, 409, 7, 109, "Text"],
Cell[2541, 71, 130, 3, 70, "Input"],

Cell[CellGroupData[{
Cell[2696, 78, 634, 11, 217, "Input"],
Cell[3333, 91, 177, 3, 63, "Message"]
}, Open  ]],
Cell[3525, 97, 28, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[3578, 101, 60, 1, 30, "Input"],
Cell[3641, 104, 36, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3714, 110, 63, 1, 30, "Input"],
Cell[3780, 113, 36, 1, 29, "Output"]
}, Open  ]],
Cell[3831, 117, 63, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[3919, 121, 61, 1, 30, "Input"],
Cell[3983, 124, 74, 1, 25, "Print"],
Cell[4060, 127, 74, 1, 25, "Print"],
Cell[4137, 130, 116, 2, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4290, 137, 61, 1, 30, "Input"],
Cell[4354, 140, 74, 1, 25, "Print"],
Cell[4431, 143, 74, 1, 25, "Print"],
Cell[4508, 146, 74, 1, 25, "Print"],
Cell[4585, 149, 116, 2, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4738, 156, 61, 1, 30, "Input"],
Cell[4802, 159, 74, 1, 25, "Print"],
Cell[4879, 162, 74, 1, 25, "Print"],
Cell[4956, 165, 116, 2, 48, "Output"]
}, Open  ]],
Cell[5087, 170, 132, 3, 52, "Text"],

Cell[CellGroupData[{
Cell[5244, 177, 137, 3, 50, "Input"],
Cell[5384, 182, 181, 3, 63, "Message"],
Cell[5568, 187, 64, 1, 29, "Output"]
}, Open  ]],
Cell[5647, 191, 89, 2, 33, "Text"],

Cell[CellGroupData[{
Cell[5761, 197, 76, 1, 30, "Input"],
Cell[5840, 200, 64, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5941, 206, 76, 1, 30, "Input"],
Cell[6020, 209, 65, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6122, 215, 76, 1, 30, "Input"],
Cell[6201, 218, 65, 1, 29, "Output"]
}, Open  ]],
Cell[6281, 222, 50, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[6356, 226, 89, 1, 50, "Input"],
Cell[6448, 229, 140, 2, 67, "Output"]
}, Open  ]],
Cell[6603, 234, 52, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[6680, 238, 53, 1, 30, "Input"],
Cell[6736, 241, 139, 2, 67, "Output"]
}, Open  ]],
Cell[6890, 246, 48, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[6963, 250, 50, 1, 30, "Input"],
Cell[7016, 253, 125, 2, 48, "Output"]
}, Open  ]],
Cell[7156, 258, 71, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[7252, 262, 44, 1, 34, "Input"],
Cell[7299, 265, 64, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7400, 271, 46, 1, 30, "Input"],
Cell[7449, 274, 38, 1, 29, "Output"]
}, Open  ]],
Cell[7502, 278, 63, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[7590, 282, 31, 0, 64, "Subtitle"],
Cell[7624, 284, 92, 2, 33, "Text"],

Cell[CellGroupData[{
Cell[7741, 290, 68, 1, 30, "Input"],
Cell[7812, 293, 74, 1, 25, "Print"],
Cell[7889, 296, 74, 1, 25, "Print"],
Cell[7966, 299, 74, 1, 25, "Print"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8077, 305, 68, 1, 30, "Input"],
Cell[8148, 308, 74, 1, 25, "Print"],
Cell[8225, 311, 74, 1, 25, "Print"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8336, 317, 68, 1, 30, "Input"],
Cell[8407, 320, 74, 1, 25, "Print"],
Cell[8484, 323, 74, 1, 25, "Print"],
Cell[8561, 326, 74, 1, 25, "Print"]
}, Open  ]],
Cell[8650, 330, 90, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[8765, 335, 82, 1, 30, "Input"],
Cell[8850, 338, 240, 3, 105, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[9127, 346, 110, 2, 70, "Input"],
Cell[9240, 350, 180, 3, 63, "Message"],
Cell[9423, 355, 681, 9, 295, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10141, 369, 65, 1, 30, "Input"],
Cell[10209, 372, 657, 9, 257, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10903, 386, 45, 1, 34, "Input"],
Cell[10951, 389, 240, 3, 105, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[11228, 397, 49, 1, 30, "Input"],
Cell[11280, 400, 38, 1, 29, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[11367, 407, 47, 0, 64, "Subtitle"],
Cell[11417, 409, 459, 11, 90, "Text"],

Cell[CellGroupData[{
Cell[11901, 424, 79, 1, 30, "Input"],
Cell[11983, 427, 74, 1, 25, "Print"],
Cell[12060, 430, 74, 1, 25, "Print"],
Cell[12137, 433, 74, 1, 25, "Print"],
Cell[12214, 436, 74, 1, 25, "Print"],
Cell[12291, 439, 74, 1, 25, "Print"],
Cell[12368, 442, 74, 1, 25, "Print"],
Cell[12445, 445, 74, 1, 25, "Print"],
Cell[12522, 448, 74, 1, 25, "Print"],
Cell[12599, 451, 74, 1, 25, "Print"],
Cell[12676, 454, 74, 1, 25, "Print"],
Cell[12753, 457, 74, 1, 25, "Print"],
Cell[12830, 460, 491, 10, 200, "Output"]
}, Open  ]],
Cell[13336, 473, 68, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[13429, 478, 68, 1, 30, "Input"],
Cell[13500, 481, 63, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[13600, 487, 83, 1, 30, "Input"],
Cell[13686, 490, 210, 3, 105, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[13933, 498, 53, 1, 30, "Input"],
Cell[13989, 501, 182, 3, 86, "Output"]
}, Open  ]],
Cell[14186, 507, 50, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[14261, 511, 39, 1, 34, "Input"],
Cell[14303, 514, 63, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[14403, 520, 41, 1, 30, "Input"],
Cell[14447, 523, 38, 1, 29, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

