(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     13132,        339]*)
(*NotebookOutlinePosition[     13781,        362]*)
(*  CellTagsIndexPosition[     13737,        358]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["RSA - Adaptive Chosen Plaintext Attack", "Title"],

Cell["Introduction", "Subtitle"],

Cell["\<\
We will use the multiplicative properties of RSA ciphertexts to fool the \
receiver into helping us see the message.  This time Malory alters a message \
encrypted with Alice's public key. She then sends it to Alice, who decrypts \
it. Malory knows that Alice will not be able to make sense of the message (as \
she as tampered with it).  Malory hopes Alice will email the decrypted mess \
back to her, or to technical support, so that she can get a copy of it. Alice \
may be naive enough to do this, or might simply forget to delete the garbage \
completely from her workstation. If Malory does manage to get a hold of this \
message, she can recover the plaintext of the original message\
\>", "Text"],

Cell[CellGroupData[{

Cell["Alices' Key", "Subtitle"],

Cell[TextData[{
  "Note that Alice's private key will be used in this scenario only once, and \
by Alice.  Malory will recover the plaintext ",
  StyleBox["without",
    FontSlant->"Italic"],
  " the private key."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(kp = GenerateRSAKey[150]\)], "Input"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \(RSAKey[
      391768821822159899080523926842524333529982917433507295401390143191563173\
163630518847275274093311157038909821216334159794084031185708684609814325795591\
, 2943666698899435651424543077646061335369807909729776627808733369871682577966\
183857998467503173148158476078777290692838013939032806508212963905104003336188\
172946512793803098670055222233654260382521013593505021553945133208257971262441\
90487781010442642270462440843783303505456890112785531012722150262942778647, 
      379069418793739351914063499078127363208737731600485913386713289829194170\
431222518410975042329767394826910694787206542699770980202536483988994438032640\
535166811128337677864361836028354633137457802277526374989473037260076478560364\
407897336961633628877354566075546177469482555857118946445707986358221506416667\
]\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["A message for Alice", "Subtitle"],

Cell[BoxData[
    \(\(pub = GetPublic[kp]; \)\)], "Input"],

Cell["\<\
The plaintext is taken from ptv.ie's description of a Stargate SG-1 program \
description. (That web page just happened to be open). We assume that Malory \
is an adept computer hacker and will be able to sniff the message as it \
crosses the network, getting a complet copy of the ciphertext.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(plaintext = 
      "\<The team leave an alien civilisation facing extinction after \
accidentally spreading a disease throughout their community. Starring Richard \
Dean Anderson\>"\ \)\)], "Input"],

Cell[BoxData[
    \("The team leave an alien civilisation facing extinction after \
accidentally spreading a disease throughout their community. Starring Richard \
Dean Anderson"\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ciphertext = EncryptMessage[plaintext, pub]\)], "Input"],

Cell[BoxData[
    \({22025201477002131837913719796375768976407944586289110812954676755848240\
394511761252988673245867719129547774343716601915896307784321324138518673887560\
964660486550455566400260677948641090933910961007040133414760990239336087683274\
126528295979750164367037882264060689951753532072018204212806178088633443289984\
2, 216002071110912419344579539890707510151219760236150877337408509737643270196\
448808550177884900999062386132971201011696013885731246011295212279086490472807\
534891993847404348956697393015462508496332805510180863871745228529300052823942\
214136360821135680945401830030782917777537224225787257930154417376069444534}
      \)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Malory's Part", "Subtitle"],

Cell["\<\
At some point this is sent to Alice, but it is intercepted by Malory. She \
then adapts the message by multiplicating it modulus n by a random value that \
has been encrypted.\
\>", "Text"],

Cell["First step is to find a random integer", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(x = Random[Integer, {1, GetModulus[pub] - 1}]\)], "Input"],

Cell[BoxData[
    \(124970294184786417863046696038085470021986957351176219083364940765488024\
485420018415688553920038952471847015027654446501212886445487548584767202557956\
839132534539378855260195605634117763238648858192192548573083380395421562023814\
557053266890508749251572032796245444586821429675311483407487372388601032161879
    \)], "Output"]
}, Open  ]],

Cell["Now encryot that number.", "Text"],

Cell[BoxData[
    \(\(xe = EncryptBlock[x, pub]; \)\)], "Input"],

Cell["Multiply it by every block of the message.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(adaptedCT = Mod[ciphertext*xe, GetModulus[pub]]\)], "Input"],

Cell[BoxData[
    \({16276485164569150938824229315702575247259813272093198266614141039653059\
176530334358420563589415530864799762244426821841931162790483768871219077845166\
895445554279057161047291945292713614479756935418350657064795378071213025866983\
463178985764584042277310397184749589773215905437679498219031002235948836570124\
7, 111875430464819482916250939130767409759976383564795131985835526358320786159\
884682355061220556748919774183254845390572338208193594759429969331488940300254\
831671217245695895162179871786314998044078997129855252586573127288524849693567\
904253848471877576490030785372432783969864265613150371014189569707108179337}
      \)], "Output"]
}, Open  ]],

Cell["\<\
Malory now sends this to Alice. Alice will attempt to decrypt the message, \
thinking that it is a legitimate message. \
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Alice Receives the Message", "Subtitle"],

Cell["Note, this is the first time that the private key is used.", "Text"],

Cell[BoxData[
    \(\(prv = GetPrivate[kp]; \)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(recovered = DecryptMessage[adaptedCT, prv]\)], "Input"],

Cell[BoxData[
    \("*7\[DownExclamation]\[Paragraph]\[Not]c\[OAcute]K\[ARing]\
\[CapitalEGrave]3\[RawEscape]\.88\.1f0\[CapitalOHat]=O\[RightGuillemet]\
\[CapitalEDoubleDot]ow\.08O\.04zL:\[Yen]%\.05\.9d\.8b\[CapitalIHat]jd\.93g\.96\
\[OHat]\[CapitalIGrave]Nc\[CCedilla]\[CapitalIAcute]J\[CapitalAHat]\[Thorn]\
\[CapitalOAcute]J\[CapitalAGrave]\tO\.1a\.af.\.1f\[CapitalARing]\
\[CapitalUGrave]Y\.90A\.ba\.bc\[IHat]tYu\.b29\[ARing]$*\[ODoubleDot]\[NTilde]\
\.0eB\[EAcute]?\[CCedilla]\[EHat]\.89\[CapitalEHat]\[CapitalEAcute]\.be\.88\
\.13\.17[\[AE]\[AE]j\[CapitalOGrave]-D=\"3\[CapitalAAcute]\.10=\[CCedilla]I\
\.a6\.12\[CapitalUDoubleDot]Jx\[Copyright]y[]\.94\[CapitalEDoubleDot]\
\[CapitalUHat]\.00\.06\[CapitalEth]\.18M\[YDoubleDot]\[ADoubleDot]W\.93\.0b\
\[OHat]\[Cent]x?\.95\[CapitalUGrave]\.92\[Times]\.82\[AHat]\[OTilde]Z\
\[CapitalAHat]^3\[CapitalOHat]\.939N&Ie\[CapitalEth]\[CCedilla]\[CapitalEth]=\
4\.bd\.91\"\[CapitalUDoubleDot]\.91\.8cA\[DownExclamation]\.a8\[YAcute]\
\[CapitalThorn]\.1do\.af\\\.03\[SZ]\.1eE\[CapitalThorn]\[Currency]S\.17\
\[IAcute]c\[CapitalCCedilla]#\.14\[RawEscape]\.98|h\.82F\.93\.b4r\.14V&\
\[IGrave]j\.b3\[CapitalIAcute]\.be\[OSlash]F>\[Sterling]\[Currency]\[IGrave]\
\[EGrave]\[CapitalIDoubleDot]S\.84\.b8gLl\[ADoubleDot]G\[RightGuillemet]\.8c\
\[AHat]\.b4^q\[CapitalEth]\.b8\.b9U{\[CapitalADoubleDot]+\[CapitalUHat]\
\[NTilde]%[}\.80\.86\.19\[RightGuillemet]w^\.07\[Eth]u\[CapitalEHat]\.8cgk\
\[LeftGuillemet]\.9d\.19q\[ODoubleDot]v\.1c\.8bC\.bd"\)], "Output"]
}, Open  ]],

Cell["\<\
This is clearly nonsense. If Alice does not smell a rat, then she may reply, \
displaying the garbled output from the program. This is exactly what Malory \
wants. If she does not, then she may leave the garbled output in a file on \
her workstation open for an experienced hacker such as Malory to get at. If \
Malory did have access to Alice's workstation, one could argue that she does \
not need to go to such lengths to decrypt the message. She should be able to \
read the decrypted file from the computer. But if the message is of sensitive \
nature, perhaps Alice was careful to wipe it completely. But a garbled \
message might not seem as important. Unless Alice is aware of this type of \
attack, she might not be as careful.\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Back to Malory", "Subtitle"],

Cell["\<\
Having somehow gotten the result of the decrytpion, Malory now converts the \
text back into its numerical format.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(numerical = EncodeMessage[recovered, GetModulus[pub]]\)], "Input"],

Cell[BoxData[
    \({11580470383018154322419450680922606791586138509919073686901621948871027\
226144314028868943410698139586404767423504332574456201361744276289932289985892\
675639660652944148792883673659144008494908772953814093251114815769298619353984\
128790897252431141915228578092991184924843080267348947975755976126221754275958\
6, 329848766638206930191592439622701152210167621230825218870640278294924044473\
069896918789209892340968058953047552090578240533037486411902159533418772702939\
173421521433722884262902622146032537593966106506673944674476709702643172987133\
873624430166055660224604923526481688873815202139517562398083695445843592125}
      \)], "Output"]
}, Open  ]],

Cell["\<\
Now all that is left is to extract the x that garbled up the message. We \
first find the multiplicative inverse of x mod n.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(xInv = PowerMod[x, \(-1\), GetModulus[pub]]\)], "Input"],

Cell[BoxData[
    \(266816250457692900502039500770967063487836265936391202086366176519639801\
787194340163386834426159600243343656730043337549770472574421697609878547637992\
866801913929349259537208069292142395699656293636661037007600090315399940975165\
019986081062681256611939468304026555245443527452335161275029692395430908146030
    \)], "Output"]
}, Open  ]],

Cell["And then multiply it across the message blocks.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(correct = Map[Mod[#*xInv, GetModulus[pub]]&, numerical]\)], "Input"],

Cell[BoxData[
    \({90443676790400222611789581902487432021221293085936362204477002745192925\
344269966010818235016946480488935377497793630329651345780163833728042471479763\
519879526340689208938648635356199800330316551119351165945838732938289743342011\
5295124642334819148727361605837915800900612992260288713201686471186071183730, 
      122593272249373779198518601478649065247062019752481865006029672613725820\
995371617026878388404173494124873012077563505920549946873887795460431008417212\
311820656054703002013799686460034712023190193056898347117772701109133149998730\
7247379496565854240222689514328864813996043467837490636744540023706261397536}
      \)], "Output"]
}, Open  ]],

Cell["Now we simply use the standard decoder.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(DecodeMessage[correct]\)], "Input"],

Cell[BoxData[
    \("The team leave an alien civilisation facing extinction after \
accidentally spreading a disease throughout their community. Starring Richard \
Dean Anderson                                                                 \
                                   "\)], "Output"]
}, Open  ]],

Cell["And voila! The recovered message!", "Text"]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"Microsoft Windows 3.0",
ScreenRectangle->{{0, 1024}, {0, 740}},
WindowSize->{499, 635},
WindowMargins->{{0, Automatic}, {Automatic, 0}}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 55, 0, 150, "Title"],
Cell[1789, 53, 32, 0, 64, "Subtitle"],
Cell[1824, 55, 714, 10, 166, "Text"],

Cell[CellGroupData[{
Cell[2563, 69, 31, 0, 64, "Subtitle"],
Cell[2597, 71, 225, 6, 52, "Text"],

Cell[CellGroupData[{
Cell[2847, 81, 57, 1, 30, "Input"],
Cell[2907, 84, 74, 1, 25, "Print"],
Cell[2984, 87, 74, 1, 25, "Print"],
Cell[3061, 90, 840, 12, 314, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[3950, 108, 39, 0, 64, "Subtitle"],
Cell[3992, 110, 58, 1, 30, "Input"],
Cell[4053, 113, 317, 5, 90, "Text"],

Cell[CellGroupData[{
Cell[4395, 122, 220, 4, 90, "Input"],
Cell[4618, 128, 192, 3, 67, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4847, 136, 76, 1, 30, "Input"],
Cell[4926, 139, 671, 9, 238, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[5646, 154, 33, 0, 64, "Subtitle"],
Cell[5682, 156, 199, 4, 52, "Text"],
Cell[5884, 162, 54, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[5963, 166, 78, 1, 30, "Input"],
Cell[6044, 169, 351, 5, 124, "Output"]
}, Open  ]],
Cell[6410, 177, 40, 0, 33, "Text"],
Cell[6453, 179, 64, 1, 30, "Input"],
Cell[6520, 182, 58, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[6603, 186, 80, 1, 30, "Input"],
Cell[6686, 189, 671, 9, 238, "Output"]
}, Open  ]],
Cell[7372, 201, 143, 3, 52, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7552, 209, 46, 0, 64, "Subtitle"],
Cell[7601, 211, 74, 0, 33, "Text"],
Cell[7678, 213, 59, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[7762, 218, 75, 1, 30, "Input"],
Cell[7840, 221, 1498, 20, 105, "Output"]
}, Open  ]],
Cell[9353, 244, 760, 11, 166, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10150, 260, 34, 0, 64, "Subtitle"],
Cell[10187, 262, 138, 3, 52, "Text"],

Cell[CellGroupData[{
Cell[10350, 269, 86, 1, 30, "Input"],
Cell[10439, 272, 671, 9, 238, "Output"]
}, Open  ]],
Cell[11125, 284, 148, 3, 52, "Text"],

Cell[CellGroupData[{
Cell[11298, 291, 76, 1, 30, "Input"],
Cell[11377, 294, 351, 5, 124, "Output"]
}, Open  ]],
Cell[11743, 302, 63, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[11831, 306, 88, 1, 30, "Input"],
Cell[11922, 309, 671, 9, 238, "Output"]
}, Open  ]],
Cell[12608, 321, 55, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[12688, 325, 55, 1, 30, "Input"],
Cell[12746, 328, 294, 4, 105, "Output"]
}, Open  ]],
Cell[13055, 335, 49, 0, 33, "Text"]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

