(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     10522,        390]*)
(*NotebookOutlinePosition[     11285,        419]*)
(*  CellTagsIndexPosition[     11212,        413]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["RSA - 13 Low Private Exponent", "Title"],

Cell["\<\
This attack uses the Continued Fraction Algorithm for estimating fractions. \
We can make some use of the Continued Fractions algorithm\
\>", "Text"],

Cell["<<NumberTheory`ContinuedFractions`", "Input",
  CellTags->"S5.67.1"],

Cell["\<\
An example of representing a number in continued fraction form\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(ContinuedFraction[4/11]\)], "Input"],

Cell[BoxData[
    TagBox[
      StyleBox[\(0 + 1\/\(2 + 1\/\(1 + 1\/3\)\)\),
        ScriptSizeMultipliers->1],
      (ContinuedFractionForm[ {0, 2, 1, 3}]&)]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(form = ContinuedFractionForm[{0, 2, 1, 3}]\)], "Input"],

Cell[BoxData[
    TagBox[
      StyleBox[\(0 + 1\/\(2 + 1\/\(1 + 1\/3\)\)\),
        ScriptSizeMultipliers->1],
      (ContinuedFractionForm[ {0, 2, 1, 3}]&)]], "Output"]
}, Open  ]],

Cell["And back again", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Normal[form]\)], "Input"],

Cell[BoxData[
    \(4\/11\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Normal[ContinuedFractionForm[{0, 2, 1, 4}]]\)], "Input"],

Cell[BoxData[
    \(5\/14\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(4/11 > 5/14\)], "Input"],

Cell[BoxData[
    \(True\)], "Output"]
}, Open  ]],

Cell["\<\
The function was a nightmare to write. I had to learn all about continued \
fractions. Basically what's going on is we estimate k/dg based on public \
information: e/n. Then we do several iterations, hopefully resulting in a \
factor of n.  We do this by representing e/n as a continued fraction but \
taking it one value of q at a time. To ensure that we overestimate e/n each \
time we use a trick to make sure that the representation is bigger than the \
original - we add 1 to the final q only when on an even iteration. You'll \
have to consult a book on continued fractions because I have no time to try \
to explain it.  The testing phase painstackingly attempts to get d from the \
estimate. Along the way there are clues that allow the test to be aborted if \
on the wrong track. Enjoy\
\>", "Text"],

Cell[BoxData[{
    \(CFrac[e_, pq_, lim_:  10] := 
      Module[{q, r, n, d, i, frac}, \n\t\tq = {0}; \n\t\tr = {e/pq}; \n\t\t
        i = 0; \n\t\tn = {0}; \n\t\td = {1}; \n\t\t
        While[i < lim, \n\t\t\t (*\ Ensure\ that\ we\ overestimate\ *) \n
          \t\t\tIf[EvenQ[i], \n\t\t\t\t
            frac = Normal[
                ContinuedFractionForm[
                  ReplacePart[q, 
                    q\[LeftDoubleBracket]i + 1\[RightDoubleBracket] + 1, 
                    i + 1]]], \n\t\t\t\t
            frac = Normal[ContinuedFractionForm[q]]]; \n\t\t\t
          res = Test[frac, e, pq]; \n\t\t\t (*Print[res]; *) \n\t\t\t
          If[IntegerQ[res], Return[res]]; \n
          \t\t\t (*\ Now\ to\ calculate\ the\ next\ q, \ r\ *) \n\t\t\t
          i = i + 1; \n\t\t\t
          q = Insert[q, Floor[1/r\[LeftDoubleBracket]i\[RightDoubleBracket]], 
              i + 1]; \n\t\t\t
          r = Insert[r, 
              \((1/r\[LeftDoubleBracket]i\[RightDoubleBracket])\) - 
                q\[LeftDoubleBracket]i + 1\[RightDoubleBracket], i + 1]; \n
          \t\t\t (*Print["\<q=\>", q]*) \n\t\t\t\t\t]; "\<nothing\>"]\), 
    \(Test[frac_, e_, pq_] := 
      Module[{k, dg, edg, pm1qm1, g, ppq, pmq, d}, \n
        \t\t (*Print["\<testing \>", frac]; *) \n\t\tk = Numerator[frac]; \n
        \t\tdg = Denominator[frac]; \n\t\tedg = e*dg; \n\t\t
        pm1qm1 = Floor[edg/k]; \n\t\tg = Mod[edg, k]; \n\t\t
        ppq = \((pq - pm1qm1 + 1)\)/2; \n\t\t
        If[\[Not] IntegerQ[ppq] \[Or] ppq \[Equal] 0, 
          Return[String["\<(p+q)/2=\>", ppq, "\< is not an Integer\>"]]]; \n
        \t\tpmq = ppq\^2 - pq; \n\t\tIf[IntegerQ[\@pmq], Return[dg/g]]; \n\t\t
        False]\n\t\t\)}], "Input"],

Cell[CellGroupData[{

Cell["First Test", "Subtitle"],

Cell["\<\
I will start by testing using the numbers given in the article by Wiener \
(\"Cryptanalysis of short RSA encrypting exponents\")\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(ContinuedFraction[2621/8927]\)], "Input"],

Cell[BoxData[
    TagBox[
      StyleBox[
        \(0 + 1\/\(3 + 
              1\/\(2 + 
                  1\/\(2 + 1\/\(6 + 1\/\(3 + 1\/\(8 + 1\/3\)\)\)\)\)\)\),
        ScriptSizeMultipliers->1],
      (ContinuedFractionForm[ {0, 3, 2, 2, 6, 3, 8, 3}]&)]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(CFrac[2621, 8927]\)], "Input"],

Cell[BoxData[
    \(5\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Another Example", "Subtitle"],

Cell["\<\
For some reason I get better results when I do the calculations by hand.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(p1 = GenerateRandomPrime[50]\), 
    \(q1 = GenerateRandomPrime[51]\)}], "Input"],

Cell[BoxData[
    \(83144395672638490684231235600579959253369501246641\)], "Output"],

Cell[BoxData[
    \(288774508260985664435546905871213588346738460549309\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(n1 = p1\ q1\)], "Input"],

Cell[BoxData[
    \(240099819750230045579170258177210807070782841404431114340332890213206194\
41856646068645527873151121069\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(d1 = 17\)], "Input"],

Cell[BoxData[
    \(17\)], "Output"]
}, Open  ]],

Cell["\<\
There are two methods to get d. (yes this means there are several d for each \
e) One is using the lowest common multiple and the other just uses \
(p-1)(q-1). Any multiple of the following m1 will work ( and (p-1)(q-1) is \
such a multiple).\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(m1 = LCM[p1 - 1, q1 - 1]\)], "Input"],

Cell[BoxData[
    \(200083183125191704649308548481009005892319034503689495959411294976379163\
8642931189591493980432443760\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(e1 = PowerMod[d1, \(-1\), m1]\)], "Input"],

Cell[BoxData[
    \(117695990073642179205475616753534709348422961472758527034947820574340684\
6260547758583231753195555153\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(PowerMod[PowerMod[13212311, e1, n1], d1, n1]\)], "Input"],

Cell[BoxData[
    \(13212311\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(e1alt = PowerMod[d1, \(-1\), \((p1 - 1)\) \((q1 - 1)\)]\)], "Input"],

Cell[BoxData[
    \(211852782132555922569856110156362476827161330650965348662906077033813232\
32689859654498171557519992753\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(PowerMod[PowerMod[13212311, e1alt, n1], d1, n1]\)], "Input"],

Cell[BoxData[
    \(13212311\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(CFrac[e1, n1, 100]\)], "Input"],

Cell[BoxData[
    \("nothing"\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(CFrac[e1alt, n1, 4]\)], "Input"],

Cell[BoxData[
    \(17\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Big Example", "Subtitle"],

Cell["\<\
I figured out the porblem. The Specify exponent function was messed. It \
should work well now\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(kp = SpecifyExponent[75, GenerateRandomPrime[25]]\)], "Input"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \("Generating \!\(\(p\^--\)\) Strong prime"\)], "Print"],

Cell[BoxData[
    \(RSAKey[1112991242607556555952689, 
      785374335709601678479678232053316923760926191637707412877393224536163525\
772449738983234270448352240556829192991178595754330110587501195595091701010519\
953, 1149081380761424551807301435326527303953903264049069756345184608711955290\
379804444414704871804816617220056069512921383658058101833512537851377320873515\
521]\)], "Output"]
}, Open  ]],

Cell["Reverse public and private", "Text"],

Cell[BoxData[
    \(pub = GetPrivate[kp]; \nprv = GetPublic[kp]; \ne = GetExponent[pub]; \n
    d = GetExponent[prv]; \nn = GetModulus[kp]; \)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(n > e\ d\)], "Input"],

Cell[BoxData[
    \(False\)], "Output"]
}, Open  ]],

Cell["I apologise for the ridiculous output", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(CFrac[e, n, 80]\)], "Input"],

Cell[BoxData[
    \(1112991242607556555952689\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(% == d\)], "Input"],

Cell[BoxData[
    \(True\)], "Output"]
}, Open  ]],

Cell["Let's just say it works. And its fasst too.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[CFrac[e, n, 80]]\)], "Input"],

Cell[BoxData[
    \({1.76000000000021827`\ Second, 1112991242607556555952689}\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"Microsoft Windows 3.0",
ScreenRectangle->{{0, 1024}, {0, 712}},
WindowSize->{690, 575},
WindowMargins->{{4, Automatic}, {Automatic, 5}}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "S5.67.1"->{
    Cell[1942, 58, 74, 1, 30, "Input",
      CellTags->"S5.67.1"]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"S5.67.1", 11119, 406}
  }
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 46, 0, 105, "Title"],
Cell[1780, 53, 159, 3, 52, "Text"],
Cell[1942, 58, 74, 1, 30, "Input",
  CellTags->"S5.67.1"],
Cell[2019, 61, 86, 2, 33, "Text"],

Cell[CellGroupData[{
Cell[2130, 67, 56, 1, 30, "Input"],
Cell[2189, 70, 170, 4, 64, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2396, 79, 75, 1, 30, "Input"],
Cell[2474, 82, 170, 4, 64, "Output"]
}, Open  ]],
Cell[2659, 89, 30, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[2714, 93, 45, 1, 30, "Input"],
Cell[2762, 96, 39, 1, 42, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2838, 102, 76, 1, 30, "Input"],
Cell[2917, 105, 39, 1, 42, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2993, 111, 44, 1, 30, "Input"],
Cell[3040, 114, 38, 1, 29, "Output"]
}, Open  ]],
Cell[3093, 118, 818, 12, 128, "Text"],
Cell[3914, 132, 1717, 31, 675, "Input"],

Cell[CellGroupData[{
Cell[5656, 167, 30, 0, 64, "Subtitle"],
Cell[5689, 169, 152, 3, 33, "Text"],

Cell[CellGroupData[{
Cell[5866, 176, 61, 1, 30, "Input"],
Cell[5930, 179, 269, 7, 112, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6236, 191, 50, 1, 30, "Input"],
Cell[6289, 194, 35, 1, 29, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[6373, 201, 35, 0, 64, "Subtitle"],
Cell[6411, 203, 96, 2, 33, "Text"],

Cell[CellGroupData[{
Cell[6532, 209, 102, 2, 50, "Input"],
Cell[6637, 213, 84, 1, 29, "Output"],
Cell[6724, 216, 85, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6846, 222, 44, 1, 30, "Input"],
Cell[6893, 225, 137, 2, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7067, 232, 40, 1, 30, "Input"],
Cell[7110, 235, 36, 1, 29, "Output"]
}, Open  ]],
Cell[7161, 239, 266, 5, 52, "Text"],

Cell[CellGroupData[{
Cell[7452, 248, 57, 1, 30, "Input"],
Cell[7512, 251, 136, 2, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7685, 258, 62, 1, 30, "Input"],
Cell[7750, 261, 136, 2, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7923, 268, 77, 1, 30, "Input"],
Cell[8003, 271, 42, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8082, 277, 88, 1, 30, "Input"],
Cell[8173, 280, 137, 2, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8347, 287, 80, 1, 30, "Input"],
Cell[8430, 290, 42, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8509, 296, 51, 1, 30, "Input"],
Cell[8563, 299, 43, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8643, 305, 52, 1, 30, "Input"],
Cell[8698, 308, 36, 1, 29, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[8783, 315, 31, 0, 64, "Subtitle"],
Cell[8817, 317, 118, 3, 33, "Text"],

Cell[CellGroupData[{
Cell[8960, 324, 82, 1, 30, "Input"],
Cell[9045, 327, 74, 1, 25, "Print"],
Cell[9122, 330, 74, 1, 25, "Print"],
Cell[9199, 333, 393, 6, 105, "Output"]
}, Open  ]],
Cell[9607, 342, 42, 0, 33, "Text"],
Cell[9652, 344, 153, 2, 110, "Input"],

Cell[CellGroupData[{
Cell[9830, 350, 41, 1, 30, "Input"],
Cell[9874, 353, 39, 1, 29, "Output"]
}, Open  ]],
Cell[9928, 357, 53, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[10006, 361, 48, 1, 30, "Input"],
Cell[10057, 364, 59, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10153, 370, 39, 1, 30, "Input"],
Cell[10195, 373, 38, 1, 29, "Output"]
}, Open  ]],
Cell[10248, 377, 59, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[10332, 381, 56, 1, 30, "Input"],
Cell[10391, 384, 91, 1, 29, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

