(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[      7353,        274]*)
(*NotebookOutlinePosition[      8002,        297]*)
(*  CellTagsIndexPosition[      7958,        293]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Martin Gardners Challenge", "Title"],

Cell["\<\
In 1977, Martin Gardner published a challenge in his Mathematical Games \
article in the Scientific American magazine. The article described the RSA \
system and gave a message and a public key. The winner (i.e. the one who \
revealed the message plaintext) was to receive $100.\
\>", "Text"],

Cell["The public ke was as follows:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(e = 9007\)], "Input"],

Cell[BoxData[
    \(9007\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(n = 
      114381625757888867669235779976146612010218296721242362562561842935706935\
245733897830597123563958705058989075147599290026879543541\)], "Input"],

Cell[BoxData[
    \(114381625757888867669235779976146612010218296721242362562561842935706935\
245733897830597123563958705058989075147599290026879543541\)], "Output"]
}, Open  ]],

Cell["\<\
He described how to encode a message. Encode 'A' as '01', 'B' as '02' etc. \
and use '00' for the space character. So the phrase \"ITS ALL GREEK TO ME\" \
becomes\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(m = 9201900011212000718050511002015001305\)], "Input"],

Cell[BoxData[
    \(9201900011212000718050511002015001305\)], "Output"]
}, Open  ]],

Cell["He then showed how to encrypt it", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(PowerMod[m, e, n]\)], "Input"],

Cell[BoxData[
    \(199935131497805100452317122740260647423204017058391463103703717406259716\
08948927504309920962672582675012893554461353823769748026\)], "Output"]
}, Open  ]],

Cell["\<\
To prove that the system worked and was invertible, he published a signiture \
calculated by Rivest with the corresponding secret key.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(sig = 
      167178611503808442460152713891683982454369010323583112178350384469290626\
55448792237114490509578608655662496577974840004057020373\)], "Input"],

Cell[BoxData[
    \(167178611503808442460152713891683982454369010323583112178350384469290626\
55448792237114490509578608655662496577974840004057020373\)], "Output"]
}, Open  ]],

Cell["\<\
If this is a signature, the public key should decrypt it as follows\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(mess = PowerMod[sig, e, n]\)], "Input"],

Cell[BoxData[
    \(609181920001915122205180023091419001514050008211404180504000415121201181\
9\)], "Output"]
}, Open  ]],

Cell["\<\
To decode it we split it up, convert to ASCII code (first the letters then \
the spaces), and then convert to a string.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(l1 = IntegerDigits[mess, 100]\)], "Input"],

Cell[BoxData[
    \({6, 9, 18, 19, 20, 0, 19, 15, 12, 22, 5, 18, 0, 23, 9, 14, 19, 0, 15, 
      14, 5, 0, 8, 21, 14, 4, 18, 5, 4, 0, 4, 15, 12, 12, 1, 18, 19}\)], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(l2 = Map[Plus[#, 64]&, l1]\)], "Input"],

Cell[BoxData[
    \({70, 73, 82, 83, 84, 64, 83, 79, 76, 86, 69, 82, 64, 87, 73, 78, 83, 
      64, 79, 78, 69, 64, 72, 85, 78, 68, 82, 69, 68, 64, 68, 79, 76, 76, 65, 
      82, 83}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(l3 = Map[If[# \[Equal] 64, 32, #]&, l2]\)], "Input"],

Cell[BoxData[
    \({70, 73, 82, 83, 84, 32, 83, 79, 76, 86, 69, 82, 32, 87, 73, 78, 83, 
      32, 79, 78, 69, 32, 72, 85, 78, 68, 82, 69, 68, 32, 68, 79, 76, 76, 65, 
      82, 83}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FromCharacterCode[l3]\)], "Input"],

Cell[BoxData[
    \("FIRST SOLVER WINS ONE HUNDRED DOLLARS"\)], "Output"]
}, Open  ]],

Cell["\<\
That's the details of the prize. The ciphertext that was to be decrypted \
was\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(ct = 
      968696137546220614771409222543558829057599911245743198746951209308162982\
25145708356931476622883989628013391990551829945157815154\)], "Input"],

Cell[BoxData[
    \(968696137546220614771409222543558829057599911245743198746951209308162982\
25145708356931476622883989628013391990551829945157815154\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Solution", "Subtitle"],

Cell["\<\
The solution was found by a team who distributed the effort over the \
Internet. The prime factorization was:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(p = 3490529510847650949147849619903898133417764638493387843990820577
      \)], "Input"],

Cell[BoxData[
    \(3490529510847650949147849619903898133417764638493387843990820577\)], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(q = n/p\)], "Input"],

Cell[BoxData[
    \(32769132993266709549961988190834461413177642967992942539798288533\)], 
  "Output"]
}, Open  ]],

Cell["The decrypting exponent is therefore", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(d = PowerMod[e, \(-1\), \((p - 1)\) \((q - 1)\)]\)], "Input"],

Cell[BoxData[
    \(106698614368578024442868771328920154780709906633937862801226224496631063\
125911774470873340168597462306553968544513277109053606095\)], "Output"]
}, Open  ]],

Cell["So we can decrypt", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(pt = PowerMod[ct, d, n]\)], "Input"],

Cell[BoxData[
    \(200805001301070903002315180419000118050019172105011309190800151919090618\
010705\)], "Output"]
}, Open  ]],

Cell["And convert it", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(FromCharacterCode[
      \(If[# \[Equal] 64, 32, #]&\)/@
        \((\(Plus[#, 64]&\)/@IntegerDigits[pt, 100])\)]\)], "Input"],

Cell[BoxData[
    \("THE MAGIC WORDS ARE SQUEAMISH OSSIFRAGE"\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"Microsoft Windows 3.0",
ScreenRectangle->{{0, 1024}, {0, 740}},
WindowSize->{499, 635},
WindowMargins->{{0, Automatic}, {Automatic, 0}}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 42, 0, 105, "Title"],
Cell[1776, 53, 302, 5, 71, "Text"],
Cell[2081, 60, 45, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[2151, 64, 41, 1, 30, "Input"],
Cell[2195, 67, 38, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2270, 73, 175, 3, 90, "Input"],
Cell[2448, 78, 165, 2, 67, "Output"]
}, Open  ]],
Cell[2628, 83, 186, 4, 52, "Text"],

Cell[CellGroupData[{
Cell[2839, 91, 74, 1, 30, "Input"],
Cell[2916, 94, 71, 1, 29, "Output"]
}, Open  ]],
Cell[3002, 98, 48, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[3075, 102, 50, 1, 30, "Input"],
Cell[3128, 105, 164, 2, 67, "Output"]
}, Open  ]],
Cell[3307, 110, 158, 3, 52, "Text"],

Cell[CellGroupData[{
Cell[3490, 117, 176, 3, 90, "Input"],
Cell[3669, 122, 164, 2, 67, "Output"]
}, Open  ]],
Cell[3848, 127, 91, 2, 33, "Text"],

Cell[CellGroupData[{
Cell[3964, 133, 59, 1, 30, "Input"],
Cell[4026, 136, 109, 2, 48, "Output"]
}, Open  ]],
Cell[4150, 141, 143, 3, 52, "Text"],

Cell[CellGroupData[{
Cell[4318, 148, 62, 1, 30, "Input"],
Cell[4383, 151, 176, 3, 67, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4596, 159, 59, 1, 30, "Input"],
Cell[4658, 162, 196, 3, 67, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4891, 170, 72, 1, 30, "Input"],
Cell[4966, 173, 196, 3, 67, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5199, 181, 54, 1, 30, "Input"],
Cell[5256, 184, 73, 1, 29, "Output"]
}, Open  ]],
Cell[5344, 188, 102, 3, 33, "Text"],

Cell[CellGroupData[{
Cell[5471, 195, 175, 3, 90, "Input"],
Cell[5649, 200, 164, 2, 67, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5850, 207, 28, 0, 64, "Subtitle"],
Cell[5881, 209, 133, 3, 52, "Text"],

Cell[CellGroupData[{
Cell[6039, 216, 108, 2, 70, "Input"],
Cell[6150, 220, 101, 2, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6288, 227, 40, 1, 30, "Input"],
Cell[6331, 230, 102, 2, 48, "Output"]
}, Open  ]],
Cell[6448, 235, 52, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[6525, 239, 81, 1, 30, "Input"],
Cell[6609, 242, 165, 2, 67, "Output"]
}, Open  ]],
Cell[6789, 247, 33, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[6847, 251, 56, 1, 30, "Input"],
Cell[6906, 254, 114, 2, 48, "Output"]
}, Open  ]],
Cell[7035, 259, 30, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[7090, 263, 145, 3, 50, "Input"],
Cell[7238, 268, 75, 1, 29, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

