(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     24993,        721]*)
(*NotebookOutlinePosition[     25645,        744]*)
(*  CellTagsIndexPosition[     25601,        740]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Elliptic Curves", "Title"],

Cell[BoxData[
    \(<< \ Graphics`ImplicitPlot`\)], "Input"],

Cell[TextData[{
  "Define an elliptic curve based on a and b where\n\t",
  Cell[BoxData[
      \(TraditionalForm\`y\^2\  = \ x\^3\  + \ a\ x\  + \ b\)]],
  "\nas long as\n\t4 ",
  Cell[BoxData[
      \(TraditionalForm\`a\^3\  + \ 27\ b\^2\  \[NotEqual] \ 0\)]],
  "\n\nWe define a test for this based on an EC[a,b] structure that \
represents our elliptic curve."
}], "Text"],

Cell[BoxData[
    \(IsValidCurve[EC[a_, b_]] := 4  a\^3 + 27  b\^2 \[NotEqual] 0\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(IsValidCurve[EC[\(-6\), 6]]\)], "Input"],

Cell[BoxData[
    \(True\)], "Output"]
}, Open  ]],

Cell["\<\
Define a function to return the eqution for the curve and another to get the \
curve itself.\
\>", "Text"],

Cell[BoxData[
    \(GetEquation[EC[a_, b_]] := x\^3 + a\ x + b\  - y\^2 == 0\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(e = GetEquation[EC[\(-6\), 6]]\)], "Input"],

Cell[BoxData[
    \(6 - 6\ x + x\^3 - y\^2 == 0\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(GetCurve[a_, b_] := EC[a, b]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(c = GetCurve[\(-6\), 6]\)], "Input"],

Cell[BoxData[
    \(EC[\(-6\), 6]\)], "Output"]
}, Open  ]],

Cell["Define a function to plot the curve", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(PlotCurve[EC[a_, b_], min_: - 110, max_:  10] := 
      ImplicitPlot[GetEquation[EC[a, b]], {x, min, max}]\)], "Input"],

Cell[BoxData[
    \(General::"spell1" \( : \ \) 
      "Possible spelling error: new symbol name \"\!\(min\)\" is similar to \
existing symbol \"\!\(Min\)\"."\)], "Message"],

Cell[BoxData[
    \(General::"spell1" \( : \ \) 
      "Possible spelling error: new symbol name \"\!\(max\)\" is similar to \
existing symbol \"\!\(Max\)\"."\)], "Message"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(PlotCurve[c, \(-5\), 5]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 2.56135 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.369371 0.121364 1.28068 0.121364 [
[.12664 1.26818 -6 -9 ]
[.12664 1.26818 6 0 ]
[.6121 1.26818 -3 -9 ]
[.6121 1.26818 3 0 ]
[.85483 1.26818 -3 -9 ]
[.85483 1.26818 3 0 ]
[.35687 .06704 -18 -4.5 ]
[.35687 .06704 0 4.5 ]
[.35687 .67386 -12 -4.5 ]
[.35687 .67386 0 4.5 ]
[.35687 1.88749 -6 -4.5 ]
[.35687 1.88749 0 4.5 ]
[.35687 2.49431 -12 -4.5 ]
[.35687 2.49431 0 4.5 ]
[ 0 0 0 0 ]
[ 1 2.56135 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
.12664 1.28068 m
.12664 1.28693 L
s
[(-2)] .12664 1.26818 0 1 Mshowa
.6121 1.28068 m
.6121 1.28693 L
s
[(2)] .6121 1.26818 0 1 Mshowa
.85483 1.28068 m
.85483 1.28693 L
s
[(4)] .85483 1.26818 0 1 Mshowa
.125 Mabswid
.18733 1.28068 m
.18733 1.28443 L
s
.24801 1.28068 m
.24801 1.28443 L
s
.30869 1.28068 m
.30869 1.28443 L
s
.43005 1.28068 m
.43005 1.28443 L
s
.49074 1.28068 m
.49074 1.28443 L
s
.55142 1.28068 m
.55142 1.28443 L
s
.67278 1.28068 m
.67278 1.28443 L
s
.73346 1.28068 m
.73346 1.28443 L
s
.79414 1.28068 m
.79414 1.28443 L
s
.06596 1.28068 m
.06596 1.28443 L
s
.00528 1.28068 m
.00528 1.28443 L
s
.91551 1.28068 m
.91551 1.28443 L
s
.97619 1.28068 m
.97619 1.28443 L
s
.25 Mabswid
0 1.28068 m
1 1.28068 L
s
.36937 .06704 m
.37562 .06704 L
s
[(-10)] .35687 .06704 1 0 Mshowa
.36937 .67386 m
.37562 .67386 L
s
[(-5)] .35687 .67386 1 0 Mshowa
.36937 1.88749 m
.37562 1.88749 L
s
[(5)] .35687 1.88749 1 0 Mshowa
.36937 2.49431 m
.37562 2.49431 L
s
[(10)] .35687 2.49431 1 0 Mshowa
.125 Mabswid
.36937 .1884 m
.37312 .1884 L
s
.36937 .30977 m
.37312 .30977 L
s
.36937 .43113 m
.37312 .43113 L
s
.36937 .55249 m
.37312 .55249 L
s
.36937 .79522 m
.37312 .79522 L
s
.36937 .91658 m
.37312 .91658 L
s
.36937 1.03795 m
.37312 1.03795 L
s
.36937 1.15931 m
.37312 1.15931 L
s
.36937 1.40204 m
.37312 1.40204 L
s
.36937 1.5234 m
.37312 1.5234 L
s
.36937 1.64477 m
.37312 1.64477 L
s
.36937 1.76613 m
.37312 1.76613 L
s
.36937 2.00886 m
.37312 2.00886 L
s
.36937 2.13022 m
.37312 2.13022 L
s
.36937 2.25159 m
.37312 2.25159 L
s
.36937 2.37295 m
.37312 2.37295 L
s
.25 Mabswid
.36937 0 m
.36937 2.56135 L
s
0 0 m
1 0 L
1 2.56135 L
0 2.56135 L
closepath
clip
newpath
.5 Mabswid
.02381 1.28051 m
.02788 1.18624 L
.04004 1.09665 L
.06006 1.01654 L
.08761 .95017 L
.12221 .90127 L
.16328 .87285 L
.21011 .86708 L
.2619 .8852 L
.31777 .92738 L
.37675 .99257 L
.43784 1.0778 L
.5 1.17288 L
.56216 1.19683 L
.62325 1.08489 L
.68223 .9447 L
.7381 .79804 L
.78989 .65229 L
.83672 .51313 L
.87779 .38569 L
.91239 .27454 L
.93994 .18367 L
.95996 .11634 L
.97212 .07495 L
.97619 .06098 L
s
.02381 1.28084 m
.02788 1.37511 L
.04004 1.4647 L
.06006 1.54482 L
.08761 1.61118 L
.12221 1.66008 L
.16328 1.6885 L
.21011 1.69427 L
.2619 1.67616 L
.31777 1.63397 L
.37675 1.56878 L
.43784 1.48355 L
.5 1.38847 L
.56216 1.36452 L
.62325 1.47646 L
.68223 1.61665 L
.7381 1.76331 L
.78989 1.90907 L
.83672 2.04822 L
.87779 2.17566 L
.91239 2.28681 L
.93994 2.37768 L
.95996 2.44501 L
.97212 2.4864 L
.97619 2.50037 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{112.438, 287.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40001`00017a000`40O003h00O:Goo00<007ooOol0
A7oo000YOol00`00Oomoo`14Ool002Uoo`03001oogoo04Aoo`00:Goo00<007ooOol0A7oo000YOol0
0`00Oomoo`14Ool002Uoo`03001oogoo04Aoo`007Goo1@000Woo00<007ooOol00Woo00<007ooOol0
A7oo000OOol01`00Oomoogoo001oo`0000=oo`03001oogoo041oo`03001oogoo005oo`006Goo0`00
0goo00L007ooOomoo`00Ool00003Ool00`00Oomoo`10Ool00`00Oomoo`01Ool001moo`07001oogoo
Ool007oo00000goo00<007ooOol0@7oo00<007ooOol00Goo000OOol01`00Oomoogoo001oo`0000=o
o`03001oogoo041oo`03001oogoo005oo`007Woo0P0017oo00<007ooOol00Woo00<007ooOol0?goo
00<007ooOol00Woo000YOol00`00Oomoo`0oOol00`00Oomoo`02Ool002Uoo`03001oogoo03ioo`03
001oogoo00=oo`00:Goo00<007ooOol0?Woo00<007ooOol00goo000YOol00`00Oomoo`0nOol00`00
Oomoo`03Ool002Uoo`03001oogoo03eoo`03001oogoo00Aoo`00:Goo00<007ooOol0?Goo00<007oo
Ool017oo000YOol00`00Oomoo`0mOol00`00Oomoo`04Ool002Uoo`03001oogoo03eoo`03001oogoo
00Aoo`00:Goo00<007ooOol0?7oo00<007ooOol01Goo000YOol00`00Oomoo`0lOol00`00Oomoo`05
Ool002Uoo`03001oogoo03aoo`03001oogoo00Eoo`00:Goo00<007ooOol0>goo00<007ooOol01Woo
000YOol00`00Oomoo`0kOol00`00Oomoo`06Ool002Uoo`03001oogoo03]oo`03001oogoo00Ioo`00
:Goo00<007ooOol0>goo00<007ooOol01Woo000YOol00`00Oomoo`0jOol00`00Oomoo`07Ool002Uo
o`03001oogoo03Yoo`03001oogoo00Moo`00:Goo00<007ooOol0>Woo00<007ooOol01goo000YOol0
0`00Oomoo`0iOol00`00Oomoo`08Ool002Uoo`03001oogoo03Uoo`03001oogoo00Qoo`00:Goo00<0
07ooOol0>Goo00<007ooOol027oo000YOol00`00Oomoo`0hOol00`00Oomoo`09Ool002Uoo`03001o
ogoo03Qoo`03001oogoo00Uoo`00:Goo00<007ooOol0>7oo00<007ooOol02Goo000YOol00`00Oomo
o`0gOol00`00Oomoo`0:Ool002Uoo`03001oogoo03Moo`03001oogoo00Yoo`00:Goo00<007ooOol0
=goo00<007ooOol02Woo000YOol00`00Oomoo`0fOol00`00Oomoo`0;Ool002Uoo`03001oogoo03Io
o`03001oogoo00]oo`00:Goo00<007ooOol0=Woo00<007ooOol02goo000YOol00`00Oomoo`0eOol0
0`00Oomoo`0<Ool002Uoo`03001oogoo03Eoo`03001oogoo00aoo`00:Goo00<007ooOol0=Goo00<0
07ooOol037oo000YOol00`00Oomoo`0dOol00`00Oomoo`0=Ool002Uoo`03001oogoo03Aoo`03001o
ogoo00eoo`00:Goo00<007ooOol0=7oo00<007ooOol03Goo000YOol00`00Oomoo`0dOol00`00Oomo
o`0=Ool002Uoo`03001oogoo03=oo`03001oogoo00ioo`00:Goo00<007ooOol0<goo00<007ooOol0
3Woo000YOol00`00Oomoo`0cOol00`00Oomoo`0>Ool002Uoo`03001oogoo039oo`03001oogoo00mo
o`00:Goo00<007ooOol0<Woo00<007ooOol03goo000YOol00`00Oomoo`0bOol00`00Oomoo`0?Ool0
02Uoo`03001oogoo039oo`03001oogoo00moo`00:Goo00<007ooOol0<Goo00<007ooOol047oo000Y
Ool00`00Oomoo`0aOol00`00Oomoo`0@Ool002Uoo`03001oogoo035oo`03001oogoo011oo`00:Goo
00<007ooOol0<7oo00<007ooOol04Goo000YOol00`00Oomoo`0`Ool00`00Oomoo`0AOol002Uoo`03
001oogoo031oo`03001oogoo015oo`00:Goo00<007ooOol0;goo00<007ooOol04Woo000YOol00`00
Oomoo`0_Ool00`00Oomoo`0BOol002Uoo`03001oogoo02moo`03001oogoo019oo`00:Goo00<007oo
Ool0;goo00<007ooOol04Woo000YOol00`00Oomoo`0^Ool00`00Oomoo`0COol002Uoo`03001oogoo
02ioo`03001oogoo01=oo`00:Goo00<007ooOol0;Woo00<007ooOol04goo000YOol00`00Oomoo`0]
Ool00`00Oomoo`0DOol002Uoo`03001oogoo02eoo`03001oogoo01Aoo`00:Goo00<007ooOol0;Goo
00<007ooOol057oo000YOol00`00Oomoo`0/Ool00`00Oomoo`0EOol002Aoo`03001oogoo009oo`03
001oogoo02aoo`03001oogoo01Eoo`008goo00<007oo00000goo00<007ooOol0;7oo00<007ooOol0
5Goo000NOol30004Ool01@00Oomoogoo0000;Goo00<007ooOol05Woo000SOol20004Ool00`00Oomo
o`0[Ool00`00Oomoo`0FOol002=oo`03001oogoo00=oo`03001oogoo02]oo`03001oogoo01Ioo`00
8goo0`000goo00<007ooOol0:Woo00<007ooOol05goo000YOol00`00Oomoo`0ZOol00`00Oomoo`0G
Ool002Uoo`03001oogoo02Uoo`03001oogoo01Qoo`00:Goo00<007ooOol0:Goo00<007ooOol067oo
000YOol00`00Oomoo`0YOol00`00Oomoo`0HOol002Uoo`03001oogoo02Qoo`03001oogoo01Uoo`00
:Goo00<007ooOol0:7oo00<007ooOol06Goo000YOol00`00Oomoo`0XOol00`00Oomoo`0IOol002Uo
o`03001oogoo02Moo`03001oogoo01Yoo`00:Goo00<007ooOol09goo00<007ooOol06Woo000YOol0
0`00Oomoo`0VOol00`00Oomoo`0KOol002Uoo`03001oogoo02Ioo`03001oogoo01]oo`00:Goo00<0
07ooOol09Woo00<007ooOol06goo000YOol00`00Oomoo`0UOol00`00Oomoo`0LOol002Uoo`03001o
ogoo02Eoo`03001oogoo01aoo`00:Goo00<007ooOol09Goo00<007ooOol077oo000YOol00`00Oomo
o`0TOol00`00Oomoo`0MOol002Uoo`03001oogoo02Aoo`03001oogoo01eoo`00:Goo00<007ooOol0
97oo00<007ooOol07Goo000EOol4000@Ool00`00Oomoo`0SOol00`00Oomoo`0NOol0019oo`<000Ao
o`<000eoo`03001oogoo02=oo`03001oogoo01ioo`0047oo0P002Woo0P002goo00<007ooOol08Woo
00<007ooOol07goo000>Ool2000>Ool00`00Oomoo`08Ool00`00Oomoo`0ROol00`00Oomoo`0OOol0
00eoo`03001oogoo00moo`8000Qoo`03001oogoo029oo`03001oogoo01moo`0037oo00<007ooOol0
4Woo00<007ooOol01Goo00<007ooOol08Goo00<007ooOol087oo000;Ool00`00Oomoo`0DOol00`00
Oomoo`04Ool00`00Oomoo`0QOol00`00Oomoo`0POol000]oo`03001oogoo01Eoo`03001oogoo00=o
o`03001oogoo025oo`03001oogoo021oo`002Woo00<007ooOol05goo00<007ooOol00Woo00<007oo
Ool087oo00<007ooOol08Goo0009Ool00`00Oomoo`0IOol01@00Oomoogoo00008Woo00<007ooOol0
8Goo0009Ool00`00Oomoo`0JOol01000Oomoo`008Woo00<007ooOol08Goo0008Ool00`00Oomoo`0K
Ool01000Oomoo`008Goo00<007ooOol08Woo0008Ool00`00Oomoo`0LOol00`00Ool0000QOol00`00
Oomoo`0ROol000Qoo`03001oogoo01eoo`80021oo`03001oogoo02=oo`001goo00<007ooOol07goo
00<007ooOol07Woo00<007ooOol08goo0007Ool00`00Oomoo`0OOol2000NOol00`00Oomoo`0TOol0
00Ioo`03001oogoo021oo`03001oo`0001eoo`03001oogoo02Aoo`001Woo00<007ooOol087oo00<0
07oo000077oo00<007ooOol09Goo0006Ool00`00Oomoo`0POol01000Oomoo`006goo00<007ooOol0
9Goo0006Ool00`00Oomoo`0POol01@00Oomoogoo00006Goo00<007ooOol09Woo0005Ool00`00Oomo
o`0QOol00`00Oomoo`02Ool00`00Oomoo`0FOol00`00Oomoo`0VOol000Eoo`03001oogoo025oo`03
001oogoo009oo`03001oogoo01Eoo`03001oogoo02Moo`001Goo00<007ooOol08Goo00<007ooOol0
0goo00<007ooOol057oo00<007ooOol09goo0005Ool00`00Oomoo`0QOol00`00Oomoo`04Ool00`00
Oomoo`0BOol00`00Oomoo`0XOol000Aoo`03001oogoo029oo`03001oogoo00Eoo`03001oogoo015o
o`03001oogoo02Qoo`0017oo00<007ooOol08Woo00<007ooOol01Goo00<007ooOol047oo00<007oo
Ool0:Goo0004Ool00`00Oomoo`0ROol00`00Oomoo`06Ool00`00Oomoo`0?Ool00`00Oomoo`0YOol0
00Aoo`03001oogoo029oo`03001oogoo00Moo`03001oogoo00eoo`03001oogoo02Yoo`0017oo00<0
07ooOol08Woo00<007ooOol01goo00<007ooOol03Goo00<007ooOol0:Woo0004Ool00`00Oomoo`0R
Ool00`00Oomoo`08Ool00`00Oomoo`0;Ool00`00Oomoo`0[Ool000Aoo`03001oogoo029oo`03001o
ogoo00Qoo`03001oogoo00]oo`03001oogoo02]oo`0017oo00<007ooOol08Woo00<007ooOol02Goo
00<007ooOol02Goo00<007ooOol0;7oo0003Ool00`00Oomoo`0SOol00`00Oomoo`0:Ool00`00Oomo
o`08Ool00`00Oomoo`0/Ool000=oo`03001oogoo02=oo`03001oogoo00Yoo`03001oogoo00Moo`03
001oogoo02eoo`000goo00<007ooOol08goo00<007ooOol02goo0P001goo00<007ooOol0;Goo0003
Ool00`00Oomoo`0SOol00`00Oomoo`0=Ool20004Ool00`00Oomoo`0^Ool000=oo`03001oogoo02=o
o`03001oogoo00moo`80009oo`03001oogoo02ioo`000goo00<007ooOol02Goo0`005goo00<007oo
Ool04Goo0P000goo0`006Goo0`003goo0003Ool00`00Oomoo`09Ool00`00Oomoo`0GOol00`00Oomo
o`0FOol00`00Oomoo`0JOol00`00Oomoo`0>Ool000=oo`03001oogoo00Aoo`<000=oo`03001oogoo
01Ioo`03001oogoo01Moo`03001oogoo01Moo`@000moo`000goo00<007ooOol02Woo00<007ooOol0
5Woo00<007ooOol05goo00<007ooOol05goo00<007oo000047oo0003Ool00`00Oomoo`09Ool00`00
Ool0000GOol00`00Oomoo`0FOol00`00Ool0000IOol2000@Ool0009oo`03001oogoo00]oo`03001o
ogoo01Ioo`03001oogoo01Moo`03001oogoo01Uoo`03001oogoo00ioo`000Woo00<007ooOol097oo
00<007ooOol0A7oo0002Ool00`00Oomoo`0TOol00`00Oomoo`14Ool0009oo`03001oogoo02Aoo`03
001oogoo04Aoo`00L0000002Ool00`00Oomoo`09Ool00`00Oomoo`0HOol00`00Oomoo`0HOol00`00
Oomoo`0HOol00`00Oomoo`0>Ool0009oo`03001oogoo02Aoo`03001oogoo04Aoo`000Woo00<007oo
Ool097oo00<007ooOol0A7oo0002Ool00`00Oomoo`0TOol00`00Oomoo`14Ool0009oo`03001oogoo
02Aoo`03001oogoo04Aoo`000goo00<007ooOol08goo00<007ooOol0A7oo0003Ool00`00Oomoo`0S
Ool00`00Oomoo`14Ool000=oo`03001oogoo02=oo`03001oogoo04Aoo`000goo00<007ooOol08goo
00<007ooOol04Goo0P00<Goo0003Ool00`00Oomoo`0SOol00`00Oomoo`0?Ool20002Ool00`00Oomo
o`0^Ool000=oo`03001oogoo02=oo`03001oogoo00eoo`8000Aoo`03001oogoo02ioo`000goo00<0
07ooOol08goo00<007ooOol02goo0P001goo00<007ooOol0;Goo0003Ool00`00Oomoo`0SOol00`00
Oomoo`0:Ool00`00Oomoo`07Ool00`00Oomoo`0]Ool000=oo`03001oogoo02=oo`03001oogoo00Yo
o`03001oogoo00Qoo`03001oogoo02aoo`000goo00<007ooOol08goo00<007ooOol02Goo00<007oo
Ool02Goo00<007ooOol0;7oo0004Ool00`00Oomoo`0ROol00`00Oomoo`08Ool00`00Oomoo`0;Ool0
0`00Oomoo`0[Ool000Aoo`03001oogoo029oo`03001oogoo00Qoo`03001oogoo00]oo`03001oogoo
02]oo`0017oo00<007ooOol08Woo00<007ooOol01goo00<007ooOol03Goo00<007ooOol0:Woo0004
Ool00`00Oomoo`0ROol00`00Oomoo`07Ool00`00Oomoo`0=Ool00`00Oomoo`0ZOol000Aoo`03001o
ogoo029oo`03001oogoo00Ioo`03001oogoo00moo`03001oogoo02Uoo`0017oo00<007ooOol08Woo
00<007ooOol01Goo00<007ooOol047oo00<007ooOol0:Goo0004Ool00`00Oomoo`0ROol00`00Oomo
o`05Ool00`00Oomoo`0AOol00`00Oomoo`0XOol000Eoo`03001oogoo025oo`03001oogoo00Aoo`03
001oogoo019oo`03001oogoo02Qoo`001Goo00<007ooOol08Goo00<007ooOol00goo00<007ooOol0
57oo00<007ooOol09goo0005Ool00`00Oomoo`0QOol00`00Oomoo`02Ool00`00Oomoo`0EOol00`00
Oomoo`0WOol000Eoo`03001oogoo025oo`03001oogoo009oo`03001oogoo01Ioo`03001oogoo02Io
o`001Woo00<007ooOol087oo00D007ooOomoo`0001Uoo`03001oogoo02Ioo`001Woo00<007ooOol0
87oo00@007ooOol001]oo`03001oogoo02Eoo`001Woo00<007ooOol087oo00<007oo000077oo00<0
07ooOol09Goo0006Ool00`00Oomoo`0POol00`00Ool0000MOol00`00Oomoo`0TOol000Moo`03001o
ogoo01moo`8001ioo`03001oogoo02Aoo`001goo00<007ooOol07goo00<007ooOol07Woo00<007oo
Ool08goo0008Ool00`00Oomoo`0MOol2000POol00`00Oomoo`0SOol000Qoo`03001oogoo01aoo`03
001oo`00025oo`03001oogoo029oo`0027oo00<007ooOol06goo00@007ooOol0025oo`03001oogoo
029oo`002Goo00<007ooOol06Woo00@007ooOol0029oo`03001oogoo025oo`002Goo00<007ooOol0
6Goo00D007ooOomoo`00029oo`03001oogoo025oo`002Woo00<007ooOol05goo00<007ooOol00Woo
00<007ooOol087oo00<007ooOol08Goo000;Ool00`00Oomoo`0EOol00`00Oomoo`03Ool00`00Oomo
o`0QOol00`00Oomoo`0POol000]oo`03001oogoo01Aoo`03001oogoo00Aoo`03001oogoo025oo`03
001oogoo021oo`0037oo00<007ooOol04Goo0P001goo00<007ooOol08Goo00<007ooOol087oo000=
Ool00`00Oomoo`0?Ool00`00Oomoo`07Ool00`00Oomoo`0ROol00`00Oomoo`0OOol000ioo`8000io
o`03001oogoo00Qoo`03001oogoo029oo`03001oogoo01moo`0047oo0P002Woo0P002goo00<007oo
Ool08Woo00<007ooOol07goo000BOol30004Ool3000=Ool00`00Oomoo`0SOol00`00Oomoo`0NOol0
01Eoo`@0011oo`03001oogoo02=oo`03001oogoo01ioo`00:Goo00<007ooOol097oo00<007ooOol0
7Goo000YOol00`00Oomoo`0TOol00`00Oomoo`0MOol002Uoo`03001oogoo02Aoo`03001oogoo01eo
o`00:Goo00<007ooOol09Goo00<007ooOol077oo000YOol00`00Oomoo`0UOol00`00Oomoo`0LOol0
02Uoo`03001oogoo02Eoo`03001oogoo01aoo`00:Goo00<007ooOol09Woo00<007ooOol06goo000Y
Ool00`00Oomoo`0VOol00`00Oomoo`0KOol002Uoo`03001oogoo02Ioo`03001oogoo01]oo`00:Goo
00<007ooOol09goo00<007ooOol06Woo000YOol00`00Oomoo`0WOol00`00Oomoo`0JOol002Uoo`03
001oogoo02Qoo`03001oogoo01Uoo`00:Goo00<007ooOol0:7oo00<007ooOol06Goo000YOol00`00
Oomoo`0XOol00`00Oomoo`0IOol002Uoo`03001oogoo02Uoo`03001oogoo01Qoo`00:Goo00<007oo
Ool0:Goo00<007ooOol067oo000YOol00`00Oomoo`0YOol00`00Oomoo`0HOol002Uoo`03001oogoo
02Yoo`03001oogoo01Moo`00:Goo00<007ooOol0:Woo00<007ooOol05goo000TOol00`00Oomoo`02
Ool00`00Oomoo`0[Ool00`00Oomoo`0FOol002=oo`03001oo`0000=oo`03001oogoo02]oo`03001o
ogoo01Ioo`009Goo00D007ooOomoo`0002eoo`03001oogoo01Ioo`008goo0P0017oo00<007ooOol0
;7oo00<007ooOol05Goo000SOol00`00Oomoo`03Ool00`00Oomoo`0/Ool00`00Oomoo`0EOol002=o
o`<000=oo`03001oogoo02aoo`03001oogoo01Eoo`00:Goo00<007ooOol0;Goo00<007ooOol057oo
000YOol00`00Oomoo`0]Ool00`00Oomoo`0DOol002Uoo`03001oogoo02eoo`03001oogoo01Aoo`00
:Goo00<007ooOol0;Woo00<007ooOol04goo000YOol00`00Oomoo`0^Ool00`00Oomoo`0COol002Uo
o`03001oogoo02ioo`03001oogoo01=oo`00:Goo00<007ooOol0;goo00<007ooOol04Woo000YOol0
0`00Oomoo`0_Ool00`00Oomoo`0BOol002Uoo`03001oogoo02moo`03001oogoo019oo`00:Goo00<0
07ooOol0;goo00<007ooOol04Woo000YOol00`00Oomoo`0`Ool00`00Oomoo`0AOol002Uoo`03001o
ogoo031oo`03001oogoo015oo`00:Goo00<007ooOol0<7oo00<007ooOol04Goo000YOol00`00Oomo
o`0aOol00`00Oomoo`0@Ool002Uoo`03001oogoo035oo`03001oogoo011oo`00:Goo00<007ooOol0
<Goo00<007ooOol047oo000YOol00`00Oomoo`0bOol00`00Oomoo`0?Ool002Uoo`03001oogoo039o
o`03001oogoo00moo`00:Goo00<007ooOol0<Woo00<007ooOol03goo000YOol00`00Oomoo`0bOol0
0`00Oomoo`0?Ool002Uoo`03001oogoo03=oo`03001oogoo00ioo`00:Goo00<007ooOol0<goo00<0
07ooOol03Woo000YOol00`00Oomoo`0cOol00`00Oomoo`0>Ool002Uoo`03001oogoo03Aoo`03001o
ogoo00eoo`00:Goo00<007ooOol0=7oo00<007ooOol03Goo000YOol00`00Oomoo`0dOol00`00Oomo
o`0=Ool002Uoo`03001oogoo03Aoo`03001oogoo00eoo`00:Goo00<007ooOol0=Goo00<007ooOol0
37oo000YOol00`00Oomoo`0eOol00`00Oomoo`0<Ool002Uoo`03001oogoo03Eoo`03001oogoo00ao
o`00:Goo00<007ooOol0=Woo00<007ooOol02goo000YOol00`00Oomoo`0fOol00`00Oomoo`0;Ool0
02Uoo`03001oogoo03Ioo`03001oogoo00]oo`00:Goo00<007ooOol0=goo00<007ooOol02Woo000Y
Ool00`00Oomoo`0gOol00`00Oomoo`0:Ool002Uoo`03001oogoo03Moo`03001oogoo00Yoo`00:Goo
00<007ooOol0>7oo00<007ooOol02Goo000YOol00`00Oomoo`0hOol00`00Oomoo`09Ool002Uoo`03
001oogoo03Qoo`03001oogoo00Uoo`00:Goo00<007ooOol0>Goo00<007ooOol027oo000YOol00`00
Oomoo`0iOol00`00Oomoo`08Ool002Uoo`03001oogoo03Uoo`03001oogoo00Qoo`00:Goo00<007oo
Ool0>Woo00<007ooOol01goo000YOol00`00Oomoo`0jOol00`00Oomoo`07Ool002Uoo`03001oogoo
03Yoo`03001oogoo00Moo`00:Goo00<007ooOol0>goo00<007ooOol01Woo000YOol00`00Oomoo`0k
Ool00`00Oomoo`06Ool002Uoo`03001oogoo03]oo`03001oogoo00Ioo`00:Goo00<007ooOol0>goo
00<007ooOol01Woo000YOol00`00Oomoo`0lOol00`00Oomoo`05Ool002Uoo`03001oogoo03aoo`03
001oogoo00Eoo`00:Goo00<007ooOol0?7oo00<007ooOol01Goo000YOol00`00Oomoo`0mOol00`00
Oomoo`04Ool002Uoo`03001oogoo03eoo`03001oogoo00Aoo`00:Goo00<007ooOol0?Goo00<007oo
Ool017oo000YOol00`00Oomoo`0mOol00`00Oomoo`04Ool002Uoo`03001oogoo03ioo`03001oogoo
00=oo`00:Goo00<007ooOol0?Woo00<007ooOol00goo000YOol00`00Oomoo`0nOol00`00Oomoo`03
Ool002Uoo`03001oogoo03moo`03001oogoo009oo`00:Goo00<007ooOol0?goo00<007ooOol00Woo
000MOol50002Ool00`00Oomoo`02Ool00`00Oomoo`10Ool00`00Oomoo`01Ool001moo`07001oogoo
Ool007oo00000goo00<007ooOol0@7oo00<007ooOol00Goo000OOol01`00Oomoogoo001oo`0000=o
o`03001oogoo041oo`03001oogoo005oo`007goo00L007ooOomoo`00Ool00003Ool00`00Oomoo`10
Ool00`00Oomoo`01Ool001moo`07001oogooOol007oo00000goo00<007ooOol0A7oo000NOol20004
Ool00`00Oomoo`02Ool00`00Oomoo`14Ool002Uoo`03001oogoo04Aoo`00:Goo00<007ooOol0A7oo
000YOol00`00Oomoo`14Ool002Uoo`03001oogoo04Aoo`00:Goo00<007ooOol0A7oo0000\
\>"],
  ImageRangeCache->{{{0, 111.438}, {286.938, 0}} -> {-3.04354, -10.6082, 
  0.0739406, 0.0739406}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell["\<\
We have to have the idea of a point on a curve. We will allow this as a \
Point[x,y] structure or as numbers. We might want to check a point is on a \
curve\
\>", "Text"],

Cell[BoxData[{
    \(\(PointOnCurve[EC[a_, b_], x1_, y1_] = 
      PointOnCurve[EC[a, b], P[x1, y1]]; \)\), 
    \(\[IndentingNewLine]\), 
    \(PointOnCurve[EC[a_, b_], P[x1_, y1_]] := 
      GetEquation[EC[a, b]] /. {x \[Rule] x1, y \[Rule] y1}\)}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(PointOnCurve[EC[\(-16\), 16], 0, 4]\)], "Input"],

Cell[BoxData[
    \(True\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(PointOnCurve[EC[\(-16\), 16], P[1, 4]]\)], "Input"],

Cell[BoxData[
    \(False\)], "Output"]
}, Open  ]],

Cell["\<\
What about getting a point on a curve. We might get a point by specifying an \
x and calculating the y value. But the problem is that each x will have two \
corresponding points. This will return all possible points.  Note that the \
Flatten is used so that Map can be applied correctly.\
\>", "Text"],

Cell[BoxData[
    \(GetPoints[ec_, x1_] := 
      Module[{ys}, 
        \[IndentingNewLine]ys = 
          Flatten[Solve[GetEquation[ec] /. x \[Rule] x1]]; 
        \[IndentingNewLine]Map[\((P[x1, y] /. #)\)&, ys]]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(GetPoints[EC[\(-16\), 16], 0]\)], "Input"],

Cell[BoxData[
    \({P[0, \(-4\)], P[0, 4]}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(GetPoints[c, 4]\)], "Input"],

Cell[BoxData[
    \({P[4, \(-\@46\)], P[4, \@46]}\)], "Output"]
}, Open  ]],

Cell["Now we define addition in the group.", "Text"],

Cell[BoxData[{
    \(\(Add[EC[a_, b_], x1_, y1_, x2_, y2_] := 
      Add[EC[a, b], P[x1, y1], P[x2, y2]]; \)\), 
    \(\[IndentingNewLine]\), 
    \(Add[EC[a_, b_], P[x1_, y1_], P[x2_, y2_]] := 
      \[IndentingNewLine]Module[{m, x3, y3}, 
        \[IndentingNewLine]If[x1 \[Equal] x2, 
          \[IndentingNewLine]m = \((3\ x1\^2 + a)\)/\((2\ y1)\), 
          \[IndentingNewLine]m = \((y2 - y1)\)/\((x2 - x1)\)]; 
        \[IndentingNewLine]x3 = m\^2 - x1 - x2; 
        \[IndentingNewLine]y3 = m \((x1 - x3)\) - y1; 
        \[IndentingNewLine]P[x3, y3]]\)}], "Input"],

Cell["Let's try some addition", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(curve = EC[\(-16\), 16]\)], "Input"],

Cell[BoxData[
    \(EC[\(-16\), 16]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(GetPoints[curve, 0]\)], "Input"],

Cell[BoxData[
    \({P[0, \(-4\)], P[0, 4]}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(p = %\[LeftDoubleBracket]2\[RightDoubleBracket]\)], "Input"],

Cell[BoxData[
    \(P[0, 4]\)], "Output"]
}, Open  ]],

Cell["Let's add p + p", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(p2 = Add[curve, p, p]\)], "Input"],

Cell[BoxData[
    \(P[4, 4]\)], "Output"]
}, Open  ]],

Cell["Which is correct. Now to get p3=p2+p", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(p3 = Add[curve, p, p2]\)], "Input"],

Cell[BoxData[
    \(P[\(-4\), \(-4\)]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(p4 = Add[curve, p, p3]\)], "Input"],

Cell[BoxData[
    \(P[8, \(-20\)]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(p5 = Add[curve, p, p4]\)], "Input"],

Cell[BoxData[
    \(P[1, \(-1\)]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(p6 = Add[curve, p, p5]\)], "Input"],

Cell[BoxData[
    \(P[24, 116]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(p7 = Add[curve, p2, p5]\)], "Input"],

Cell[BoxData[
    \(P[\(-\(20\/9\)\), 172\/27]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(p8 = Add[curve, p2, p6]\)], "Input"],

Cell[BoxData[
    \(P[84\/25, \(-\(52\/125\)\)]\)], "Output"]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"Microsoft Windows 3.0",
ScreenRectangle->{{0, 1024}, {0, 740}},
WindowSize->{667, 656},
WindowMargins->{{Automatic, 162}, {Automatic, -5}}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 32, 0, 105, "Title"],
Cell[1766, 53, 60, 1, 30, "Input"],
Cell[1829, 56, 375, 9, 128, "Text"],
Cell[2207, 67, 93, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[2325, 72, 60, 1, 30, "Input"],
Cell[2388, 75, 38, 1, 29, "Output"]
}, Open  ]],
Cell[2441, 79, 116, 3, 33, "Text"],
Cell[2560, 84, 89, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[2674, 89, 63, 1, 30, "Input"],
Cell[2740, 92, 61, 1, 29, "Output"]
}, Open  ]],
Cell[2816, 96, 61, 1, 30, "Input"],

Cell[CellGroupData[{
Cell[2902, 101, 56, 1, 30, "Input"],
Cell[2961, 104, 47, 1, 29, "Output"]
}, Open  ]],
Cell[3023, 108, 51, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[3099, 112, 139, 2, 50, "Input"],
Cell[3241, 116, 173, 3, 44, "Message"],
Cell[3417, 121, 173, 3, 44, "Message"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3627, 129, 56, 1, 30, "Input"],
Cell[3686, 132, 17317, 406, 296, 3386, 230, "GraphicsData", 
"PostScript", "Graphics"],
Cell[21006, 540, 130, 3, 29, "Output"]
}, Open  ]],
Cell[21151, 546, 180, 4, 52, "Text"],
Cell[21334, 552, 260, 5, 90, "Input"],

Cell[CellGroupData[{
Cell[21619, 561, 68, 1, 30, "Input"],
Cell[21690, 564, 38, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[21765, 570, 71, 1, 30, "Input"],
Cell[21839, 573, 39, 1, 29, "Output"]
}, Open  ]],
Cell[21893, 577, 311, 5, 71, "Text"],
Cell[22207, 584, 227, 5, 70, "Input"],

Cell[CellGroupData[{
Cell[22459, 593, 62, 1, 30, "Input"],
Cell[22524, 596, 57, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[22618, 602, 48, 1, 30, "Input"],
Cell[22669, 605, 63, 1, 32, "Output"]
}, Open  ]],
Cell[22747, 609, 52, 0, 33, "Text"],
Cell[22802, 611, 573, 11, 235, "Input"],
Cell[23378, 624, 39, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[23442, 628, 56, 1, 30, "Input"],
Cell[23501, 631, 49, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[23587, 637, 52, 1, 30, "Input"],
Cell[23642, 640, 57, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[23736, 646, 80, 1, 30, "Input"],
Cell[23819, 649, 41, 1, 29, "Output"]
}, Open  ]],
Cell[23875, 653, 31, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[23931, 657, 54, 1, 30, "Input"],
Cell[23988, 660, 41, 1, 29, "Output"]
}, Open  ]],
Cell[24044, 664, 52, 0, 33, "Text"],

Cell[CellGroupData[{
Cell[24121, 668, 55, 1, 30, "Input"],
Cell[24179, 671, 51, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[24267, 677, 55, 1, 30, "Input"],
Cell[24325, 680, 47, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[24409, 686, 55, 1, 30, "Input"],
Cell[24467, 689, 46, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[24550, 695, 55, 1, 30, "Input"],
Cell[24608, 698, 44, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[24689, 704, 56, 1, 30, "Input"],
Cell[24748, 707, 60, 1, 42, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[24845, 713, 56, 1, 30, "Input"],
Cell[24904, 716, 61, 1, 42, "Output"]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

