(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[      7075,        248]*)
(*NotebookOutlinePosition[      7726,        271]*)
(*  CellTagsIndexPosition[      7682,        267]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Index to Notebooks", "Title"],

Cell[TextData[{
  "The notebooks may be difficult to navigate so this will serve to help \
newcomers to understand what is going on. All the important functions are \
contained in the ",
  ButtonBox["00_funct.nb",
    ButtonData:>{"00_funct.nb", None},
    Active->True,
    ButtonStyle->"Hyperlink"],
  " notebook (note that by clicking on the underlined name, that notebook \
should be loaded), but with no comments. The idea is that one can use the \
Evaluate Notebook command in the Kernal menu to quickly initialise the entire \
notebook. (Creating packages turned out to be far too time consuming. I only \
completed a couple and have decided to leave them out.)"
}], "Text"],

Cell[CellGroupData[{

Cell["RSA Functions", "Subtitle"],

Cell[TextData[{
  "The basic functions to due with Key Generation, Encoding and Encryption \
can be found in the following notebooks:\n\t",
  ButtonBox["01_ranprime.nb",
    ButtonData:>{"01_ranpr.nb", None},
    Active->True,
    ButtonStyle->"Hyperlink"],
  " - For generating large random primes\n\t",
  ButtonBox["02_keygen.nb",
    ButtonData:>{"02_keyge.nb", None},
    Active->True,
    ButtonStyle->"Hyperlink"],
  " - Key generation\n\t",
  ButtonBox["03_keyman.nb",
    ButtonData:>{"03_keyma.nb", None},
    Active->True,
    ButtonStyle->"Hyperlink"],
  " - Key Management\n\t",
  ButtonBox["04_encoding.nb",
    ButtonData:>{"04_encod.nb", None},
    Active->True,
    ButtonStyle->"Hyperlink"],
  " - Character and Message Encoding\n\t",
  ButtonBox["05_encrypt.nb",
    ButtonData:>{"05_encry.nb", None},
    Active->True,
    ButtonStyle->"Hyperlink"],
  "  - Encryption and Decryption Functions"
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Martin Gardner", "Subtitle"],

Cell[TextData[{
  "The Martin Gardner challenge caused a great deal of interest in RSA when \
it was only new. Here is a description of that ",
  ButtonBox["challenge",
    ButtonData:>{"21_gardn.nb", None},
    Active->True,
    ButtonStyle->"Hyperlink"],
  ". "
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Factoring Functions", "Subtitle"],

Cell[TextData[{
  "My factoring functions are available in ",
  ButtonBox["07_factoring.nb",
    ButtonData:>{"07_facto.nb", None},
    Active->True,
    ButtonStyle->"Hyperlink"],
  ". It contains implementations of Trial Division, Fermat's algorithm, \
Pollard p-1 and rho algorithms and the Elliptic Curve method of Lenstra"
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Attacks", "Subtitle"],

Cell[TextData[{
  "There is a separate notebook for each attack. Code and examples of use are \
given.\n\t",
  ButtonBox["06_cycle.nb",
    ButtonData:>{"06_cycle.nb", None},
    ButtonStyle->"Hyperlink"],
  " - Cycle Attack\n\t",
  ButtonBox["08_faulty.nb",
    ButtonData:>{"08_fault.nb", None},
    Active->True,
    ButtonStyle->"Hyperlink"],
  " - Common Modulus and Faulty Encryption\n\t",
  ButtonBox["10_exhaustive.nb",
    ButtonData:>{"10_exhau.nb", None},
    Active->True,
    ButtonStyle->"Hyperlink"],
  "  - Exhaustive Search\n\t",
  ButtonBox["09_guessed.nb",
    ButtonData:>{"09_gues.nb", None},
    Active->True,
    ButtonStyle->"Hyperlink"],
  " - Guessed Decrypting Exponent\n\t",
  ButtonBox["11_lowexponent.nb",
    ButtonData:>{"11_lowex.nb", None},
    Active->True,
    ButtonStyle->"Hyperlink"],
  " - Low Encrypting Exponent\n\t",
  ButtonBox["13_lowsecretexponent.nb",
    ButtonData:>{"13_lowse.nb", None},
    Active->True,
    ButtonStyle->"Hyperlink"],
  " - Low Private Exponent\n\t",
  ButtonBox["12_adaptive.nb",
    ButtonData:>{"12_adapt.nb", None},
    Active->True,
    ButtonStyle->"Hyperlink"],
  "  - Adaptive Chosen Plaintext Attack"
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Student Code Problem", "Subtitle"],

Cell[TextData[{
  "The student code problem is described in ",
  ButtonBox["codes.nb",
    ButtonData:>{"codes.nb", None},
    ButtonStyle->"Hyperlink"],
  "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Testing Factoring Algorithms", "Subtitle"],

Cell[TextData[{
  "There are a few testing notebooks.\n",
  ButtonBox["FctrTest.nb",
    ButtonData:>{"fctrTest.nb", None},
    Active->True,
    ButtonStyle->"Hyperlink"],
  "   \n",
  ButtonBox["partitionChall.nb",
    ButtonData:>{"prtchall.nb", None},
    Active->True,
    ButtonStyle->"Hyperlink"],
  "  \n",
  ButtonBox["challenge.nb",
    ButtonData:>{"challeng.nb", None},
    Active->True,
    ButtonStyle->"Hyperlink"],
  " and the ",
  ButtonBox["results",
    ButtonData:>{"results.txt", None},
    ButtonStyle->"Hyperlink"],
  " that went with it"
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Storage of Primes", "Subtitle"],

Cell[TextData[{
  ButtonBox["storage.nb",
    ButtonData:>{"storage.nb", None},
    ButtonStyle->"Hyperlink"],
  " and ",
  ButtonBox["tables.nb",
    ButtonData:>{"tables.nb", None},
    ButtonStyle->"Hyperlink"],
  " contain some work I started on trying to store large numbers of primes. "
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Elliptic Curves", "Subtitle"],

Cell[TextData[{
  "Some of my early work on elliptic curves is in ",
  ButtonBox["curves.nb",
    ButtonData:>{"curves.nb", None},
    ButtonStyle->"Hyperlink"],
  ". I was trying to understand their nature."
}], "Text"]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"Microsoft Windows 3.0",
ScreenRectangle->{{0, 1024}, {0, 712}},
WindowSize->{473, 581},
WindowMargins->{{10, Automatic}, {Automatic, 15}}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 35, 0, 105, "Title"],
Cell[1769, 53, 681, 13, 147, "Text"],

Cell[CellGroupData[{
Cell[2475, 70, 33, 0, 64, "Subtitle"],
Cell[2511, 72, 924, 28, 147, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3472, 105, 34, 0, 64, "Subtitle"],
Cell[3509, 107, 275, 8, 52, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3821, 120, 39, 0, 64, "Subtitle"],
Cell[3863, 122, 339, 8, 71, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4239, 135, 27, 0, 64, "Subtitle"],
Cell[4269, 137, 1190, 37, 166, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5496, 179, 40, 0, 64, "Subtitle"],
Cell[5539, 181, 171, 6, 33, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5747, 192, 48, 0, 64, "Subtitle"],
Cell[5798, 194, 573, 21, 90, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6408, 220, 37, 0, 64, "Subtitle"],
Cell[6448, 222, 304, 9, 52, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6789, 236, 35, 0, 64, "Subtitle"],
Cell[6827, 238, 220, 6, 52, "Text"]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

