(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[      4707,        155]*)
(*NotebookOutlinePosition[      5358,        178]*)
(*  CellTagsIndexPosition[      5314,        174]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Tables of Primes", "Title"],

Cell["\<\
We will store as binary tables.  A bit set to 1 will indicate that the number \
represented by that slot is prime and a 0 that it is composite.  30 numbers \
will be represented by a single byte (8 bits).  \
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Quotient[1000000, 30*8]\)], "Input"],

Cell[BoxData[
    \(4166\)], "Output"]
}, Open  ]],

Cell["\<\
To table all primes up to 1 million will take approximately 4,000 bytes.\
\>", "Text"],

Cell["\<\
Only numbers that are not multiples of 2, 3, and 5 will have a posiible bit \
to set (these cannot be prime).  The reason groups of thirty numbers are \
chosen is because the pattern of positions repeats every 30.\
\>", "Text"],

Cell[BoxData[
    \(\({1, 7, 11, 13, 17, 19, 23, 29}; \)\)], "Input"],

Cell[BoxData[
    \(\({31, 37, 41, 43, 47, 49, 53, 59}; \)\)], "Input"],

Cell[BoxData[
    \(\({61, 67, 71, 73, 77, 79, 83, 89}; \)\)], "Input"],

Cell["\<\
The numbers shown above are the only numbers less than 90 and apart from 2, 3 \
and 5, that can be prime.  There is no need to include any of the other \
numbers because they are composites with 2, 3, or 5 as factors.  Look at the \
table of primes for this range to see if you think I am wrong\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[Prime[i], {i, 1, 24}]\)], "Input"],

Cell[BoxData[
    \({2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37, 41, 43, 47, 53, 59, 61, 
      67, 71, 73, 79, 83, 89}\)], "Output"]
}, Open  ]],

Cell["\<\
Not all the numbers in my lists were prime, but some were.  We could show \
these lists in the following manner.\
\>", "Text"],

Cell[BoxData[
    \(\({0, 1, 1, 1, 1, 1, 1, 1}; \)\)], "Input"],

Cell[BoxData[
    \(\({1, 1, 1, 1, 1, 0, 1, 1}; \)\)], "Input"],

Cell[BoxData[
    \(\({1, 1, 1, 1, 0, 1, 1, 1}; \)\)], "Input"],

Cell["\<\
As should be obvious there is very little overhead, only 1 in 8 is not a \
prime so far.\
\>", "Text"],

Cell["\<\
The next step should be to construct a suitable table containing the \
information.  This should only be done once - afterwards we must build \
testing functions that can lookup the tables.\
\>", "Text"],

Cell[CellGroupData[{

Cell["Bulding the first table of primes", "Subtitle"],

Cell[BoxData[
    \(BuildTable[n_] := 
      Module[{l, incs, ndx1, ndx2}, \n\t\tl = {0, 1, 1, 1, 1, 1, 1, 1}; \n\t\t
        incs = {6, 4, 2, 4, 2, 4, 6, 2}; \n\t\tndx1 = 1; \n\t\tndx2 = 31; \n
        \t\tWhile[ndx2 <= n, \n\t\t\t
          If[PrimeQ[ndx2], \n\t\t\t\tl = Join[l, {1}], \n\t\t\t\t
            l = Join[l, {0}]]; \n\t\t\t
          ndx2 = ndx2 + incs\[LeftDoubleBracket]ndx1\[RightDoubleBracket]; \n
          \t\t\tndx1 = ndx1 + 1; \n\t\t\tIf[ndx1 == 9, ndx1 = 1]]; \n\t
        Map[FromDigits[#, 2]&, Partition[l, 8]]]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(BuildTable[90]\)], "Input"],

Cell[BoxData[
    \({127, 251, 247}\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"Microsoft Windows 3.0",
ScreenRectangle->{{0, 1024}, {0, 740}},
WindowSize->{491, 622},
WindowMargins->{{60, Automatic}, {16, Automatic}}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 33, 0, 105, "Title"],
Cell[1767, 53, 230, 4, 71, "Text"],

Cell[CellGroupData[{
Cell[2022, 61, 56, 1, 30, "Input"],
Cell[2081, 64, 38, 1, 29, "Output"]
}, Open  ]],
Cell[2134, 68, 96, 2, 33, "Text"],
Cell[2233, 72, 237, 4, 71, "Text"],
Cell[2473, 78, 69, 1, 30, "Input"],
Cell[2545, 81, 71, 1, 30, "Input"],
Cell[2619, 84, 71, 1, 30, "Input"],
Cell[2693, 87, 318, 5, 90, "Text"],

Cell[CellGroupData[{
Cell[3036, 96, 60, 1, 30, "Input"],
Cell[3099, 99, 133, 2, 48, "Output"]
}, Open  ]],
Cell[3247, 104, 136, 3, 52, "Text"],
Cell[3386, 109, 63, 1, 30, "Input"],
Cell[3452, 112, 63, 1, 30, "Input"],
Cell[3518, 115, 63, 1, 30, "Input"],
Cell[3584, 118, 112, 3, 33, "Text"],
Cell[3699, 123, 213, 4, 52, "Text"],

Cell[CellGroupData[{
Cell[3937, 131, 53, 0, 64, "Subtitle"],
Cell[3993, 133, 550, 9, 270, "Input"],

Cell[CellGroupData[{
Cell[4568, 146, 47, 1, 30, "Input"],
Cell[4618, 149, 49, 1, 29, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

